#include "PathSelector.h"
#include "ui_PathSelector.h"

void PathSelector::showEvent(QShowEvent*)
{
    ui->pathView->verticalScrollBar()->setValue(0);
}

void PathSelector::keyPressEvent(QKeyEvent* event)
{
    if(event->key() == Qt::Key_Escape)
        emit cancel();
    else
        event->ignore();
}

void PathSelector::setImageWidth(int width)
{
    imageWidth = width;
}

void PathSelector::setImageHeight(int height)
{
    imageHeight = height;
}

void PathSelector::cancelClicked()
{
    emit cancel();
}

void PathSelector::pathClicked(Paths::Type type)
{
    if(type == Paths::DIAMOND)
    {
        QPainterPath diamondPath;
        QVector<QPointF> diamondPoints;
        diamondPoints << QPointF(0.0f,static_cast<float>(imageHeight/4))
                << QPointF(static_cast<float>(imageWidth/4),0.0f)
                << QPointF(static_cast<float>(imageWidth - (imageWidth/4)),0.0f)
                << QPointF(static_cast<float>(imageWidth),static_cast<float>(imageHeight/4))
                << QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight))
                << QPointF(0.0f,static_cast<float>(imageHeight/4));
        QPolygonF diamondPoligon(diamondPoints);
        diamondPath.addPolygon(diamondPoligon);
        QList<QPainterPath> paths;
        paths << diamondPath;
        emit pathSelected(paths);
    }
    else if(type == Paths::RECT)
    {
        QPainterPath rectPath;
        QVector<QPointF> rectPoints;
        rectPoints << QPointF(static_cast<float>(imageWidth/2),0.0f)
                << QPointF(static_cast<float>(imageWidth),static_cast<float>(imageHeight/2))
                << QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight))
                << QPointF(0.0f,static_cast<float>(imageHeight/2))
                << QPointF(static_cast<float>(imageWidth/2),0.0f);
        QPolygonF rectPoligon(rectPoints);
        rectPath.addPolygon(rectPoligon);
        QList<QPainterPath> paths;
        paths << rectPath;
        emit pathSelected(paths);
    }
    else if(type == Paths::RECTS)
    {
        QPainterPath rectPath;
        QVector<QPointF> rectPoints;
        rectPoints << QPointF(static_cast<float>(imageWidth/2),0.0f)
                << QPointF(static_cast<float>(imageWidth),static_cast<float>(imageHeight/2))
                << QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight))
                << QPointF(0.0f,static_cast<float>(imageHeight/2))
                << QPointF(static_cast<float>(imageWidth/2),0.0f);
        QPolygonF rectPoligon(rectPoints);
        rectPath.addPolygon(rectPoligon);
        QPainterPath rectPath2;
        QVector<QPointF> rectPoints2;
        rectPoints2 << QPointF(static_cast<float>(imageWidth* 0.15f),static_cast<float>(imageHeight * 0.15f))
                << QPointF(static_cast<float>(imageWidth* 0.85f),static_cast<float>(imageHeight * 0.15f))
                << QPointF(static_cast<float>(imageWidth* 0.85f),static_cast<float>(imageHeight * 0.85f))
                << QPointF(static_cast<float>(imageWidth* 0.15f),static_cast<float>(imageHeight * 0.85f))
                << QPointF(static_cast<float>(imageWidth* 0.15f),static_cast<float>(imageHeight * 0.15f));
        QPolygonF rectPoligon2(rectPoints2);
        rectPath2.addPolygon(rectPoligon2);
        QList<QPainterPath> paths;
        paths << rectPath;
        paths << rectPath2;
        emit pathSelected(paths);
    }
    else if(type == Paths::STAR)
    {
        QPainterPath starPath;
        QVector<QPointF> starPoints;
        starPoints << QPointF(0.0f,0.0f)
                << QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight/4))
                << QPointF(static_cast<float>(imageWidth),0.0f)
                << QPointF(static_cast<float>(imageWidth - (imageWidth/4)),static_cast<float>(imageHeight/2))
                << QPointF(static_cast<float>(imageWidth),static_cast<float>(imageHeight))
                << QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight - (imageHeight/4)))
                << QPointF(0.0f,static_cast<float>(imageHeight))
                << QPointF(static_cast<float>(imageWidth/4),static_cast<float>(imageHeight/2))
                << QPointF(0.0f,0.0f);
        QPolygonF starPoligon(starPoints);
        starPath.addPolygon(starPoligon);
        QList<QPainterPath> paths;
        paths << starPath;
        emit pathSelected(paths);
    }
    else if(type == Paths::CROSS)
    {
        QPainterPath crossPath;
        QVector<QPointF> crossPoints;
        crossPoints << QPointF(static_cast<float>(imageWidth/2),0.0f)
                << QPointF(static_cast<float>(imageWidth * 0.6f),static_cast<float>(imageHeight * 0.4f))
                << QPointF(static_cast<float>(imageWidth),static_cast<float>(imageHeight/2))
                << QPointF(static_cast<float>(imageWidth * 0.6f),static_cast<float>(imageHeight * 0.6f))
                << QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight))
                << QPointF(static_cast<float>(imageWidth * 0.4f),static_cast<float>(imageHeight * 0.6f))
                << QPointF(0.0f,static_cast<float>(imageHeight/2))
                << QPointF(static_cast<float>(imageWidth * 0.4f),static_cast<float>(imageHeight * 0.4f))
                << QPointF(static_cast<float>(imageWidth/2),0.0f) ;
        QPolygonF crossPoligon(crossPoints);
        crossPath.addPolygon(crossPoligon);
        QList<QPainterPath> paths;
        paths << crossPath;
        emit pathSelected(paths);
    }
    else if(type == Paths::ASTERIX)
    {
        QPainterPath crossPath;
        QVector<QPointF> crossPoints;
        crossPoints << QPointF(static_cast<float>(imageWidth/2),0.0f)
                << QPointF(static_cast<float>(imageWidth * 0.6f),static_cast<float>(imageHeight * 0.4f))
                << QPointF(static_cast<float>(imageWidth),static_cast<float>(imageHeight/2))
                << QPointF(static_cast<float>(imageWidth * 0.6f),static_cast<float>(imageHeight * 0.6f))
                << QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight))
                << QPointF(static_cast<float>(imageWidth * 0.4f),static_cast<float>(imageHeight * 0.6f))
                << QPointF(0.0f,static_cast<float>(imageHeight/2))
                << QPointF(static_cast<float>(imageWidth * 0.4f),static_cast<float>(imageHeight * 0.4f))
                << QPointF(static_cast<float>(imageWidth/2),0.0f) ;
        QPolygonF crossPoligon(crossPoints);
        crossPath.addPolygon(crossPoligon);
        QPainterPath starPath2;
        QVector<QPointF> starPoints2;
        starPoints2 << QPointF(0.0f,0.0f)
                << QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight * 0.4f))
                << QPointF(static_cast<float>(imageWidth),0.0f)
                << QPointF(static_cast<float>(imageWidth * 0.6f),static_cast<float>(imageHeight/2))
                << QPointF(static_cast<float>(imageWidth),static_cast<float>(imageHeight))
                << QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight * 0.6f))
                << QPointF(0.0f,static_cast<float>(imageHeight))
                << QPointF(static_cast<float>(imageWidth * 0.4f),static_cast<float>(imageHeight/2))
                << QPointF(0.0f,0.0f);
        QPolygonF starPoligon2(starPoints2);
        starPath2.addPolygon(starPoligon2);
        QList<QPainterPath> paths;
        paths << starPath2;
        paths << crossPath;
        emit pathSelected(paths);
    }
    else if(type == Paths::ELLIPSE)
    {
        QPainterPath ellipsePath;
        ellipsePath.addEllipse(QPoint(imageWidth/2,imageHeight/2),
                               imageWidth/2,imageHeight/2);
        QList<QPainterPath> paths;
        paths << ellipsePath;
        emit pathSelected(paths);
    }
    else if(type == Paths::HART)
    {
        QPainterPath hartPath;
        hartPath.moveTo(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight));
        hartPath.cubicTo(QPointF(static_cast<float>(imageWidth * 1.25f),static_cast<float>(imageHeight/2)),
                         QPointF(static_cast<float>(imageWidth),static_cast<float>(imageHeight * (-0.35f))),
                         QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight/4)));
        hartPath.moveTo(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight));
        hartPath.cubicTo(QPointF(static_cast<float>(imageWidth * (-0.25f)),static_cast<float>(imageHeight/2)),
                         QPointF(0.0f,static_cast<float>(imageHeight * (-0.35f))),
                         QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight/4)));
        QList<QPainterPath> hartPaths;
        hartPaths << hartPath;
        emit pathSelected(hartPaths);
    }
    else if(type == Paths::GINGERBREAD)
    {
        QPainterPath gingerbreadPath;
        gingerbreadPath.addEllipse(QPointF(static_cast<float>(imageWidth/4),static_cast<float>(imageHeight/4))
                                   ,static_cast<float>(imageWidth/4),static_cast<float>(imageHeight/4));
        gingerbreadPath.addEllipse(QPointF(static_cast<float>(imageWidth * 0.75f), static_cast<float>(imageHeight/4))
                                   ,static_cast<float>(imageWidth/4),static_cast<float>(imageHeight/4));
        gingerbreadPath.addEllipse(QPointF(static_cast<float>(imageWidth/4),static_cast<float>(imageHeight * 0.75f)),static_cast<float>(imageWidth/4),
                                   static_cast<float>(imageHeight/4));
        gingerbreadPath.addEllipse(QPointF(static_cast<float>(imageWidth * 0.75f),static_cast<float>(imageHeight * 0.75f)),
                                   static_cast<float>(imageWidth/4),static_cast<float>(imageHeight/4));
        QPainterPath gingerbreadPath2;
        gingerbreadPath2.addEllipse(QPointF(static_cast<float>(imageWidth/2),static_cast<float>(imageHeight/2)),
                                    static_cast<float>(imageWidth/4),static_cast<float>(imageHeight/4));
        QList<QPainterPath> gingerbreadPaths;
        gingerbreadPaths << gingerbreadPath;
        gingerbreadPaths << gingerbreadPath2;
        emit pathSelected(gingerbreadPaths);
    }
}

void PathSelector::makeScene()
{
    QGraphicsScene* scene = new QGraphicsScene(0,0,400,350);
    scene->setBackgroundBrush(qApp->palette().background());
    ui->pathView->setScene(scene);
    
    //Diamond
    QPainterPath diamondPath;
    QVector<QPointF> diamondPoints;
    diamondPoints << QPointF(5.0f,25.0f) << QPointF(25.0f,5.0f)
            << QPointF(75.0f,5.0f) << QPointF(95.0f,25.0f)
            << QPointF(50.0f,95.0f) << QPointF(5.0f,25.0f);
    QPolygonF diamondPoligon(diamondPoints);
    diamondPath.addPolygon(diamondPoligon);
    QList<QPainterPath> diamondPaths;
    diamondPaths << diamondPath;
    PathItem* diamond = new PathItem(Paths::DIAMOND,diamondPaths);
    diamond->setPos(25,10);
    scene->addItem(diamond);
    connect(diamond,SIGNAL(clicked(Paths::Type)),
            this,SLOT(pathClicked(Paths::Type)));
    
    //Rect
    QPainterPath rectPath;
    QVector<QPointF> rectPoints;
    rectPoints << QPointF(50.0f,5.0f) << QPointF(95.0f,50.0f)
            << QPointF(50.0f,95.0f) << QPointF(5.0f,50.0f)
            << QPointF(50.0f,5.0f);
    QPolygonF rectPoligon(rectPoints);
    rectPath.addPolygon(rectPoligon);
    QList<QPainterPath> rectPaths;
    rectPaths << rectPath;
    PathItem* rect = new PathItem(Paths::RECT,rectPaths);
    rect->setPos(150,10);
    scene->addItem(rect);
    connect(rect,SIGNAL(clicked(Paths::Type)),
            this,SLOT(pathClicked(Paths::Type)));
    
    //Rects
    QPainterPath rectsPath;
    QVector<QPointF> rectsPoints;
    rectsPoints << QPointF(50.0f,5.0f) << QPointF(95.0f,50.0f)
            << QPointF(50.0f,95.0f) << QPointF(5.0f,50.0f)
            << QPointF(50.0f,5.0f);
    QPolygonF rectsPoligon(rectsPoints);
    rectsPath.addPolygon(rectsPoligon);
    QPainterPath rectsPath2;
    QVector<QPointF> rectsPoints2;
    rectsPoints2 << QPointF(20.0f,20.0f) << QPointF(80.0f,20.0f)
            << QPointF(80.0f,80.0f) << QPointF(20.0f,80.0f)
            << QPointF(20.0f,20.0f);
    QPolygonF rectsPoligon2(rectsPoints2);
    rectsPath2.addPolygon(rectsPoligon2);
    QList<QPainterPath> rectsPaths;
    rectsPaths << rectsPath;
    rectsPaths << rectsPath2;
    PathItem* rects = new PathItem(Paths::RECTS,rectsPaths);
    rects->setPos(275,10);
    scene->addItem(rects);
    connect(rects,SIGNAL(clicked(Paths::Type)),
            this,SLOT(pathClicked(Paths::Type)));
    
    //Star
    QPainterPath starPath;
    QVector<QPointF> starPoints;
    starPoints << QPointF(5.0f,5.0f) << QPointF(50.0f,25.0f)
            << QPointF(95.0f,5.0f) << QPointF(75.0f,50.0f)
            << QPointF(95.0f,95.0f) << QPointF(50.0f,75.0f)
            << QPointF(5.0f,95.0f) << QPointF(25.0f,50.0f)
            << QPointF(5.0f,5.0f);
    QPolygonF starPoligon(starPoints);
    starPath.addPolygon(starPoligon);
    QList<QPainterPath> starPaths;
    starPaths << starPath;
    PathItem* start = new PathItem(Paths::STAR,starPaths);
    start->setPos(25,125);
    scene->addItem(start);
    connect(start,SIGNAL(clicked(Paths::Type)),
            this,SLOT(pathClicked(Paths::Type)));
    
    //Cross
    QPainterPath crossPath;
    QVector<QPointF> crossPoints;
    crossPoints << QPointF(50.0f,5.0f) << QPointF(60.0f,40.0f)
            << QPointF(95.0f,50.0f) << QPointF(60.0f,60.0f)
            << QPointF(50.0f,95.0f) << QPointF(40.0f,60.0f)
            << QPointF(5.0f,50.0f) << QPointF(40.0f,40.0f)
            << QPointF(50.0f,5.0f);
    QPolygonF crossPoligon(crossPoints);
    crossPath.addPolygon(crossPoligon);
    QList<QPainterPath> crossPaths;
    crossPaths << crossPath;
    PathItem* cross = new PathItem(Paths::CROSS,crossPaths);
    cross->setPos(150,125);
    scene->addItem(cross);
    connect(cross,SIGNAL(clicked(Paths::Type)),
            this,SLOT(pathClicked(Paths::Type)));
    
    //Asterix
    QPainterPath starPath2;
    QVector<QPointF> starPoints2;
    starPoints2 << QPointF(5.0f,5.0f) << QPointF(50.0f,40.0f)
            << QPointF(95.0f,5.0f) << QPointF(60.0f,50.0f)
            << QPointF(95.0f,95.0f) << QPointF(50.0f,60.0f)
            << QPointF(5.0f,95.0f) << QPointF(40.0f,50.0f)
            << QPointF(5.0f,5.0f);
    QPolygonF starPoligon2(starPoints2);
    starPath2.addPolygon(starPoligon2);
    QList<QPainterPath> asterixPaths;
    asterixPaths << starPath2;
    asterixPaths << crossPath;
    PathItem* asterix = new PathItem(Paths::ASTERIX,asterixPaths);
    asterix->setPos(275,125);
    scene->addItem(asterix);
    connect(asterix,SIGNAL(clicked(Paths::Type)),
            this,SLOT(pathClicked(Paths::Type)));
    
    //Ellipse
    QPainterPath ellipsePath;
    ellipsePath.addEllipse(QPointF(50.0f,50.0f),45.0f,45.0f);
    QList<QPainterPath> paths;
    paths << ellipsePath;
    PathItem* ellipse = new PathItem(Paths::ELLIPSE,paths);
    ellipse->setPos(25,250);
    scene->addItem(ellipse);
    connect(ellipse,SIGNAL(clicked(Paths::Type)),
            this,SLOT(pathClicked(Paths::Type)));
    
    //Hart
    QPainterPath hartPath;
    hartPath.moveTo(50.0f,95.0f);
    hartPath.cubicTo(QPointF(125.0f,50.0f),QPointF(100.0f,-35.0f),
                     QPointF(50.0f,25.0f));
    hartPath.moveTo(50.0f,95.0f);
    hartPath.cubicTo(QPointF(-25.0f,50.0f),QPointF(0.0f,-35.0f),
                     QPointF(50.0f,25.0f));
    QList<QPainterPath> hartPaths;
    hartPaths << hartPath;
    PathItem* hart = new PathItem(Paths::HART,hartPaths);
    hart->setPos(150,250);
    scene->addItem(hart);
    connect(hart,SIGNAL(clicked(Paths::Type)),
            this,SLOT(pathClicked(Paths::Type)));
    
    //Gingerbread
    QPainterPath gingerbreadPath;
    gingerbreadPath.addEllipse(QPointF(25.0f,25.0f),25.0f,25.0f);
    gingerbreadPath.addEllipse(QPointF(75.0f,25.0f),25.0f,25.0f);
    gingerbreadPath.addEllipse(QPointF(25.0f,75.0f),25.0f,25.0f);
    gingerbreadPath.addEllipse(QPointF(75.0f,75.0f),25.0f,25.0f);
    QPainterPath gingerbreadPath2;
    gingerbreadPath2.addEllipse(QPointF(50.0f,50.0f),25.0f,25.0f);
    QList<QPainterPath> gingerbreadPaths;
    gingerbreadPaths << gingerbreadPath;
    gingerbreadPaths << gingerbreadPath2;
    PathItem* gingerbread = new PathItem(Paths::GINGERBREAD,gingerbreadPaths);
    gingerbread->setPos(275,250);
    scene->addItem(gingerbread);
    connect(gingerbread,SIGNAL(clicked(Paths::Type)),
            this,SLOT(pathClicked(Paths::Type)));
    
}

PathSelector::PathSelector(QWidget *parent)
    :QDialog(parent),ui(new Ui::PathSelector)
{
    imageWidth = 0;
    imageHeight = 0;
    ui->setupUi(this);
    this->setModal(true);
    this->setWindowFlags(Qt::FramelessWindowHint|Qt::Dialog);
    connect(ui->cancelButton,SIGNAL(clicked()),this,SLOT(cancelClicked()));
    makeScene();
}

PathSelector::~PathSelector()
{
    delete ui;
}

