#include "colectionManipulator.h"

QImage ColectionManipulator::getPhotoMini(int id)
{
    QSqlQuery miniQuery(QSqlDatabase::database("MINIS"));
    miniQuery.prepare("SELECT ImageM FROM Mini where Id=:id");
    miniQuery.bindValue(":id",id);
    if(!miniQuery.exec())
    {
        qFatal("Failed to get image");
    }
    if(!miniQuery.next())
    {
        qDebug(QObject::tr("%1").arg(id).toAscii());
    }
    QByteArray array;
    array = miniQuery.value(0).toByteArray();
    QString format = getPhotoFormat(id);
    QBuffer buffer(&array);
    buffer.open(QIODevice::ReadOnly);
    QImageReader reader(&buffer,format.toAscii());
    QImage photoMini = reader.read();
    return photoMini;
}

QImage ColectionManipulator::getPhoto(int id)
{
    QSqlQuery qry(QSqlDatabase::database("PHOTOS"));
    qry.prepare("SELECT Image FROM Photo where Id=:id");
    qry.bindValue(":id",id);
    if(!qry.exec())
    {
        qFatal("Failed to get image");
    }
    if(!qry.next())
    {
        qFatal(QObject::tr("%1").arg(id).toAscii());
    }
    QByteArray array;
    array = qry.value(0).toByteArray();
    QString format = getPhotoFormat(id);
    QBuffer buffer(&array);
    buffer.open(QIODevice::ReadOnly);
    QImageReader reader(&buffer,format.toAscii());
    QImage photo = reader.read();
    return photo;
}

QString ColectionManipulator::getPhotoFormat(int id)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT Format FROM Info where Id=:id");
    qry.bindValue(":id",id);
    if(!qry.exec())
    {
        qFatal("Failed to get image");
    }
    if(!qry.next())
    {
        qFatal(QObject::tr("%1").arg(id).toAscii());
    }
    QString format;
    format = qry.value(0).toString();
    return format;
}

QString ColectionManipulator::getPhotoName(int id)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    QString name;
    qry.prepare("SELECT Name FROM IndexTable WHERE Id=:id");
    qry.bindValue(":id",id);
    if(!qry.exec())
    {
        qFatal("Failed to retrive Image Name.");
        return name;
    }
    while(qry.next())
    {
        name = qry.value(0).toString();
    }
    return name;
}

QStringList ColectionManipulator::getMd5Sums()
{
    QStringList sums;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT MD5SUM FROM Info");
    if(!qry.exec())
    {
        qFatal("Failed to retrive Sums.");
    }
    while(qry.next())
    {
        sums<<qry.value(0).toString();
    }
    return sums;
}

QStringList ColectionManipulator::getAuthors()
{
    QStringList authors;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT DISTINCT Autor FROM Info");
    if(!qry.exec())
    {
        qFatal("Failed to retrive Authors.");
    }
    while(qry.next())
    {
        authors<<qry.value(0).toString();
    }
    return authors;
}

QStringList ColectionManipulator::getTags()
{
    QStringList tags;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT DISTINCT Tag FROM TagTable");
    if(!qry.exec())
        qFatal("Failed to retrive Tags.");
    while(qry.next())
        tags.append(qry.value(0).toString());
    return tags;
}

QStringList ColectionManipulator::getPhotoInfo(int id)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    QStringList list;
    qry.prepare("SELECT Name, Catalog FROM IndexTable WHERE Id=:id");
    qry.bindValue(":id",id);
    if(!qry.exec())
    {
        qFatal("Failed to retrive Image Name and Catalog.");
        return list;
    }
    while(qry.next())
    {
        list<<qry.value(0).toString();
        list<<qry.value(1).toString();
    }

    qry.prepare("SELECT Autor, Disc, InsDate FROM Info WHERE Id=:id");
    qry.bindValue(":id",id);
    if(!qry.exec())
    {
        qFatal("Failed to retrive Author, Description and Date.");
        return list;
    }
    while(qry.next())
    {
        list<<qry.value(0).toString();
        list<<qry.value(1).toString();
        list<<qry.value(2).toString();
    }
    QString tags = "";
    qry.prepare("SELECT Tag FROM TagTable WHERE Id=:id");
    qry.bindValue(":id",id);
    if(!qry.exec())
    {
        qFatal("Failed to retrive Tags.");
        return list;
    }
    while(qry.next())
        tags += QString("%1;").arg(qry.value(0).toString());
    list<<tags;
    return list;
}

QList<int> ColectionManipulator::getPhotosFromFolder(QString folderName)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    QList<int> list;
    qry.prepare("SELECT Id FROM IndexTable WHERE Catalog=:cat");
    qry.bindValue(":cat",folderName);
    if(!qry.exec())
    {
        qFatal("Failed to retrive index.");
        return list;
    }
    while(qry.next())
        list<<qry.value(0).toInt();
    return list;
}

QStringList ColectionManipulator::getFolders()
{
    QStringList folders;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT DISTINCT Catalog FROM IndexTable");
    if(!qry.exec())
    {
        qFatal("Failed to retrive Catalogs.");
        return folders;
    }
    while(qry.next())
        folders<<qry.value(0).toString();
    return folders;
}

QString ColectionManipulator::getCatalog(int index)
{
    QString folder;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT Catalog FROM IndexTable WHERE Id=:id");
    qry.bindValue(":id",index);
    if(!qry.exec())
    {
        qFatal("Failed to retrive Catalogs.");
        return folder;
    }
    while(qry.next())
        folder = qry.value(0).toString();
    return folder;
}

int ColectionManipulator::getFolderContentCount(QString folderName)
{
    int contents = 0;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT count(*) FROM IndexTable where Catalog=:cat GROUP BY Catalog");
    qry.bindValue(":cat",folderName);
    if(!qry.exec())
    {
        qFatal("Failed to retrive data.");
        return contents;
    }
    while(qry.next())
    {
        contents = qry.value(0).toInt();
    }
    return contents;
}

int ColectionManipulator::getTagImageCount(QString tag)
{
    int contents = 0;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT count(*) FROM TagTable where Tag=:ta GROUP BY Tag");
    qry.bindValue(":ta",tag);
    if(!qry.exec())
    {
        qFatal("Failed to retrive data.");
        return contents;
    }
    while(qry.next())
    {
        contents = qry.value(0).toInt();
    }
    return contents;
}

QList<int> ColectionManipulator::searchForPhotos(QString name,QString author)
{
    QList<int> resault;
    if(name != "")
    {
        if(author != "")
        {
            QSqlQuery qry(QSqlDatabase::database("INDEX"));
            qry.prepare("SELECT i.Id FROM IndexTable i, Info n "
                        "where i.Id = n.Id and i.Name like :na "
                        "and n.Autor = :au");
            qry.bindValue(":na","%"+name+"%");
            qry.bindValue(":au",author);
            if(!qry.exec())
            {
                qFatal("Search Fail.");
            }
            while(qry.next())
            {
                resault<<qry.value(0).toInt();
            }
        }
        else
        {
            QSqlQuery qry(QSqlDatabase::database("INDEX"));
            qry.prepare("SELECT i.Id FROM IndexTable i "
                        "where i.Name like :na");
            qry.bindValue(":na","%"+name+"%");
            if(!qry.exec())
            {
                qFatal("Search Fail.");
            }
            while(qry.next())
            {
                resault<<qry.value(0).toInt();
            }
        }
    }
    else
    {
        if(author != "")
        {
            QSqlQuery qry(QSqlDatabase::database("INDEX"));
            qry.prepare("SELECT n.Id FROM Info n "
                        "where n.Autor = :au");
            qry.bindValue(":au",author);
            if(!qry.exec())
            {
                qFatal("Search Fail.");
            }
            while(qry.next())
            {
                resault<<qry.value(0).toInt();
            }
        }
        else
        {
            QList<int> empty;
            return empty;
        }
    }
    return resault;
}

QList<int> ColectionManipulator::searchForPhotos(QString name,QString author,QDate from, QDate to)
{
    QList<int> resault;
    if(name != "")
    {
        if(author != "")
        {
            QSqlQuery qry(QSqlDatabase::database("INDEX"));
            qry.prepare("select i.Id, i.Name from IndexTable i "
                        "where i.Name like :na "
                        "and i.Id in (select n.id from Info n "
                        "where n.Autor = :au "
                        "and n.InsDate between :from and :to)"
                        );            
            qry.bindValue(":na","%"+name+"%");
            qry.bindValue(":au",author);
            qry.bindValue(":from",from.toString("yyyy-MM-dd"));
            qry.bindValue(":to",to.toString("yyyy-MM-dd"));
            if(!qry.exec())
            {
                qFatal("Search Fail.");
            }
            while(qry.next())
            {
                resault<<qry.value(0).toInt();
            }
        }
        else
        {
            QSqlQuery qry(QSqlDatabase::database("INDEX"));
            qry.prepare("select i.id, i.name from IndexTable i "
                        "where i.name like :na "
                        "and i.id in (select n.id from info n "
                        "where n.InsDate between :from and :to)"
                        );
            qry.bindValue(":na","%"+name+"%");
            qry.bindValue(":from",from.toString("yyyy-MM-dd"));
            qry.bindValue(":to",to.toString("yyyy-MM-dd"));
            if(!qry.exec())
            {
                qFatal("Search Fail.");
            }
            while(qry.next())
            {
                resault<<qry.value(0).toInt();
            }
        }
    }
    else
    {
        if(author != "")
        {
            QSqlQuery qry(QSqlDatabase::database("INDEX"));
            qry.prepare("select n.id from info n "
                        "where n.autor = :au "
                        "and n.InsDate between :from and :to"
                        );
            qry.bindValue(":au",author);
            qry.bindValue(":from",from.toString("yyyy-MM-dd"));
            qry.bindValue(":to",to.toString("yyyy-MM-dd"));
            if(!qry.exec())
            {
                qFatal("Search Fail.");
            }
            while(qry.next())
            {
                resault<<qry.value(0).toInt();
            }

        }
        else
        {
            QSqlQuery qry(QSqlDatabase::database("INDEX"));
            qry.prepare("select n.id from info n "
                        "where n.InsDate between :from and :to"
                        );
            qry.bindValue(":from",from.toString("yyyy-MM-dd"));
            qry.bindValue(":to",to.toString("yyyy-MM-dd"));
            if(!qry.exec())
            {
                qFatal("Search Fail.");
            }
            while(qry.next())
            {
                resault<<qry.value(0).toInt();
            }
        }
    }
    return resault;
}

QList<int> ColectionManipulator::searchForPhotosByTag(QString tag)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("select Id from TagTable where Tag=:tag");
    qry.bindValue(":tag",tag.trimmed().toUpper());
    if(!qry.exec())
        qFatal("Tag Search Fail.");
    QList<int> resault;
    while(qry.next())
        resault<<qry.value(0).toInt();
    return resault;
}

bool ColectionManipulator::hasImage(QString md5sum)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT COUNT(*) FROM Info WHERE MD5SUM=:md");
    qry.bindValue(":md",md5sum);
    if(!qry.exec())
    {
        qFatal("Failed to retrive data.");
    }
    bool resault = false;
    while(qry.next())
    {
        int temp;
        temp=qry.value(0).toInt();
        if(temp == 0)
            resault = true;
        else resault = false;
    }
    return resault;
} 

int ColectionManipulator::addImage(QImage image,ImageData& data,int quality)
{
    int temp = 0;
    QString Katalog = data.catalog;
    QString Nazwa = data.name;
    QString Autor = data.author;
    QString Opis = data.desc;
    QString Format = data.format;
    QString MD5Sum = data.hash;
    
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT Id FROM IndexTable Order by Id");
    if(!qry.exec())
    {
        qFatal("Failed to count records");
    }
    while(qry.next())
    {
        int index = qry.value(0).toInt();
        if(temp < index)
            break;
        else
            ++temp;
    }
    QSqlQuery qryInsert(QSqlDatabase::database("INDEX"));
    qryInsert.prepare("INSERT INTO IndexTable (Id,Catalog,Name) VALUES(:id, :kat, :na)");
    qryInsert.bindValue(":id", temp);
    qryInsert.bindValue(":kat", Katalog);
    qryInsert.bindValue(":na", Nazwa);
    if(!qryInsert.exec())
    {
        qFatal("Failed to insert into Indeks");
    }
    qryInsert.prepare("INSERT INTO Info (Id, Autor, Disc, InsDate, Format, MD5SUM) "
                      "VALUES(:id, :au, :op, :da, :fo, :md)");
    qryInsert.bindValue(":id", temp);
    qryInsert.bindValue(":au", Autor);
    qryInsert.bindValue(":op", Opis);
    qryInsert.bindValue(":da", data.date.toString("yyyy-MM-dd"));
    qryInsert.bindValue(":fo", Format);
    qryInsert.bindValue(":md",MD5Sum);
    if(!qryInsert.exec())
    {
        qFatal("Failed to insert into info");
        QSqlQuery qry1(QSqlDatabase::database("INDEX"));
        qry1.prepare("DELETE FROM IndexTable WHERE Id=:id");
        qry1.bindValue(":id",temp);
        if(!qry1.exec())
            qFatal("Failed to drop data base dirty :(");
    }
    
    QBuffer buffer;
    image.save(&buffer,Format.toAscii(),quality);

    QBuffer buffer2;
    data.mini.save(&buffer2,Format.toAscii());
    
    QSqlQuery qryInsertPhoto(QSqlDatabase::database("PHOTOS"));
    qryInsertPhoto.prepare("INSERT INTO Photo (Id, Image) VALUES(:id, :fot)");
    qryInsertPhoto.bindValue(":id",temp);
    qryInsertPhoto.bindValue(":fot",buffer.data());
    if(!qryInsertPhoto.exec())
    {
        qFatal("Failed to insert into Photo");
        QSqlQuery qry1(QSqlDatabase::database("INDEX"));
        qry1.prepare("DELETE FROM IndexTable WHERE Id=:id");
        qry1.bindValue(":id",temp);
        if(!qry1.exec())
            qFatal("Failed to drop data base dirty :(");
        qry1.prepare("DELETE FROM Info WHERE Id=:id");
        qry1.bindValue(":id",temp);
        if(!qry1.exec())
            qFatal("Failed to drop data base dirty :(");
    }
    QSqlQuery qryInsertMini(QSqlDatabase::database("MINIS"));
    qryInsertMini.prepare("INSERT INTO Mini (Id, ImageM) VALUES(:id, :min)");
    qryInsertMini.bindValue(":id",temp);
    qryInsertMini.bindValue(":min",buffer2.data());
    if(!qryInsertMini.exec())
    {
        qFatal("Failed to insert into Mini");
        QSqlQuery qry1(QSqlDatabase::database("INDEX"));
        qry1.prepare("DELETE FROM IndexTable WHERE Id=:id");
        qry1.bindValue(":id",temp);
        if(!qry1.exec())
            qFatal("Failed to drop data base dirty :(");
        qry1.prepare("DELETE FROM Info WHERE Id=:id");
        qry1.bindValue(":id",temp);
        if(!qry1.exec())
            qFatal("Failed to drop data base dirty :(");
        QSqlQuery qryP1(QSqlDatabase::database("PHOTOS"));
        qryP1.prepare("DELETE FROM Photo WHERE Id=:id");
        qryP1.bindValue(":id",temp);
        if(!qryP1.exec())
            qFatal("Failed to drop data base dirty :(");
    }
    return temp;
}

void ColectionManipulator::openDataBase(QString filename, bool cheack)
{
    if(!hasDatabase)
    {
        QSqlDatabase db =  QSqlDatabase::addDatabase("QSQLITE","INDEX");
        db.setDatabaseName(filename);
        if(!db.open())
            qFatal("Failed to open database");
        QSqlDatabase dbP =  QSqlDatabase::addDatabase("QSQLITE","PHOTOS");
        dbP.setDatabaseName(filename + QString("P"));
        if(!dbP.open())
            qFatal("Failed to open database");
        QSqlDatabase dbM =  QSqlDatabase::addDatabase("QSQLITE","MINIS");
        dbM.setDatabaseName(filename + QString("M"));
        if(!dbM.open())
            qFatal("Failed to open database");
    }
    else
    {
        QSqlDatabase db = QSqlDatabase::database("INDEX");
        db.close();
        QSqlDatabase::removeDatabase("INDEX");
        QSqlDatabase dbP = QSqlDatabase::database("PHOTOS");
        dbP.close();
        QSqlDatabase::removeDatabase("PHOTOS");
        QSqlDatabase dbM = QSqlDatabase::database("MINIS");
        dbM.close();
        QSqlDatabase::removeDatabase("MINIS");

        QSqlDatabase db1 =  QSqlDatabase::addDatabase("QSQLITE","INDEX");
        db1.setDatabaseName(filename);
        if(!db1.open())
            qFatal("Failed to open database");
        QSqlDatabase dbP1 =  QSqlDatabase::addDatabase("QSQLITE","PHOTOS");
        dbP1.setDatabaseName(filename + QString("P"));
        if(!dbP1.open())
            qFatal("Failed to open database");
        QSqlDatabase dbM1 =  QSqlDatabase::addDatabase("QSQLITE","MINIS");
        dbM1.setDatabaseName(filename + QString("M"));
        if(!dbM1.open())
            qFatal("Failed to open database");
    }
    if(cheack)
        checkBaseCount();
    hasDatabase = true;
}

void ColectionManipulator::checkBaseCount()
{
    QSqlQuery photoQuery(QSqlDatabase::database("PHOTOS"));
    photoQuery.prepare("SELECT count(Id) FROM Photo");
    if(!photoQuery.exec())
        qFatal("Failed to count photos.");
    int photoCount = 0;
    while(photoQuery.next())
        photoCount = photoQuery.value(0).toInt();

    QSqlQuery miniQuery(QSqlDatabase::database("MINIS"));
    miniQuery.prepare("SELECT count(Id) FROM Mini");
    if(!miniQuery.exec())
        qFatal("Failed to count miniatures.");
    int miniatureCount = 0;
    while(miniQuery.next())
        miniatureCount = miniQuery.value(0).toInt();

    QSqlQuery indexQuery(QSqlDatabase::database("INDEX"));
    indexQuery.prepare("SELECT count(Id) FROM IndexTable");
    if(!indexQuery.exec())
        qFatal("Failed to count index.");
    int indexCount = 0;
    while(indexQuery.next())
        indexCount = indexQuery.value(0).toInt();

    if(photoCount != miniatureCount || photoCount != indexCount)
    {
        QMessageBox::critical(NULL,QObject::tr("Album corrupt"),
                              QObject::tr("The album sums do not match! Fixing."));
        fixBase(photoCount,miniatureCount,indexCount);
    }
}

void ColectionManipulator::fixBase(int photoCount,int miniCount,int indexCount)
{
    if(photoCount != miniCount)
    {
        QSqlQuery photoQuery(QSqlDatabase::database("PHOTOS"));
        QSqlQuery miniQuery(QSqlDatabase::database("MINIS"));
        QList<int> photoId;
        QList<int> miniId;

        photoQuery.prepare("SELECT Id From Photo");
        if(!photoQuery.exec())
            qFatal("Failed to retrive photo id.");
        while(photoQuery.next())
            photoId << photoQuery.value(0).toInt();

        miniQuery.prepare("SELECT Id From Mini");
        if(!miniQuery.exec())
            qFatal("Failed to retrive mini id.");
        while(miniQuery.next())
            miniId << miniQuery.value(0).toInt();
        if(miniId.count() < photoId.count())
        {
            for(int i = 0; i < photoId.count(); ++i)
            {
                if(!miniId.contains(photoId.at(i)))
                {
                    QImage mini = getPhoto(photoId.at(i)).scaled(128,128,Qt::KeepAspectRatio,
                                                                 Qt::FastTransformation);
                    QBuffer buffer;
                    mini.save(&buffer,"PNG");
                    miniQuery.prepare("INSERT INTO Mini (Id, ImageM) VALUES(:id, :min)");
                    miniQuery.bindValue(":id",photoId.at(i));
                    miniQuery.bindValue(":min",buffer.data());
                    if(!miniQuery.exec())
                        qFatal("Failed to insert mini.");
                }
            }
        }
        else
        {
            for(int i = 0; i < miniId.count(); ++i)
            {
                if(!photoId.contains(miniId.at(i)))
                {
                    miniQuery.prepare("DELETE FROM Mini WHERE Id=:id");
                    miniQuery.bindValue(":id",photoId.at(i));
                    if(!miniQuery.exec())
                        qFatal("Failed to drop mini.");
                }
            }
        }
    }

    if(photoCount != indexCount)
    {
        QSqlQuery photoQuery(QSqlDatabase::database("PHOTOS"));
        QSqlQuery indexQuery(QSqlDatabase::database("INDEX"));
        QList<int> photoId;
        QList<int> indexId;

        photoQuery.prepare("SELECT Id From Photo");
        if(!photoQuery.exec())
            qFatal("Failed to retrive photo id.");
        while(photoQuery.next())
            photoId << photoQuery.value(0).toInt();

        indexQuery.prepare("SELECT Id From IndexTable");
        if(!indexQuery.exec())
            qFatal("Failed to retrive index id.");
        while(indexQuery.next())
            indexId << indexQuery.value(0).toInt();

        if(photoId.count() > indexId.count())
        {
            for(int i = 0; i < photoId.count(); ++i)
            {
                if(!indexId.contains(photoId.at(i)))
                {
                    QBuffer buffer;
                    QImageWriter writer(&buffer,"PNG");
                    writer.write(QImage(getPhoto(photoId.at(i))));
                    QString temp(QCryptographicHash::hash(buffer.data(),QCryptographicHash::Md4));
                    indexQuery.prepare("INSERT INTO IndexTable (Id,Catalog,Name) VALUES(:id, :kat, :na)");
                    indexQuery.bindValue(":id", photoId.at(i));
                    indexQuery.bindValue(":kat", "Lost+Found");
                    indexQuery.bindValue(":na", QString("%1").arg(photoId.at(i)));
                    if(!indexQuery.exec())
                    {
                        qFatal("Failed to insert into Indeks");
                    }
                    indexQuery.prepare("INSERT INTO Info (Id, Autor, Disc, InsDate, Format, MD5SUM) "
                                      "VALUES(:id, :au, :op, :da, :fo, :md)");
                    indexQuery.bindValue(":id", photoId.at(i));
                    indexQuery.bindValue(":au", "LOST");
                    indexQuery.bindValue(":op", "LOST");
                    indexQuery.bindValue(":da", "1970-01-01");
                    indexQuery.bindValue(":fo", "PNG");
                    indexQuery.bindValue(":md",temp);
                    if(!indexQuery.exec())
                    {
                        qFatal("Failed to insert into info");
                        QSqlQuery qry1(QSqlDatabase::database("INDEX"));
                        qry1.prepare("DELETE FROM IndexTable WHERE Id=:id");
                        qry1.bindValue(":id",photoId.at(i));
                        if(!qry1.exec())
                            qFatal("Failed to drop data base dirty :(");
                    }
                }
            }
        }
        else
        {
            for(int i = 0; i < indexId.count(); ++i)
            {
                if(!photoId.contains(indexId.at(i)))
                {
                    indexQuery.prepare("DELETE FROM IndexTable WHERE Id=:id");
                    indexQuery.bindValue(":id",indexId.at(i));
                    if(!indexQuery.exec())
                        qFatal("Failed delete data from Index.");

                    indexQuery.prepare("DELETE FROM Info WHERE Id=:id");
                    indexQuery.bindValue(":id",indexId.at(i));
                    if(!indexQuery.exec())
                        qFatal("Failed delete data from Info.");

                    indexQuery.prepare("DELETE FROM TagTable WHERE Id=:id");
                    indexQuery.bindValue(":id",indexId.at(i));
                    if(!indexQuery.exec())
                        qFatal("Failed delete data from Tag Table.");
                }
            }
        }
    }
}

QString ColectionManipulator::getPassWD()
{
    QString passWord;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT passWD FROM AlbumData");
    if(!qry.exec())
    {
        qFatal("Failed to retrive password.");
    }
    while(qry.next())
    {
        passWord=qry.value(0).toString();
    }
    return passWord;
}

void ColectionManipulator::moveImage(int index,QString catalogName)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("UPDATE IndexTable SET Catalog=:cat WHERE Id=:id");
    qry.bindValue(":cat",catalogName);
    qry.bindValue(":id",index);
    if(!qry.exec())
        qFatal("Failed to move image.");
}

void ColectionManipulator::dropImage(int index)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("DELETE FROM IndexTable WHERE Id=:id");
    qry.bindValue(":id",index);
    if(!qry.exec())
        qFatal("Failed delete data from Index.");

    qry.prepare("DELETE FROM Info WHERE Id=:id");
    qry.bindValue(":id",index);
    if(!qry.exec())
        qFatal("Failed delete data from Info.");

    qry.prepare("DELETE FROM TagTable WHERE Id=:id");
    qry.bindValue(":id",index);
    if(!qry.exec())
        qFatal("Failed delete data from Tag Table.");

    QSqlQuery qryP(QSqlDatabase::database("PHOTOS"));
    qryP.prepare("DELETE FROM Photo WHERE Id=:id");
    qryP.bindValue(":id",index);
    if(!qryP.exec())
        qFatal("Failed delete data from Photo.");

    QSqlQuery qryM(QSqlDatabase::database("MINIS"));
    qryM.prepare("DELETE FROM Mini WHERE Id=:id");
    qryM.bindValue(":id",index);
    if(!qryM.exec())
        qFatal("Failed delete data from Mini.");
}

void ColectionManipulator::changeImage(int index,QImage photo,QImage mini,
                                       QString hash,QString format)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));;
    qry.prepare("UPDATE Info SET MD5SUM=:md WHERE Id=:id");
    qry.bindValue(":md",hash);
    qry.bindValue(":id",index);

    if(!qry.exec())
        qFatal("Failed to update MD sum");

    QBuffer buffer;
    photo.save(&buffer,format.toAscii());

    QBuffer buffer2;
    mini.save(&buffer2,format.toAscii());

    QSqlQuery qryP(QSqlDatabase::database("PHOTOS"));
    qryP.prepare("UPDATE Photo SET Image=:fo WHERE Id=:id");
    qryP.bindValue(":fo",buffer.data());
    qryP.bindValue(":id",index);
    if(!qryP.exec())
        qFatal("Failed to update Fota");

    QSqlQuery qryM(QSqlDatabase::database("MINIS"));
    qryM.prepare("UPDATE Mini SET ImageM=:mi WHERE Id=:id");
    qryM.bindValue(":mi",buffer2.data());
    qryM.bindValue(":id",index);
    if(!qryM.exec())
        qFatal("Failed to update Minia");
}

void ColectionManipulator::updateImageDescription(int index,QString description)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("UPDATE Info SET Disc=:op WHERE Id=:id");
    qry.bindValue(":op",description);
    qry.bindValue(":id",index);
    if(!qry.exec())
        qFatal("Failed to move image.");
}

void ColectionManipulator::addTag(int index,QString tag)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    QStringList currentTags;
    qry.prepare("SELECT Tag FROM TagTable WHERE Id=:id");
    qry.bindValue(":id",index);
    if(!qry.exec())
        qFatal("Failed to retrive tags");
    while(qry.next())
        currentTags.append(qry.value(0).toString());
    if(currentTags.contains(tag)) return;

    qry.prepare("INSERT INTO TagTable (Id, Tag) VALUES(:id, :tag)");
    qry.bindValue(":id",index);
    qry.bindValue(":tag",tag);
    if(!qry.exec())
        qFatal("Failde to insert tag");
}

void ColectionManipulator::removeTag(int index,QString tag)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("DELETE FROM TagTable WHERE Id=:id AND Tag=:tag");
    qry.bindValue(":id",index);
    qry.bindValue(":tag",tag);
    if(!qry.exec())
        qFatal("Failed delete data from Tag Table.");
}

void ColectionManipulator::renameImage(int index,QString name)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("UPDATE IndexTable SET Name=:na WHERE Id=:id");
    qry.bindValue(":na",name);
    qry.bindValue(":id",index);
    if(!qry.exec())
        qFatal("Failed to move image.");
}

QStringList ColectionManipulator::getPhotoTags(int index)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT Tag FROM TagTable WHERE Id=:id");
    qry.bindValue(":id",index);
    if(!qry.exec())
        qFatal("Failed to retrive tags");
    QStringList resault;
    while(qry.next())
        resault.append(qry.value(0).toString());
    return resault;
}

void ColectionManipulator::newDataBase(QString filename,QString pass)
{
    openDataBase(filename,false);
    
    QSqlQuery indexQuery(QSqlDatabase::database("INDEX"));

    //creating AlbumData TABLE
    indexQuery.prepare("CREATE TABLE IF NOT EXISTS AlbumData"
                       "(passWD VARCHAR(30))");
    if(!indexQuery.exec())
        qFatal("Failed to create AlbumData table.");
    
    //creating INDEX TABLE
    indexQuery.prepare("CREATE TABLE IF NOT EXISTS IndexTable"
                       "(Id INTEGER PRIMARY KEY,"
                       "Catalog VARCHAR(30),"
                       "Name VARCHAR(30))");
    if(!indexQuery.exec())
        qFatal("Failed to create IndexTable table.");

    //creating INFO TABLE
    indexQuery.prepare("CREATE TABLE IF NOT EXISTS Info"
                       "(Id INTEGER,"
                       "Autor VARCHAR(30),"
                       "Disc VARCHAR(80),"
                       "InsDate DATE,"
                       "Format VARCHAR(4),"
                       "MD5SUM VARCHAR(16))");
    if(!indexQuery.exec())
        qFatal("Failed to create info table.");

    //creating TAG TABLE
    indexQuery.prepare("CREATE TABLE IF NOT EXISTS TagTable"
                       "(Id INTEGER,"
                       "Tag VARCHAR(10))");
    if(!indexQuery.exec())
        qFatal("Failed to create tag table.");

    indexQuery.prepare("INSERT INTO AlbumData (passWD) VALUES(:pass)");
    indexQuery.bindValue(":pass",pass);
    if(!indexQuery.exec())
    {
        qFatal("Failed to insert password");
    }

    QSqlQuery photoQuery(QSqlDatabase::database("PHOTOS"));

    //creating PHOTO TABLE
    photoQuery.prepare("CREATE TABLE IF NOT EXISTS Photo"
                       "(Id INTEGER,"
                       "Image BLOB)");
    if(!photoQuery.exec())
        qFatal("Failed to create Photo table");

    QSqlQuery miniQuery(QSqlDatabase::database("MINIS"));
    //creating MINI TABLE
    miniQuery.prepare("CREATE TABLE IF NOT EXISTS Mini"
                      "(Id INTEGER,"
                      "ImageM BLOB)");
    if(!miniQuery.exec())
        qFatal("Failed to create Mini table");


}

ColectionManipulator::ColectionManipulator()
{
    hasDatabase = false;
}
