#include "GS_albumer.h"

void GS_albumer::closeEvent(QCloseEvent* event)
{
    foreach(GS_albumer_viewer* view, views.values())
    {
        view->setNeedToClose();
        if(!view->close())
        {
            event->ignore();
            return;
        }
    }
    inserter->setNeedToClose();
    inserter->close();
    folder_inserter->setNeedToClose();
    folder_inserter->close();
    informator->setNeedToClose();
    informator->close();
    progress->setNeedToClose();
    progress->close();
    col_searcher->setNeedToClose();
    col_searcher->close();
    config->setNeedToClose();
    config->close();
    entryEditor->setNeedToClose();
    entryEditor->close();
    slideShow->close();
    if(mirrorView != NULL)
    {
        mirrorView->close();
        delete mirrorView;
    }
    foreach(GS_albumer_folder_view2* view, fview.values())
    {
        view->setNeedToClose();
        view->close();
    }
    foreach(GS_albumer_folder_view2* view, sview.values())
    {
        view->setNeedToClose();
        view->close();
    }
    QMainWindow::closeEvent(event);
}

void GS_albumer::openImage(int index)
{
    if(views.count()>=15)
    {
        QMessageBox::information (this,tr("NO")
                                  ,tr("Please close some photos first."));
        return;
    }
    progress->setCancel(false);
    progress->setMaxCount(2);
    progress->show();
    progress->setCount(0);
    GS_albumer_viewer* temp;
    temp = new GS_albumer_viewer(man);
    const QImage image = man->getPhoto(index);
    const QString format = man->getPhotoFormat(index);
    progress->setCount(1);
    temp->setImage(image,format,index);
    if(GS_config_holder::getViewMaxi())
        temp->showMaximized();
    else
        temp->show();
    progress->setCount(2);
    views[temp->myIndex()] = temp;
    connect(views[temp->myIndex()],SIGNAL(wantToClose(int)),this,
            SLOT(closeImage(int)));
    connect(views[temp->myIndex()],SIGNAL(imageChanged(int)),this
            ,SLOT(photoChanged(int)));
    connect(views[temp->myIndex()],SIGNAL(newImageAdded(QString,int)),this
            ,SLOT(newPhotoInserted(QString,int)));
    progress->hide();
}

void GS_albumer::exportImage(int id)
{

    QString myImageFormat = man->getPhotoFormat(id);
    QString fileFilter = myImageFormat + " (*." + myImageFormat.toLower() + ")";
    QString fname="";
    fname = QFileDialog::getSaveFileName(this, tr("Save File"),QDir::homePath(),
                                         fileFilter);
    if(fname != "")
    {
        progress->setCancel(false);
        progress->setMaxCount(2);
        progress->show();
        progress->setCount(0);
        QPixmap myImage = QPixmap::fromImage(man->getPhoto(id));
        progress->setCount(1);
        if(!fname.endsWith("." + myImageFormat.toLower()))
            fname += "." + myImageFormat.toLower();
        QImageWriter writer(fname,myImageFormat.toAscii());
        QImage temp = myImage.toImage();
        writer.write(temp);
        progress->setCount(2);
        progress->hide();
    }
}

void GS_albumer::exportImages(QList<int> items)
{
    QString dir ="";
    dir = QFileDialog::getExistingDirectory(this);
    if(dir.trimmed() != "")
    {
        progress->setCancel(false);
        progress->show();
        progress->setMaxCount(items.count());
        for(int i = 0; i < items.count(); ++i)
        {
            progress->setCount(i);
            QPixmap myImage = QPixmap::fromImage(man->getPhoto(items.at(i)));
            QString myImageFormat = man->getPhotoFormat(items.at(i));
            QStringList tempList = man->getPhotoInfo(items.at(i));
            QString myImageFileName = dir+"/"+tempList.at(1)+"_"+tempList.at(0);
            tempList.clear();
            int j = 0;
            while(QFile(myImageFileName+"."+myImageFormat.toLower()).exists())
            {
                ++j;
                if(j>1)
                    myImageFileName.chop(QString(tr("%1").arg(j)).length()+2);
                myImageFileName += QString(tr("(%1)").arg(j));
            }
            QImageWriter writer(myImageFileName+"."+myImageFormat.toLower(),
                                myImageFormat.toAscii());
            QImage temp = myImage.toImage();
            writer.write(temp);
        }
        progress->hide();
    }
}

void GS_albumer::showFolderImage(int index)
{
    openImage(index);
}

void GS_albumer::fViewWantsToMoveItem(int item,int view)
{
    QStringList catalogs = man->getFolders();
    QString sourceCatalog = man->getCatalog(item);
    QList<int> tempList;
    tempList<<item;
    catalogs.removeAll(sourceCatalog);
    imageMover->setCatalogList(catalogs,sourceCatalog,tempList,view);
    imageMover->show();
}

void GS_albumer::fViewWantsToMoveItems(QList<int> items,int view)
{
    QStringList catalogs = man->getFolders();
    QStringList sourceCatalogs;
    for(int i = 0; i < items.count(); ++i)
    {
        QString temp = man->getCatalog(items.at(i));
        if(!sourceCatalogs.contains(temp))
            sourceCatalogs<<temp;
    }
    for(int i = 0; i < sourceCatalogs.count(); ++i)
        catalogs.removeAll(sourceCatalogs.at(i));
    QString scrCat = tr("Catalog group.");
    if(sourceCatalogs.count() == 1)
        scrCat = sourceCatalogs.first();
    imageMover->setCatalogList(catalogs,scrCat,items,view);
    imageMover->show();
}

void GS_albumer::dropActionMoveImage(int imageId, int pickerItemId)
{
    QString newCatalog = folderMap.value(pickerItemId)->getName();
    QString oldCatalog = man->getCatalog(imageId);
    //if(man->getFolderContentCount(oldCatalog) == 1) return; //information needed
    if(oldCatalog != newCatalog)
    {
        man->moveImage(imageId,newCatalog);
        
        for(int j = 0; j < fview.values().count(); ++j)
        {
            if(fview.values()[j]->isItMe(newCatalog))
            {
                fview.values()[j]->addItem(imageId);
                break;
            }
        }

        updateFolderData();
        loadAlbum(false);

        for(int i = 0; i < fview.values().count(); ++i)
        {
            if(fview.values()[i]->isItMe(oldCatalog))
            {
                fview.values()[i]->removeItem(imageId,true);
                break;
            }
        }
    }
}

void GS_albumer::dropActionTagImage(int imageId, int pickedItemId)
{
    QString newTag = tagMap.value(pickedItemId)->getName();
    editionList.clear();
    editionList.append(imageId);
    addTag(newTag);
}

void GS_albumer::updateFolderData()
{
    QStringList folders=man->getFolders();
    for(int i =0; i < folders.count(); ++i)
    {
        int itemCount = man->getFolderContentCount(folders.at(i));
        FolderInfo* folderInfo = new FolderInfo(folders.at(i),itemCount);
        folderMap.erase(folderMap.find(i));
        folderMap[i] = folderInfo;
        totalPhotos += itemCount;
    }
}

void GS_albumer::catalogPicked(QList<int> items,int view)
{
    bool proceed = false;
    QString oldCatalog = imageMover->getOldCatalog();
    QString newCatalog = imageMover->getSelectedCatalog();
    if(newCatalog.trimmed() != "")
        proceed = true;
    if(!proceed)//operation canceled
    {
        if(fview.contains(view) && fview.value(view)->wasItMe())
        {
            fview.value(view)->removeItem(items.at(0),false);
            fview.value(view)->unMarkItems();
        }
        if(sview.contains(view) && sview.value(view)->wasItMe())
        {
            sview.value(view)->removeItem(items.at(0),false);
            sview.value(view)->unMarkItems();
        }
    }
    else//operation performe
    {
        //move photo to folder
        progress->setMaxCount(items.count());
        progress->setCancel(false);
        progress->show();
        for(int i = 0; i < items.count(); ++i)
        {
            progress->setCount(i);
            man->moveImage(items.at(i),newCatalog);
            foreach(GS_albumer_folder_view2* view_, fview.values())
            {
                view_->removeItem(items.at(i),true);
            }
            if(sview.contains(view) && sview.value(view)->wasItMe())
            {
                sview.value(view)->removeItem(items.at(i),false);
                sview.value(view)->unMarkItems();
            }
        }
        progress->hide();
        foreach(GS_albumer_folder_view2* view_, fview.values())
        {
            if(view_->isItMe(newCatalog))
            {
                for(int i = 0; i < items.count(); ++i)
                    view_->addItem(items.at(i));
            }
        }
        loadAlbum(false);
    }
}

void GS_albumer::resetView(int viewIndex,QString catalog)
{
    QList<int> list = man->getPhotosFromFolder(catalog);
    int count = 0;
    progress->setCancel(false);
    progress->show();
    progress->setMaxCount(list.count());
    progress->setCount(count);
    fview.value(viewIndex)->resetFolderImages(list);
    list.clear();
    progress->hide();
}

void GS_albumer::resetTagView(int viewIndex,QString tag)
{
    QList<int> list = man->searchForPhotosByTag(tag);
    int count = 0;
    progress->setCancel(false);
    progress->show();
    progress->setMaxCount(list.count());
    progress->setCount(count);
    sview.value(viewIndex)->resetFolderImages(list);
    list.clear();
    progress->hide();
}

void GS_albumer::fViewWantsToRemoveItem(int item,int view)
{
    bool proceed = false;
    if(QMessageBox::question(this,tr("Removing Photo"),
                             tr("Are you sure you want to delete this photo?"),
                             QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok)
        proceed = true;
    if(!proceed)//operation canceled
    {
        if(fview.contains(view) && fview.value(view)->wasItMe())
        {
            fview.value(view)->removeItem(item,false);
        }
        if(sview.contains(view) && sview.value(view)->wasItMe())
        {
            sview.value(view)->removeItem(item,false);
        }
    }
    else//operation performe
    {
        //removal from database
        man->dropImage(item);
        foreach(GS_albumer_folder_view2* view_, fview.values())
            view_->removeItem(item,true);
        foreach(GS_albumer_folder_view2* view_, sview.values())
            view_->removeItem(item,true);
        foreach(GS_albumer_viewer* view_, views.values())
        {
            if(view_->isMyImage(item))
            {
                view_->setNeedToClose();
                view_->close();
            }
        }
        loadAlbum(false);
    }
}

void GS_albumer::fViewWantsToRemoveItems(QList<int> items,int view)
{
    bool proceed = false;
    if(QMessageBox::question(this,tr("Removing Photos"),
                             tr("Are you sure you want to delete these photos?"),
                             QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok)
        proceed = true;
    if(!proceed)//operation canceled
    {
        if(fview.contains(view) && fview.value(view)->wasItMe())
        {
            fview.value(view)->removeItem(items.at(0),false);
            //fview.value(view)->unMarkItems();
        }
        if(sview.contains(view) && sview.value(view)->wasItMe())
        {
            sview.value(view)->removeItem(items.at(0),false);
            //sview.value(view)->unMarkItems();
        }
    }
    else//operation performe
    {
        //removal from database
        progress->setMaxCount(items.count());
        progress->setCancel(false);
        progress->show();
        for(int i=0; i< items.count(); ++i)
        {
            progress->setCount(i);
            man->dropImage(items.at(i));
            foreach(GS_albumer_folder_view2* view_, fview.values())
                view_->removeItem(items.at(i),true);
            foreach(GS_albumer_folder_view2* view_, sview.values())
                view_->removeItem(items.at(i),true);
            foreach(GS_albumer_viewer* view_, views.values())
            {
                if(view_->isMyImage(items.at(i)))
                {
                    view_->setNeedToClose();
                    view_->close();
                }
            }
        }
        progress->hide();
        loadAlbum(false);
    }
}

void GS_albumer::fViewAddTag(QList<int> itemList)
{
    if(itemList.count() > 0)
    {
        editionList = itemList;
        QStringList tagList = man->getTags();
        TagEditor* editro = new TagEditor(this);
        editro->setTagList(tagList);
        connect(editro,SIGNAL(tagPicked(QString)),this,SLOT(addTag(QString)));
        editro->show();
    }
}

void GS_albumer::addTag(QString tag)
{
    if(tag != "" && editionList.count() > 0)
    {
        tag.remove("|");
        progress->setCancel(false);
        progress->setMaxCount(editionList.count());
        progress->show();
        progress->setCount(0);
        for(int i = 0; i < editionList.count(); ++i)
        {
            man->addTag(editionList.at(i),tag);
            progress->setCount(i);
        }
        progress->hide();
        loadAlbum(false);
        QString key =tr("Tags: \"") + tag +"\"";
        foreach(GS_albumer_folder_view2* view_, sview.values())
        {
            if(view_->isItMe(key))
            {
                for(int i = 0; i < editionList.count(); ++i)
                    view_->addItem(editionList.at(i));
            }
        }
        editionList.clear();
    }
}

void GS_albumer::fViewRemoveImageTag(int index)
{
    QStringList tagList = man->getPhotoTags(index);
    if(tagList.count() > 0)
    {
        TagRemover* remover = new TagRemover(this);
        remover->setData(index,tagList);
        connect(remover,SIGNAL(removeImageTag(int,QString)),
                this,SLOT(removeTag(int,QString)));
        remover->show();
    }
    else
        QMessageBox::information(this,tr("Remove Tag"),tr("This photo has no tags"));
}

void GS_albumer::removeTag(int id,QString tag)
{
    progress->setCancel(false);
    progress->setMaxCount(1);
    progress->show();
    progress->setCount(0);
    man->removeTag(id,tag);
    progress->setCount(1);
    progress->hide();
    loadAlbum(false);
    QString key =tr("Tags: \"") + tag +"\"";
    foreach(GS_albumer_folder_view2* view_, sview.values())
    {
        if(view_->isItMe(key))
        {
            view_->removeItem(id,true);;
        }
    }
}

void GS_albumer::fViewRenameImages(QList<int> items)
{
    if(items.count() > 0)
    {
        editionList = items;
        NameEditor* editor = new NameEditor(this);
        connect(editor,SIGNAL(newNamePicked(QString)),
                this,SLOT(renameImages(QString)));
        editor->show();
    }
}

void GS_albumer::renameImages(QString newName)
{
    if(newName != "" && editionList.count() > 0)
    {
        newName.remove(".");
        newName.remove("/");
        newName.remove("\\");
        progress->setCancel(false);
        progress->setMaxCount(editionList.count());
        progress->show();
        progress->setCount(0);
        for(int i = 0; i < editionList.count(); ++i)
        {
            QString tempName = newName;
            if(i > 0)
            {
                QString number = QString("%1").arg(i);
                if(tempName.size() + number.size() > 30)
                    tempName.resize(30 - number.size());
                tempName += number;
            }
            man->renameImage(editionList.at(i),tempName);
            progress->setCount(i);
        }
        foreach(GS_albumer_folder_view2* view_, fview.values())
            view_->loadNames(editionList);
        foreach(GS_albumer_folder_view2* view_, sview.values())
            view_->loadNames(editionList);
        editionList.clear();
        progress->hide();
    }
}

void GS_albumer::tagTouched(int tagIndex)
{
    this->folderPhotos = tagMap.value(tagIndex)->getItemCount();
    slider->updateSlide(folderPhotos,totalPhotos);
    welcomeWidget->setHFolderCount(folderPhotos);
}

void GS_albumer::showImagesWithTag(int tagIndex)
{
    QString tag = tagMap.value(tagIndex)->getName();
    tagSearch(tag);
}

void GS_albumer::showImagesFromFolder(int folderIndex)
{
    QString folderName = folderMap.value(folderIndex)->getName();
    foreach(GS_albumer_folder_view2* view, fview.values())
    {
        if(view->isItMe(folderName))
        {
            folderTab->setCurrentWidget(view);
            return;
        }
    }
    GS_albumer_folder_view2* temp = new GS_albumer_folder_view2(man);
    QList<int> list = man->getPhotosFromFolder(folderName);
    QStringList dateList;
    QStringList titleList;
    progress->setCancel(false);
    progress->setMaxCount(list.count());
    int counter = 0;
    progress->setCount(counter);
    progress->show();
    foreach(const int index, list)
    {
        QStringList temp = man->getPhotoInfo(index);
        titleList<<temp.takeFirst();
        dateList<<temp.takeAt(3);
        temp.clear();
        ++counter;
        progress->setCount(counter);
    }
    ++counter;
    progress->setCount(counter);
    QString viewTitle = tr("Catalog: \"") + folderName + "\"";
    temp->setFolderImages(list,titleList,dateList,viewTitle,folderName);
    fview[temp->myIndex()] = temp;
    connect(fview[temp->myIndex()],SIGNAL(imageClicked(int)),this,
            SLOT(showFolderImage(int)));
    connect(fview[temp->myIndex()],SIGNAL(wantToClose(int)),this,
            SLOT(closeFolderView(int)));
    connect(fview[temp->myIndex()],SIGNAL(startSlideShow(QList<int>)),this,
            SLOT(startSlideShow(QList<int>)));
    connect(fview[temp->myIndex()],SIGNAL(startMirror(QList<int>)),this,
            SLOT(startMirrorView(QList<int>)));
    connect(fview[temp->myIndex()],SIGNAL(imageMovePicked(int,int)),this,
            SLOT(fViewWantsToMoveItem(int,int)));
    connect(fview[temp->myIndex()],SIGNAL(imageRemovePicked(int,int)),this,
            SLOT(fViewWantsToRemoveItem(int,int)));
    connect(fview[temp->myIndex()],SIGNAL(imageExportPicked(int)),this,
            SLOT(exportImage(int)));
    connect(fview[temp->myIndex()],SIGNAL(imageEditDscPicked(int)),this,
            SLOT(openEntryEdition(int)));
    connect(fview[temp->myIndex()],SIGNAL(imageInfoShowPicked(int)),this,
            SLOT(showImageInfo(int)));
    connect(fview[temp->myIndex()],SIGNAL(groupRemovePicked(QList<int>,int)),
            this,SLOT(fViewWantsToRemoveItems(QList<int>,int)));
    connect(fview[temp->myIndex()],SIGNAL(groupMovePicked(QList<int>,int)),
            this,SLOT(fViewWantsToMoveItems(QList<int>,int)));
    connect(fview[temp->myIndex()],SIGNAL(groupExport(QList<int>)),this,
            SLOT(exportImages(QList<int>)));
    connect(fview[temp->myIndex()],SIGNAL(addTags(QList<int>)),
            this,SLOT(fViewAddTag(QList<int>)));
    connect(fview[temp->myIndex()],SIGNAL(removeTagPicked(int)),
            this,SLOT(fViewRemoveImageTag(int)));
    connect(fview[temp->myIndex()],SIGNAL(renameImages(QList<int>)),
            this,SLOT(fViewRenameImages(QList<int>)));

    list.clear();
    titleList.clear();
    dateList.clear();
    if(folderTab->width() < 700)
    {
        QList<int> sizeList;
        sizeList << width() - 700 << 700;
        splitter->setSizes(sizeList);
    }
    folderTab->addTab(fview[temp->myIndex()],folderName);
    folderTab->setCurrentIndex(folderTab->indexOf(fview[temp->myIndex()]));
    hasWindows = true;
    progress->hide();
}

void GS_albumer::closeImage(int index)
{
    views.value(index)->setNeedToClose();
    views.value(index)->close();
    views.remove(index);
}

void GS_albumer::closeFolderView(int index)
{
    fview.value(index)->setNeedToClose();
    fview.value(index)->close();
    fview.remove(index);
}

void GS_albumer::closeSearchView(int index)
{
    sview.value(index)->setNeedToClose();
    sview.value(index)->close();
    sview.remove(index);
}

void GS_albumer::showImageInfo(int index)
{
    informator->hide();
    informator->updateData(man->getPhotoInfo(index));
    int x,y;
    x = pos().x() + ((width() - informator->width())/2);
    y = pos().y() + ((height() - informator->height())/2);
    informator->move(x,y);
    informator->show();
}

void GS_albumer::insertImage()
{
    inserter->clearData();
    inserter->clearCatalogs();
    inserter->populateCatalogs(man->getFolders());
    int x,y;
    x = pos().x() + ((width() - inserter->width())/2);
    y = pos().y() + ((height() - inserter->height())/2);
    inserter->move(x,y);
    inserter->show();
}

void GS_albumer::insertReady()
{
    progress->setMaxCount(9);
    progress->setCancel(false);
    progress->show();
    progress->setCount(0);
    if(!QFile(inserter->getFile()).exists())
    {
        progress->hide();
        QMessageBox::information (this,tr("File error")
                                  ,tr("The file was not found."));
        return;
    }
    ImageData data;
    QPixmap inImage(inserter->getFile());
    QPixmap mini = inImage;
    if(inImage.width()>128)
    {
        mini = inImage.scaled(128,128,Qt::KeepAspectRatio,
                              Qt::FastTransformation);
    }
    if(inImage.height()>128)
    {
        mini = inImage.scaled(128,128,Qt::KeepAspectRatio,
                              Qt::FastTransformation);
    }
    data.mini = mini.toImage();
    progress->setCount(1);
    data.catalog = inserter->getCatalog();
    progress->setCount(2);
    data.name = inserter->getName();
    progress->setCount(3);
    data.author = inserter->getAuthor();
    progress->setCount(4);
    data.desc = inserter->getDisc();
    progress->setCount(5);
    data.format = inserter->getFormat();
    data.date = inserter->getDate();
    QBuffer buffer;
    inImage.save(&buffer,data.format.toAscii());
    QByteArray md5 =
            QCryptographicHash::hash(buffer.data(),QCryptographicHash::Md4);
    data.hash = QString(md5);
    progress->setCount(7);
    if(!man->hasImage(data.hash))
    {
        progress->hide();
        return;
    }
    progress->setCount(8);
    int newIndex = man->addImage(inImage.toImage(),data,GS_config_holder::getQuality());
    progress->setCount(9);
    loadAlbum(false);
    foreach(GS_albumer_folder_view2* view_, fview.values())
    {
        if(view_->isItMe(data.catalog))
        {
            view_->addItem(newIndex);
        }
    }
    progress->hide();
}

void GS_albumer::finsertReady()
{
    if(firstInsert)
    {
        finsertList = folder_inserter->getOutList();
        alteredFolders.clear();
        cancelF = false;
        progress->setMaxCount(finsertList.count());
        progress->setCancel(true);
        progress->show();
        progresCounter = 0;
        progress->setCount(progresCounter);
        emit insertImages(finsertList);
        firstInsert = false;
    }
    if(!finsertList.isEmpty() && !cancelF)
    {
        ImageData item = finsertList.takeFirst();
        if(!QFile(item.filePath).exists())
        {
            timer->start();
            return;
        }
        QString katalog = item.catalog;
        QPixmap inImage(item.filePath);
        if(!item.hasmini)
        {
            QPixmap mini = inImage;
            if(inImage.width()>128)
            {
                mini = inImage.scaled(128,128,Qt::KeepAspectRatio,
                                      Qt::FastTransformation);
            }
            if(inImage.height()>128)
            {
                mini = inImage.scaled(128,128,Qt::KeepAspectRatio,
                                      Qt::FastTransformation);
            }
            item.mini = mini.toImage();
        }
        if(item.limitSize)
        {
            if(inImage.width() > item.widthLimit || inImage.height() > item.heightLimit)
            {
                if(item.keepAspect)
                    inImage = inImage.scaled(item.widthLimit,item.heightLimit,
                                         Qt::KeepAspectRatio,Qt::SmoothTransformation);
                else
                    inImage = inImage.scaled(item.widthLimit,item.heightLimit,
                                         Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
            }
        }
        man->addImage(inImage.toImage(),item,GS_config_holder::getQuality());
        if(!alteredFolders.contains(katalog))
            alteredFolders<<katalog;
        progress->setCount(++progresCounter);
        timer->start();
    }
    else
    {
        firstInsert = true;
        loadAlbum(false);
        foreach(GS_albumer_folder_view2* view_, fview.values())
        {
            foreach(const QString newCatalog,alteredFolders)
            {
                if(view_->isItMe(newCatalog))
                {
                    resetView(fview.key(view_),newCatalog);
                }
            }
        }
        progress->hide();
    }
}

void GS_albumer::cancelFinsert()
{
    cancelF = true;
}

void GS_albumer::loadAlbum(bool newalbum)
{
    if(newalbum)
    {
        if(hasWindows)
        {
            foreach(GS_albumer_viewer* view, views.values())
            {
                view->setNeedToClose();
                if(!view->close()) return;
            }
            views.clear();
            foreach(GS_albumer_folder_view2* view, fview.values())
            {
                view->setNeedToClose();
                view->close();
            }
            fview.clear();
            foreach(GS_albumer_folder_view2* view, sview.values())
            {
                view->setNeedToClose();
                view->close();
            }
            sview.clear();
        }
        hasWindows = false;
    }
    if(mirrorView != NULL)
    {
        mirrorView->close();
        delete mirrorView;
    }
    if(!slideShow->isHidden())
        slideShow->hide();
    searchFolder->setEnabled(true);
    ui.searchPathButton->setEnabled(true);
    ui.searchColectionButton->setEnabled(true);
    searchColection->setEnabled(true);
    qSearcher->setEnabled(true);
    QStringList folders=man->getFolders();
    statusLabel->setText(tr("Album loaded."));
    welcomeWidget->clearData();
    this->totalPhotos = 0;
    this->folderPhotos = 0;
    if(folders.isEmpty())
    {
        slider->updateSlide(0,1);
    }

    if(folderMap.count() != 0)
    {
        for(int i = 0; i < folderMap.count(); ++i)
        {
            FolderInfo* temp = folderMap.value(i);
            delete temp;
        }
        folderMap.clear();
    }
    catalogSelector->saveVerticalPos();
    catalogSelector->clearItems();
    for(int i =0; i < folders.count(); ++i)
    {
        int itemCount = man->getFolderContentCount(folders.at(i));
        FolderInfo* folderInfo = new FolderInfo(folders.at(i),itemCount);
        folderMap[i] = folderInfo;
        totalPhotos += itemCount;
    }
    for(int i = 0; i < folderMap.count(); ++i)
    {
        if(i%2 == 0)
            catalogSelector->addItem(QPixmap(":/eyecandy/item_pressed.png"),
                                     QPixmap(":/eyecandy/item.png"),
                                     folderMap.value(i)->getName());
        else
            catalogSelector->addItem(QPixmap(":/eyecandy/item_pressed_alt.png"),
                                     QPixmap(":/eyecandy/item_alt.png"),
                                     folderMap.value(i)->getName());
    }

    if(tagMap.count() != 0)
    {
        for(int i = 0; i < tagMap.count(); ++i)
        {
            FolderInfo* temp = tagMap.value(i);
            delete temp;
        }
        tagMap.clear();
    }
    tagSelector->saveVerticalPos();
    tagSelector->clearItems();
    QStringList tags = man->getTags();
    for(int i = 0; i < tags.count(); ++i)
    {
        int itemCount = man->getTagImageCount(tags.at(i));
        FolderInfo* tagInfo = new FolderInfo(tags.at(i),itemCount);
        tagMap[i] = tagInfo;
    }
    for(int i = 0; i < tagMap.count(); ++i)
    {
        if(i%2 == 0)
            tagSelector->addItem(QPixmap(":/eyecandy/item_pressed_tag.png"),
                                     QPixmap(":/eyecandy/item_tag.png"),
                                     tagMap.value(i)->getName());
        else
            tagSelector->addItem(QPixmap(":/eyecandy/item_pressed_tag_alt.png"),
                                     QPixmap(":/eyecandy/item_tag_alt.png"),
                                     tagMap.value(i)->getName());
    }

    welcomeWidget->setData(QFileInfo(filename + QString("P")));
    welcomeWidget->setTotaCount(totalPhotos);
    welcomeWidget->setTotalFolderCount(folderMap.count());
    if(totalPhotos != 0)
        slider->updateSlide(folderPhotos,totalPhotos);
    insImage->setEnabled(true);
    ui.addImageButton->setEnabled(true);
    this->hasDatabase = true;
    catalogSelector->loadVerticalPos();
    tagSelector->loadVerticalPos();
}

void GS_albumer::folderTouched(int folderIndex)
{
    this->folderPhotos = folderMap.value(folderIndex)->getItemCount();
    slider->updateSlide(folderPhotos,totalPhotos);
    welcomeWidget->setHFolderCount(folderPhotos);
}

void GS_albumer::newAlbumSlot()
{
    if(hasDatabase) stopTheards();
    albCreator->clear();
    albCreator->show();
}

void GS_albumer::stopTheards()
{
    file_searcher->stop();
    for(int i = 0; i < file_filter_List.count(); ++i)
        file_filter_List[i]->stop();
    searchFolder->setEnabled(true);
    stopSearch->setEnabled(false);
    ui.searchPathButton->setEnabled(true);
    ui.stopSearchButton->setEnabled(false);
    animation->stopAnimation();
}

void GS_albumer::createNewAlbum()
{
    filename = albCreator->getFileName();
    QString passWd = albCreator->getPassword();
    QFile realFile(filename);
    if(realFile.exists())
    {
        if(!realFile.remove())
        {
            QMessageBox::information (this,tr("File error")
                                      ,tr("The index file could not be removed."));
            return;
        }
    }
    QFile realPhotoFile(filename + QString("P"));
    if(realPhotoFile.exists())
    {
        if(!realPhotoFile.remove())
        {
            QMessageBox::information (this,tr("File error")
                                      ,tr("The photo file could not be removed."));
            return;
        }
    }
    QFile realMiniatureFile(filename + QString("M"));
    if(realMiniatureFile.exists())
    {
        if(!realMiniatureFile.remove())
        {
            QMessageBox::information (this,tr("File error")
                                      ,tr("The miniature file could not be removed."));
            return;
        }
    }
    man->newDataBase(filename,passWd);
    loadAlbum();
}

void GS_albumer::openAlbumSlot()
{
    if(hasDatabase) stopTheards();
    QString filenameLocal = "";
    filenameLocal = QFileDialog::getOpenFileName(this,tr("Open Album"),QDir::homePath(),
                                                 tr("Album File(*.gsAlb)"));
    if(filenameLocal == "") return;
    if(!QFile::exists(filenameLocal + QString("P")))
    {
        QMessageBox::information (this,tr("File error")
                                  ,QString(tr("This album is missing the \n%1P file!\nThis might be an old/unsupported album format.").arg(filenameLocal)));
        return;
    }
    if(!QFile::exists(filenameLocal + QString("M")))
    {
        QMessageBox::information (this,tr("File error")
                                  ,QString(tr("This album is missing the \n%1M file!\nThis might be an old/unsupported album format.").arg(filenameLocal)));
        return;
    }
    progress->setCancel(false);
    progress->setMaxCount(2);
    progress->show();
    progress->setCount(0);
    filename = filenameLocal;
    progress->setCount(1);
    man->openDataBase(filenameLocal,GS_config_holder::getCheckBase());
    progress->setCount(2);
    progress->hide();
    QString passwd = man->getPassWD();
    if(passwd.trimmed() == "")
    {
        openValidationCompelat(true);
    }
    else
    {
        passer->setPassWord(passwd);
        passer->show();
    }
}

void GS_albumer::openValidationCompelat(bool passwordOk)
{
    if(passwordOk)
    {
        loadAlbum();
    }
    else
    {
        QMessageBox::information(this,tr("Access error"),
                                 tr("The album will not be loaded."));
        if(hasWindows)
        {
            foreach(GS_albumer_viewer* view, views.values())
            {
                view->setNeedToClose();
                if(!view->close()) return;
            }
            views.clear();
            foreach(GS_albumer_folder_view2* view, fview.values())
            {
                view->setNeedToClose();
                view->close();
            }
            fview.clear();
            foreach(GS_albumer_folder_view2* view, sview.values())
            {
                view->setNeedToClose();
                view->close();
            }
            sview.clear();
        }
        hasWindows = false;
        catalogSelector->clearItems();
        welcomeWidget->clearData();
        ui.searchPathButton->setEnabled(false);
        ui.searchColectionButton->setEnabled(false);
        ui.stopSearchButton->setEnabled(false);
        searchColection->setEnabled(false);
        searchFolder->setEnabled(false);
        ui.addImageButton->setEnabled(false);
        insImage->setEnabled(false);
    }
}

void GS_albumer::startFileSearch()
{
    QString dir ="";
    dir = QFileDialog::getExistingDirectory(this);
    if(dir != "")
    {
        file_searcher->setDir(dir);
        file_searcher->start();
        searchFolder->setEnabled(false);
        stopSearch->setEnabled(true);
        ui.searchPathButton->setEnabled(false);
        ui.stopSearchButton->setEnabled(true);;
        statusLabel->setText(tr("Searching for files."));
        animation->startAnimation();
    }

}

void GS_albumer::fileSearchComp()
{
    if(!file_searcher->getFileList().isEmpty())
    {
        hashList.clear();
        hashList = man->getMd5Sums();
        filterComp = 0;
        QList<QFileInfo> tempList = file_searcher->getFileList();
        int pack = tempList.count()/ file_filter_List.count();
        for(int i = 0; i < (file_filter_List.count() - 1); ++i)
        {
            QList<QFileInfo> localList;
            for(int j = 0; j < pack; ++j)
                localList << tempList.takeFirst();
            file_filter_List[i]->setFilterLists(localList);
        }
        file_filter_List.last()->setFilterLists(tempList);
        for(int i = 0; i < file_filter_List.count(); ++i)
            file_filter_List[i]->start();
        statusLabel->setText(tr("Filtering found files."));
    }
    else
    {
        searchFolder->setEnabled(true);
        stopSearch->setEnabled(false);
        ui.searchPathButton->setEnabled(true);
        ui.stopSearchButton->setEnabled(false);
        QMessageBox::information(this,tr("File Search"),
                                 tr("No files were found."));
        statusLabel->setText(tr("Album loaded."));
        animation->stopAnimation();
    }
}

void GS_albumer::fileFilterComp()
{
    if(filterComp == (file_filter_List.count()-1))
    {
        searchFolder->setEnabled(true);
        stopSearch->setEnabled(false);
        ui.searchPathButton->setEnabled(true);
        ui.stopSearchButton->setEnabled(false);
        animation->stopAnimation();
        QMap<QString,QFileInfo> inList_;
        for(int i = 0; i < file_filter_List.count(); ++i)
        {
            QMap<QString,QFileInfo> listLocal = file_filter_List[i]->getFileList();
            inList_ = inList_.unite(listLocal);
        }
        if(!inList_.isEmpty())
        {
            progress->show();
            progress->setMaxCount(inList_.count());
            progress->setCount(0);
            folder_inserter->setInList(inList_,man->getFolders());
            progress->hide();
            int x,y;
            x = pos().x() + ((width() - folder_inserter->width())/2);
            y = pos().y() + ((height() - folder_inserter->height())/2);
            folder_inserter->move(x,y);
            folder_inserter->show();
        }
        else
        {
            QMessageBox::information(this,tr("File Filter"),
                                     tr("No new files were found."));
        }
    }
    else
        ++filterComp;
    statusLabel->setText(tr("Album loaded."));
}

void GS_albumer::startColectionSearch()
{
    if(col_searcher->isHidden())
    {
        col_searcher->setAuthorList(man->getAuthors());
        int x,y;
        x = pos().x() + ((width() - col_searcher->width())/2);
        y = pos().y() + ((height()- col_searcher->height())/2);
        col_searcher->move(x,y);
        col_searcher->show();
    }
    else col_searcher->raise();
}

void GS_albumer::colectionSearchready(bool names, bool author, bool date)
{
    progress->show();
    progress->setMaxCount(0);
    QMap<QString,QList<int> > resoultMap;
    if(names)
    {
        if(author)
        {
            if(date)//names && author && date
            {
                QStringList names = col_searcher->getNames();
                progress->setMaxCount(names.count());
                QString author = col_searcher->getAuthor();
                QDate from = col_searcher->getFromDate();
                QDate to = col_searcher->getToDate();
                int count = 0;
                progress->setCount(count);
                foreach(const QString name, names)
                {
                    QString mapKey = tr("Contains: \"") + name + "\"" +" | "
                                     + tr("By: ")
                                     + author + " | "
                                     + from.toString("yyyy-MM-dd") + " | "
                                     + to.toString("yyyy-MM-dd");
                    QList<int> tempList = man->searchForPhotos(name,author,
                                                               from,to);
                    if(!tempList.empty())
                    {
                        resoultMap[mapKey] = tempList;
                    }
                    ++count;
                    progress->setCount(count);
                }
            }
            else//names && author
            {
                QStringList names = col_searcher->getNames();
                progress->setMaxCount(names.count());
                QString author = col_searcher->getAuthor();
                int count = 0;
                progress->setCount(count);
                foreach(const QString name, names)
                {
                    QString mapKey =tr("Contains: \"") + name + "\"" +
                                    + " | " +tr("By: ")
                                    + author;
                    QList<int> tempList = man->searchForPhotos(name,author);
                    if(!tempList.empty())
                    {
                        resoultMap[mapKey] = tempList;
                    }
                    ++count;
                    progress->setCount(count);
                }
            }
        }
        else
        {
            if(date)//names && date
            {
                QStringList names = col_searcher->getNames();
                progress->setMaxCount(names.count());
                QDate from = col_searcher->getFromDate();
                QDate to = col_searcher->getToDate();
                int count = 0;
                progress->setCount(count);
                foreach(const QString name, names)
                {
                    QString mapKey =tr("Contains: \"") + name + "\"" + " | "
                                    + from.toString("yyyy-MM-dd") + " | "
                                    + to.toString("yyyy-MM-dd");
                    QList<int> tempList = man->searchForPhotos(name,"",from,to);
                    if(!tempList.empty())
                    {
                        resoultMap[mapKey] = tempList;
                    }
                    ++count;
                    progress->setCount(count);
                }
            }
            else//names
            {
                QStringList names = col_searcher->getNames();
                progress->setMaxCount(names.count());
                int count = 0;
                progress->setCount(count);
                foreach(const QString name, names)
                {
                    QString mapKey =tr("Contains: \"") + name + "\"";
                    QList<int> tempList = man->searchForPhotos(name,"");
                    if(!tempList.empty())
                    {
                        resoultMap[mapKey] = tempList;
                    }
                    ++count;
                    progress->setCount(count);
                }
            }
        }
    }
    else
    {
        if(author)
        {
            if(date)//author && date
            {
                QString author = col_searcher->getAuthor();
                QDate from = col_searcher->getFromDate();
                QDate to = col_searcher->getToDate();
                QString mapKey =tr("By: ") + author + " | "
                                + from.toString("yyyy-MM-dd") + " | "
                                + to.toString("yyyy-MM-dd");
                QList<int> tempList = man->searchForPhotos("",author,from,to);
                if(!tempList.empty())
                {
                    resoultMap[mapKey] = tempList;
                }
            }
            else//author
            {
                QString author = col_searcher->getAuthor();
                QString mapKey =tr("By: ") + author;
                QList<int> tempList = man->searchForPhotos("",author);
                if(!tempList.empty())
                {
                    resoultMap[mapKey] = tempList;
                }
            }
        }
        else
        {
            if(date)//date
            {
                QDate from = col_searcher->getFromDate();
                QDate to = col_searcher->getToDate();
                QString mapKey =from.toString("yyyy-MM-dd") + " | "
                                + to.toString("yyyy-MM-dd");
                QList<int> tempList = man->searchForPhotos("","",from,to);
                if(!tempList.empty())
                {
                    resoultMap[mapKey] = tempList;
                }
            }
            else//none
            {
                QMessageBox::information(this,tr("Big error!"),
                                         tr("This should not be possible!"));
                return;
            }
        }
    }
    progress->hide();
    showSearchResault(resoultMap);
}

void GS_albumer::showSearchResault(QMap<QString,QList<int> > searchResault)
{
    if(searchResault.isEmpty())
    {
        QMessageBox::information(this,tr("No photos"),
                                 tr("No photos found"));
    }
    else
    {
        QList<GS_albumer_folder_view2*> tempVList;
        foreach(const QString title, searchResault.keys())
        {
            bool isOpen = false;
            foreach(GS_albumer_folder_view2* view, sview.values())
            {
                if(view->isItMe(title))
                {
                    folderTab->setCurrentWidget(view);
                    isOpen = true;
                }
            }
            if(!isOpen)
            {
                QStringList dateList;
                QStringList titleList;
                foreach(const int index, searchResault.value(title))
                {
                    QStringList temp = man->getPhotoInfo(index);
                    titleList<<temp.takeFirst();
                    dateList<<temp.takeAt(3);
                    temp.clear();
                }
                GS_albumer_folder_view2* temp = new GS_albumer_folder_view2(man);
                temp->setFolderImages(searchResault.value(title),
                                      titleList,dateList,title,title);
                sview[temp->myIndex()] = temp;
                tempVList<<temp;
                connect(sview[temp->myIndex()],SIGNAL(imageClicked(int)),this,
                        SLOT(showFolderImage(int)));
                connect(sview[temp->myIndex()],SIGNAL(wantToClose(int)),this,
                        SLOT(closeSearchView(int)));
                connect(sview[temp->myIndex()],SIGNAL(startSlideShow(QList<int>)),this,
                        SLOT(startSlideShow(QList<int>)));
                connect(sview[temp->myIndex()],SIGNAL(startMirror(QList<int>)),this,
                        SLOT(startMirrorView(QList<int>)));
                connect(sview[temp->myIndex()],SIGNAL(imageMovePicked(int,int)),this,
                        SLOT(fViewWantsToMoveItem(int,int)));
                connect(sview[temp->myIndex()],SIGNAL(imageRemovePicked(int,int)),this,
                        SLOT(fViewWantsToRemoveItem(int,int)));
                connect(sview[temp->myIndex()],SIGNAL(imageExportPicked(int)),this,
                        SLOT(exportImage(int)));
                connect(sview[temp->myIndex()],SIGNAL(imageEditDscPicked(int)),this,
                        SLOT(openEntryEdition(int)));
                connect(sview[temp->myIndex()],SIGNAL(groupRemovePicked(QList<int>,int)),
                        this,SLOT(fViewWantsToRemoveItems(QList<int>,int)));
                connect(sview[temp->myIndex()],SIGNAL(groupMovePicked(QList<int>,int)),
                        this,SLOT(fViewWantsToMoveItems(QList<int>,int)));
                connect(sview[temp->myIndex()],SIGNAL(groupExport(QList<int>)),this,
                        SLOT(exportImages(QList<int>)));
                connect(sview[temp->myIndex()],SIGNAL(addTags(QList<int>)),
                        this,SLOT(fViewAddTag(QList<int>)));
                connect(sview[temp->myIndex()],SIGNAL(imageInfoShowPicked(int)),this,
                        SLOT(showImageInfo(int)));
                connect(sview[temp->myIndex()],SIGNAL(removeTagPicked(int)),
                        this,SLOT(fViewRemoveImageTag(int)));
                connect(sview[temp->myIndex()],SIGNAL(renameImages(QList<int>)),
                        this,SLOT(fViewRenameImages(QList<int>)));
                hasWindows = true;
                folderTab->addTab(sview[temp->myIndex()],title);
                folderTab->setCurrentIndex(
                        folderTab->indexOf(sview[temp->myIndex()]));
            }
        }
        tempVList.clear();
    }
}

void GS_albumer::quickSearch(QString name)
{
    progress->show();
    progress->setMaxCount(0);
    QMap<QString,QList<int> > resoultMap;
    QString mapKey =tr("Contains: \"") + name +"\"";
    QList<int> tempList = man->searchForPhotos(name,"");
    if(!tempList.empty())
    {
        resoultMap[mapKey] = tempList;
    }
    showSearchResault(resoultMap);
    progress->hide();
}

void GS_albumer::tagSearch(QString tag)
{
    progress->show();
    progress->setMaxCount(0);
    QStringList tags = tag.split("|",QString::SkipEmptyParts);
    QList<int> resaultList;
    if(tags.count() > 0)
        resaultList = man->searchForPhotosByTag(tags.at(0));
    for(int i = 1; i < tags.count(); ++i)
    {
        QList<int> tempResaults = man->searchForPhotosByTag(tags.at(i));
        QList<int> notMatched;
        for(int j = 0; j < resaultList.count(); ++j)
        {
            if(!tempResaults.contains(resaultList.at(j)))
                notMatched << resaultList.at(j);
        }
        for(int j = 0; j < notMatched.count(); ++j)
            resaultList.removeAt(resaultList.indexOf(notMatched.at(j)));;
    }
    QMap<QString,QList<int> > resaultMap;
    QString mapKey =tr("Tags: \"") + tag +"\"";
    if(!resaultList.empty())
        resaultMap[mapKey] = resaultList;
    showSearchResault(resaultMap);
    progress->hide();
}

void GS_albumer::showAboutQt()
{
    QMessageBox::aboutQt(this);
}

void GS_albumer::showAbout()
{
    albumerInfo->show();
}

void GS_albumer::openEntryEdition(int id)
{
    QStringList photoInfo = man->getPhotoInfo(id);
    QPixmap mini = QPixmap::fromImage(man->getPhotoMini(id));
    entryEditor->setEditionData(id,mini,photoInfo.at(0),photoInfo.at(3));
    entryEditor->move(this->pos().x() + this->width()/2 - entryEditor->width()/2,
                      this->pos().y() + this->height()/2 - entryEditor->height()/2);
    entryEditor->show();
}

void GS_albumer::editDesc(int id,QString dsc)
{
    man->updateImageDescription(id,dsc);
}

void GS_albumer::photoChanged(int index)
{
    foreach(GS_albumer_folder_view2* view, fview.values())
        view->updateMiniature(index);

    foreach(GS_albumer_folder_view2* view, sview.values())
        view->updateMiniature(index);
}

void GS_albumer::newPhotoInserted(QString catalog, int index)
{
    foreach(GS_albumer_folder_view2* view, fview.values())
    {
        if(view->isItMe(catalog))
            view->addItem(index);
    }
    loadAlbum(false);
}

void GS_albumer::sqlDrivers()
{
    QStringList temp = QSqlDatabase::drivers();
    QString drivers = tr("The falowing drivers were found in your system:\n");
    foreach(const QString driver,temp)
    {
        drivers = drivers+"-"+driver +"\n";
    }
    QMessageBox::about(this,tr("SQLdrivers"),drivers);
}

void GS_albumer::showConfig()
{
    if(config->isHidden())
    {
        int x,y;
        x = pos().x() + (width() - config->width())/2;
        y = pos().y() + (height() - config->height())/2;
        config->move(x,y);
        config->show();
    }
    else config->raise();
}

void GS_albumer::updateViews()
{
    foreach(const GS_albumer_folder_view2* view, fview.values())
        view->updateTheme();
    foreach(const GS_albumer_folder_view2* view, sview.values())
        view->updateTheme();
    if(menuBar()->isHidden() && GS_config_holder::getShowMenu())
        menuBar()->show();
    else if(!menuBar()->isHidden() && !GS_config_holder::getShowMenu())
        menuBar()->hide();
}

void GS_albumer::startSlideShow(QList<int> photoList)
{
    slideShow->setPhotoList(photoList);
    slideShow->setPhoto(QPixmap());
    slideShow->showFullScreen();
    QPixmap photoForSlide = QPixmap::fromImage(man->getPhoto(photoList.first()));
    slideShow->setPhoto(photoForSlide);
}

void GS_albumer::slideShowNeedsPhoto(int index)
{
    QPixmap photoForSlide = QPixmap::fromImage(man->getPhoto(index));
    slideShow->setPhoto(photoForSlide);
}

void GS_albumer::startMirrorView(QList<int> photoList)
{
    if(mirrorView != NULL)
    {
        mirrorView->close();
        delete mirrorView;
    }
    mirrorView = new MirrorView();
    connect(mirrorView,SIGNAL(neededPhoto(int)),this,
            SLOT(mirrorViewNeedsPhoto(int)));
    connect(mirrorView,SIGNAL(hideMe()),this,SLOT(hideMirrorView()));
    mirrorView->setImageList(photoList);
    int imageCount = 7;
    if(imageCount > photoList.count())
        imageCount = photoList.count();
    progress->setCancel(false);
    progress->show();
    progress->setMaxCount(imageCount);
    for(int i = 0; i < imageCount; ++i)
    {
        progress->setCount(i);
        QPixmap photo = QPixmap::fromImage(man->getPhoto(photoList.at(i)));
        mirrorView->loadImage(photoList.at(i),photo);
    }
    progress->hide();
    mirrorView->showFullScreen();
}

void GS_albumer::mirrorViewNeedsPhoto(int index)
{
    QPixmap photo = QPixmap::fromImage(man->getPhoto(index));
    mirrorView->loadImage(index,photo);
}

void GS_albumer::hideMirrorView()
{
    mirrorView->close();
    delete mirrorView;
    mirrorView = NULL;
}

void GS_albumer::createActions()
{
    newAlbum = new QAction(QIcon(":/eyecandy/new.png"),tr("&New Album"),this);
    newAlbum->setStatusTip(tr("Create a new album."));
    ui.newAlbumButton->setToolTip(tr("Create a new album."));
    connect(newAlbum,SIGNAL(triggered()),this,SLOT(newAlbumSlot()));

    openAlbum = new QAction(QIcon(":/eyecandy/open.png"),
                            tr("&Open Album"),this);
    openAlbum->setStatusTip(tr("Open an existing album."));
    ui.openAlbumButton->setToolTip(tr("Open an existing album."));
    connect(openAlbum,SIGNAL(triggered()),this,SLOT(openAlbumSlot()));

    searchFolder = new QAction(QIcon(":/eyecandy/searchP.png"),
                               tr("&Search directory"),this);
    searchFolder->setStatusTip(tr("Search for photos in a specified dir"));
    ui.searchPathButton->setToolTip(tr("Search for photos in a specified dir"));
    connect(searchFolder,SIGNAL(triggered()),this,SLOT(startFileSearch()));

    stopSearch = new QAction(QIcon(":/eyecandy/stopSearchP.png"),
                             tr("&Stop directory search"),this);
    stopSearch->setStatusTip(tr("Stops the directory seach."));
    ui.stopSearchButton->setToolTip(tr("Stops the directory seach."));
    connect(stopSearch,SIGNAL(triggered()),this,SLOT(stopTheards()));

    searchColection = new QAction(QIcon(":/eyecandy/searchC.png"),
                                  tr("Search &colection"),this);
    searchColection->setStatusTip(tr("Search the colection for photos"));
    ui.searchColectionButton->setToolTip(tr("Search the colection for photos"));
    connect(searchColection,SIGNAL(triggered()),this,
            SLOT(startColectionSearch()));

    configure = new QAction(QIcon(":/eyecandy/config.png"),
                            tr("&Configure Aalbumer"),this);
    configure->setStatusTip(tr("Configures Albumer"));
    ui.configButton->setToolTip(tr("Configures Albumer"));
    connect(configure,SIGNAL(triggered()),this,SLOT(showConfig()));

    aboutQt = new QAction(tr("About&Qt"),this);
    aboutQt->setStatusTip(tr("Shows some information about Qt"));
    connect(aboutQt,SIGNAL(triggered()),this,SLOT(showAboutQt()));

    about = new QAction(tr("&About [GS] Albumer"),this);
    about->setStatusTip(tr("Shows some information about [GS] Albumer"));
    connect(about,SIGNAL(triggered()),this,SLOT(showAbout()));

    aboutSqlDrivers = new QAction(tr("&Sqldrivers"),this);
    aboutSqlDrivers->setStatusTip(tr
                                  ("Shows SQLdrivers installed in your system"));
    connect(aboutSqlDrivers,SIGNAL(triggered()),this,SLOT(sqlDrivers()));

    insImage = new QAction(QIcon(":/eyecandy/add.png"),tr("&Insert Image"),this);
    insImage->setStatusTip(tr("Insert a singele Image"));
    ui.addImageButton->setToolTip(tr("Insert a singele Image"));
    connect(insImage,SIGNAL(triggered()),this,SLOT(insertImage()));
    insImage->setEnabled(false);
}

void GS_albumer::createMenus()
{
    FileMenu = menuBar()->addMenu(tr("&File"));
    FileMenu->addAction(newAlbum);
    FileMenu->addAction(openAlbum);

    ToolsMenu =  menuBar()->addMenu(tr("&Tools"));
    ToolsMenu->addAction(stopSearch);
    ToolsMenu->addAction(searchFolder);
    ToolsMenu->addAction(searchColection);
    ToolsMenu->addSeparator();
    ToolsMenu->addAction(configure);

    HelpMenu = menuBar()->addMenu(tr("&Help"));
    HelpMenu->addAction(aboutSqlDrivers);
    HelpMenu->addSeparator();
    HelpMenu->addAction(aboutQt);
    HelpMenu->addAction(about);
}

void GS_albumer::createToolBars()
{
    qSearcher = new GS_albumer_quick_shearch();
    qSearcher->setEnabled(false);
    animation = new GS_albumer_work_animation();
    ui.quickLayout->addWidget(qSearcher);
    ui.bar->layout()->addWidget(animation);
    connect(ui.newAlbumButton,SIGNAL(clicked()),this,SLOT(newAlbumSlot()));
    connect(ui.openAlbumButton,SIGNAL(clicked()),this,SLOT(openAlbumSlot()));
    connect(ui.addImageButton,SIGNAL(clicked()),this,SLOT(insertImage()));
    connect(ui.stopSearchButton,SIGNAL(clicked()),this,SLOT(stopTheards()));
    connect(ui.searchPathButton,SIGNAL(clicked()),this,SLOT(startFileSearch()));
    connect(ui.searchColectionButton,SIGNAL(clicked()),
            this,SLOT(startColectionSearch()));
    connect(ui.configButton,SIGNAL(clicked()),this,SLOT(showConfig()));
}

GS_albumer::GS_albumer(QWidget * parent, Qt::WFlags f)
    :QMainWindow(parent,f)
{
    ui.setupUi(this);
    createActions();
    createMenus();
    createToolBars();
    stopSearch->setEnabled(false);
    searchFolder->setEnabled(false);
    searchColection->setEnabled(false);
    ui.searchPathButton->setEnabled(false);
    ui.stopSearchButton->setEnabled(false);
    ui.searchColectionButton->setEnabled(false);
    firstInsert = true;
    cancelF = false;
    hasDatabase = false;

    splitter = new QSplitter(Qt::Horizontal);

    folderTab = new QTabWidget();
    statusLabel = new QLabel(tr("No album loaded."));
    man = new ColectionManipulator();
    inserter = new GS_albumer_inserter();
    folder_inserter = new GS_albumer_folder_inserter();
    informator = new GS_albumer_info();
    progress =  new GS_albumer_progress();
    QStringList filterList;
    filterList << "*.png"<<"*.jpeg"<<"*.jpg"<<"*.tiff";
    file_searcher = new GS_file_search(filterList);
    QMutex* mutex = new QMutex();
    for(int i = 0; i < QThread::idealThreadCount(); ++i)
        file_filter_List << new GS_file_filter(mutex,this);
    col_searcher = new GS_albumer_searcher();
    timer = new QTimer(this);
    timer->setSingleShot(true);
    timer->setInterval(15);
    donationTimer = new QTimer(this);
    donationTimer->setSingleShot(true);
    donationTimer->setInterval(50);

    config = new GS_albumer_configurator();
    //configurable options------------------------------------------------------
    if(!GS_config_holder::getShowMenu())
        menuBar()->hide();
    ui.widget->setFolded(GS_config_holder::getCollFile());
    ui.widget_2->setFolded(GS_config_holder::getCollSearch());
    ui.widget_3->setFolded(GS_config_holder::getCollConfig());
    //--------------------------------------------------------------------------
    slider = new GS_albumer_slider();
    welcomeWidget = new WelcomInformation();
    welcomeWidget->setTopWidget(slider);
    folderTab->addTab(welcomeWidget,QIcon(),QString(tr("Welcome")));
    slideShow = new GS_albumer_slideShow();
    passer =  new GS_albumer_passer();
    albCreator = new GS_albumer_album_creator();
    imageMover = new GS_albumer_image_mover();
    albumerInfo = new GS_albumer_information();
    mirrorView = NULL;
    catalogSelector = new CategorySelector();
    catalogSelector->setTitle(tr("Catalogs"));
    tagSelector = new CategorySelector();
    tagSelector->setTitle(tr("Tags"));
    entryEditor = new GS_albumer_entry_editor();

    splitter->addWidget(catalogSelector);
    splitter->addWidget(folderTab);
    splitter->addWidget(tagSelector);

    ui.layout->addWidget(splitter);

    QList<int> sizeList;
    sizeList << 140 << this->width() - 300 << 140;
    splitter->setSizes(sizeList);
    splitter->setCollapsible(1,false);

    //styling the statusbar
    ui.statusbar->setStyleSheet(QString("QStatusBar{ background-color:")+
                                QString("qlineargradient(spread:pad, x1:0, y1:0, x2:0,y2:1,")+
                                QString("stop:0 rgba(235, 235, 235, 255),")+
                                QString("stop:0.1 rgba(227, 227, 227, 255),")+
                                QString("stop:0.5 rgba(237, 237, 237, 255),")+
                                QString("stop:0.51 rgba(197, 197, 197, 255),")+
                                QString("stop:1 rgba(235, 235, 235, 255))}"));
    //--------------------------------------------------------------------------

    //styling menu
    menuBar()->setStyleSheet(QString("QMenuBar{ background-color:")+
                             QString("qlineargradient(spread:pad, x1:0, y1:0, x2:0,y2:1,")+
                             QString("stop:0 rgba(235, 235, 235, 255),")+
                             QString("stop:0.1 rgba(227, 227, 227, 255),")+
                             QString("stop:0.5 rgba(237, 237, 237, 255),")+
                             QString("stop:0.51 rgba(197, 197, 197, 255),")+
                             QString("stop:1 rgba(235, 235, 235, 255))}")+
                             QString("QMenuBar::item {padding: 1px 10px 1px 10px;")+
                             QString("border: 1px solid transparent;")+
                             QString("border-radius: 0px;}")+
                             QString("QMenuBar::item:selected {")+
                             QString("border-color: black;")+
                             QString("background: transparent;}")+
                             QString("QMenu::item {padding: 2px 10px 2px 25px;")+
                             QString("border: 1px solid transparent;")+
                             QString("border-radius: 0px;}")+
                             QString("QMenu::item:selected {")+
                             QString("border-color: black;")+
                             QString("background-color:")+
                             QString("qlineargradient(spread:pad, x1:0, y1:0, x2:0,y2:1,")+
                             QString("stop:0 rgba(235, 235, 235, 255),")+
                             QString("stop:0.1 rgba(227, 227, 227, 255),")+
                             QString("stop:0.5 rgba(237, 237, 237, 255),")+
                             QString("stop:0.51 rgba(197, 197, 197, 255),")+
                             QString("stop:1 rgba(235, 235, 235, 255))}"));
    //--------------------------------------------------------------------------

    this->totalPhotos = 0;
    this->folderPhotos = 0;
    this->statusBar()->addPermanentWidget(statusLabel);
    this->setContextMenuPolicy(Qt::NoContextMenu);

    hasWindows = false;
    setWindowIcon(QIcon(QPixmap(":/eyecandy/Icon_logo.png")));

    connect(albCreator,SIGNAL(newAlbumReady()),this,SLOT(createNewAlbum()));
    connect(inserter,SIGNAL(readyForInserting()),this,SLOT(insertReady()));
    connect(file_searcher,SIGNAL(finished()),this,SLOT(fileSearchComp()));
    for(int i = 0; i < file_filter_List.count(); ++i)
        connect(file_filter_List.at(i),SIGNAL(filterFinished()),this,SLOT(fileFilterComp()));
    connect(folder_inserter,SIGNAL(insertReady()),this,SLOT(finsertReady()));
    connect(col_searcher,SIGNAL(searchReady(bool,bool,bool)),this,
            SLOT(colectionSearchready(bool,bool,bool)));
    connect(timer,SIGNAL(timeout()),this,SLOT(finsertReady()));
    connect(folder_inserter,SIGNAL(photoProcesed(int)),progress,
            SLOT(setCount(int)));
    connect(config,SIGNAL(configUpdated()),this,SLOT(updateViews()));
    connect(qSearcher,SIGNAL(startQuickSearch(QString)),this,
            SLOT(quickSearch(QString)));
    connect(qSearcher,SIGNAL(startTagSearch(QString)),this,
            SLOT(tagSearch(QString)));
    connect(slideShow,SIGNAL(giveMeThisPhoto(int)),this,
            SLOT(slideShowNeedsPhoto(int)));
    connect(passer,SIGNAL(enteredPass(bool)),this,
            SLOT(openValidationCompelat(bool)));
    connect(imageMover,SIGNAL(catalogPicked(QList<int>,int)),this,
            SLOT(catalogPicked(QList<int>,int)));
    connect(catalogSelector,SIGNAL(itemTouchedSignal(int)),this,
            SLOT(folderTouched(int)));
    connect(catalogSelector,SIGNAL(itemClickedSignal(int)),this,
            SLOT(showImagesFromFolder(int)));
    connect(catalogSelector,SIGNAL(photoDroped(int,int)),
            this,SLOT(dropActionMoveImage(int,int)),Qt::QueuedConnection);
    connect(tagSelector,SIGNAL(itemTouchedSignal(int)),this,
            SLOT(tagTouched(int)));
    connect(tagSelector,SIGNAL(itemClickedSignal(int)),this,
            SLOT(showImagesWithTag(int)));
    connect(tagSelector,SIGNAL(photoDroped(int,int)),
            this,SLOT(dropActionTagImage(int,int)),Qt::QueuedConnection);
    connect(entryEditor,SIGNAL(editEntry(int,QString)),this,
            SLOT(editDesc(int,QString)));
    connect(progress,SIGNAL(cancelClicked()),this,SLOT(cancelFinsert()));

    slideShow->showFullScreen();
    slideShow->hide();
    if(GS_config_holder::getStartMaxi())
        this->showMaximized();
    QStringList args = QApplication::arguments();
    if(args.count() > 1)
    {
        filename = args.at(1);
        if(!QFile::exists(filename + QString("P")))
        {
            QMessageBox::information (this,tr("File error")
                                      ,QString(tr("This album is missing the \n%1P file!\nThis might be an old/unsupported album format.").arg(filename)));
        }
        else if(!QFile::exists(filename + QString("M")))
        {
            QMessageBox::information (this,tr("File error")
                                      ,QString(tr("This album is missing the \n%1M file!\nThis might be an old/unsupported album format.").arg(filename)));
        }
        else
        {
            progress->setCancel(false);
            progress->setMaxCount(2);
            progress->show();
            progress->setCount(0);
            progress->setCount(1);
            man->openDataBase(filename,GS_config_holder::getCheckBase());
            progress->setCount(2);
            progress->hide();
            QString passwd = man->getPassWD();
            if(passwd.trimmed() == "")
            {
                openValidationCompelat(true);
            }
            else
            {
                passer->setPassWord(passwd);
                passer->show();
            }
        }
    }
}

GS_albumer::~GS_albumer()
{
    views.clear();
    fview.clear();
    sview.clear();
    delete entryEditor;
    delete albumerInfo;
    delete imageMover;
    delete albCreator;
    delete passer;
    delete slideShow;
    delete config;
    delete col_searcher;
    file_searcher->exit(0);
    for(int i = 0; i < file_filter_List.count(); ++i)
        file_filter_List[i]->exit(0);
    file_filter_List.clear();
    delete file_searcher;
    delete progress;
    delete informator;
    delete folder_inserter;
    delete inserter;
    delete man;
}
