#ifndef Gsalb
#define Gsalb
#include "ui_GS_albumer.h"
#include <QApplication>
#include <QMainWindow>
#include <QTextCodec>
#include <QMap>
#include <QStringList>
#include <QMenuBar>
#include <QFileDialog>
#include <QMessageBox>
#include <QCloseEvent>
#include <QCryptographicHash>
#include <QFile>
#include <QFileInfo>
#include <QSqlDatabase>
#include <QTimer>
#include <QToolBar>
#include <QDesktopWidget>
#include <QDesktopServices>
#include <QPixmap>
#include <QSplitter>
#include <QTabWidget>
#include "Colection/colectionManipulator.h"
#include "Inserters/GS_albumer_inserter.h"
#include "Information/GS_albumer_info.h"
#include "Views/GS_albumer_viewer.h"
#include "Views/gs_albumer_folder_view2.h"
#include "Misc/GS_albumer_progress.h"
#include "Tools/GS_file_search.h"
#include "Tools/GS_file_filter.h"
#include "Inserters/gs_albumer_folder_inserter.h"
#include "Tools/gs_albumer_searcher.h"
#include "Misc/gs_albumer_work_animation.h"
#include "Tools/gs_albumer_configurator.h"
#include "Misc/gs_albumer_slider.h"
#include "Tools/gs_albumer_quick_shearch.h"
#include "Views/gs_albumer_slideshow.h"
#include "Tools/gs_albumer_passer.h"
#include "Tools/gs_albumer_album_creator.h"
#include "Tools/gs_albumer_image_mover.h"
#include "Information/gs_albumer_information.h"
#include "Tools/gs_hash_holder.h"
#include "Selectors/categoryselector.h"
#include "Information/folderinfo.h"
#include "Colection/ImageData.h"
#include "Views/mirrorview.h"
#include "Tools/gs_albumer_entry_editor.h"
#include "Information/welcominformation.h"
#include "Tools/TagEditor.h"
#include "Tools/TagRemover.h"
#include "Tools/NameEditor.h"

using namespace Ui;

class GS_albumer: public QMainWindow, protected GS_hash_holder
{
    Q_OBJECT
public:
    GS_albumer(QWidget * parent= 0, Qt::WFlags f = 0);
    ~GS_albumer();

protected:
    void closeEvent(QCloseEvent* event);

private slots:
    void newAlbumSlot();
    void createNewAlbum();
    void openAlbumSlot();
    void openValidationCompelat(bool);

    void insertImage();
    void folderTouched(int folderIndex);
    void showImagesFromFolder(int folderIndex);
    void tagTouched(int tagIndex);
    void showImagesWithTag(int tagIndex);
    void showImageInfo(int index);

    void closeImage(int index);
    void closeFolderView(int index);
    void closeSearchView(int index);

    void insertReady();
    void finsertReady();
    void cancelFinsert();

    void showFolderImage(int index);

    void startFileSearch();
    void fileSearchComp();
    void fileFilterComp();
    void stopTheards();

    void startColectionSearch();
    void colectionSearchready(bool names, bool author, bool date);
    void quickSearch(QString name);
    void tagSearch(QString tag);
    void showSearchResault(QMap<QString,QList<int> >);

    void showAboutQt();
    void showAbout();
    void sqlDrivers();
    void showConfig();

    void updateViews();

    void startSlideShow(QList<int>);
    void slideShowNeedsPhoto(int);
    void startMirrorView(QList<int>);
    void mirrorViewNeedsPhoto(int);
    void hideMirrorView();

    void fViewWantsToMoveItem(int,int);
    void fViewWantsToMoveItems(QList<int>,int);
    void dropActionMoveImage(int imageId, int pickerItemId);
    void dropActionTagImage(int imageId, int pickedItemId);
    void catalogPicked(QList<int>,int);
    void fViewWantsToRemoveItem(int,int);
    void fViewWantsToRemoveItems(QList<int>,int);
    void resetView(int,QString);
    void resetTagView(int,QString);
    void fViewAddTag(QList<int>);
    void fViewRemoveImageTag(int id);
    void fViewRenameImages(QList<int>);

    void exportImage(int);
    void exportImages(QList<int>);

    void openEntryEdition(int id);
    void editDesc(int id,QString dsc);
    void addTag(QString tag);
    void removeTag(int id,QString tag);
    void renameImages(QString newName);

    void photoChanged(int index);
    void newPhotoInserted(QString catalog, int index);

signals:
    void cancelClicked();
    void insertImages(QList<ImageData>);

private:
    void openImage(int index);
    void createActions();
    void createMenus();
    void createToolBars();
    void loadAlbum(bool newalbum = true);
    void updateFolderData();

    QAction* newAlbum;
    QAction* openAlbum;
    QAction* stopSearch;
    QAction* searchFolder;
    QAction* searchColection;
    QAction* aboutQt;
    QAction* about;
    QAction* aboutSqlDrivers;
    QAction* configure;
    QAction* insImage;

    QMenu* FileMenu;
    QMenu* ToolsMenu;
    QMenu* HelpMenu;

    Ui_albumer ui;

    QMap<int,GS_albumer_viewer*> views;
    QMap<int,GS_albumer_folder_view2*> fview;
    QMap<int,GS_albumer_folder_view2*> sview;
    QList<ImageData> finsertList;
    QMap<int,FolderInfo*> folderMap;
    QMap<int,FolderInfo*> tagMap;

    ColectionManipulator* man;
    GS_albumer_inserter* inserter;
    GS_albumer_info* informator;
    GS_albumer_progress* progress;
    GS_file_search* file_searcher;
    QList<GS_file_filter*> file_filter_List;
    GS_albumer_folder_inserter* folder_inserter;
    GS_albumer_searcher* col_searcher;
    GS_albumer_work_animation* animation;
    GS_albumer_configurator* config;
    GS_albumer_slider* slider;
    GS_albumer_quick_shearch* qSearcher;
    GS_albumer_slideShow* slideShow;
    GS_albumer_passer* passer;
    GS_albumer_album_creator* albCreator;
    GS_albumer_image_mover* imageMover;
    GS_albumer_information* albumerInfo;
    MirrorView* mirrorView;
    CategorySelector* catalogSelector;
    CategorySelector* tagSelector;
    GS_albumer_entry_editor* entryEditor;
    WelcomInformation* welcomeWidget;

    bool firstInsert;
    bool cancelF;
    bool hasWindows, hasDatabase;
    int totalPhotos, folderPhotos, progresCounter, filterComp;

    QLabel* statusLabel;
    QTimer* timer;
    QTimer* donationTimer;
    QToolBar* toolBar;
    QTabWidget* folderTab;
    QSplitter* splitter;

    QString filename;
    QStringList alteredFolders;
    QList<int> editionList;
};
#endif
