#include "FolderInserterPhotoItem.h"

void FolderInserterPhotoItem::setMini(QPixmap mini)
{
    hasMini = true;
    myMini = mini;
    update();
}

QRectF FolderInserterPhotoItem::boundingRect() const
{
    return QRectF(0.0f,0.0f,140.0f,180.0f);
}

int FolderInserterPhotoItem::width() const
{
    return 140;
}

int FolderInserterPhotoItem::height() const
{
    return 180;
}

void FolderInserterPhotoItem::paint(QPainter *painter,
                                    const QStyleOptionGraphicsItem *option,
                                    QWidget *widget)
{
    Q_UNUSED(widget);

    painter->save();
    painter->setPen(Qt::DashLine);
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setBrush(option->palette.brush(QPalette::AlternateBase));
    painter->drawRoundedRect(0,0,140,180,15.0f,15.0f);

    painter->setPen(Qt::black);

    if(hasMini)
        painter->drawPixmap(70 - (myMini.width()/2),
                            70 - (myMini.height()/2),
                            myMini);
    else
        painter->drawPixmap(6,6,*defaultNoMiniPixmap);

    painter->setFont(QFont("Courier",8,8));
    QString drawnText;
    if(myName.length() > 50)
    {
        drawnText = myName.left(47) + "...";
    }
    else
        drawnText = myName;

    painter->drawText(QRectF(7.0f,135.0f,126.0f,42.0f),
                      Qt::AlignHCenter|Qt::AlignVCenter|Qt::TextWrapAnywhere,
                      drawnText,
                      new QRectF(0.0f,0.0f,120.0f,42.0f));
    painter->restore();
}

void FolderInserterPhotoItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    if(event->button() != Qt::LeftButton)
    {
        event->ignore();
        return;
    }
    setCursor(Qt::ClosedHandCursor);
}

void FolderInserterPhotoItem::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
    if (QLineF(event->screenPos(), event->buttonDownScreenPos(Qt::LeftButton))
        .length() < QApplication::startDragDistance())
        return;

    QDrag *drag = new QDrag(event->widget());
    QMimeData *mime = new QMimeData;
    drag->setMimeData(mime);
    mime->setText(myName);
    if(hasMini)
    {
        drag->setPixmap(myMini);
        drag->setHotSpot(QPoint(myMini.width()/2,myMini.height()/2));
    }
    else
    {
        drag->setPixmap(*defaultNoMiniPixmap);
        drag->setHotSpot(QPoint(64,64));
    }
    drag->exec();
    setCursor(Qt::OpenHandCursor);
}

void FolderInserterPhotoItem::mouseReleaseEvent(QGraphicsSceneMouseEvent*)
{
    setCursor(Qt::OpenHandCursor);
}

FolderInserterPhotoItem::FolderInserterPhotoItem(QPixmap* noMiniPixmap,QString name)
        :QGraphicsItem()
{
    defaultNoMiniPixmap = noMiniPixmap;
    myName = name;
    hasMini = false;
    setCursor(Qt::OpenHandCursor);
}
