#include "GS_albumer_inserter.h"

void GS_albumer_inserter::openFile()
{
    filename = "";
    filename = QFileDialog::getOpenFileName(this,tr("Select Photo"),QDir::homePath(),
                                            tr("Images(*.png *.jpeg *.jpg *.tiff)"));
    if(filename == "") return;
    ui.label->setPixmap(QPixmap(filename).scaled(120,120,Qt::KeepAspectRatio));
    if(filename.endsWith("png",Qt::CaseInsensitive))
    {
        format = "PNG";
        return;
    }
    if(filename.endsWith("jpg",Qt::CaseInsensitive))
    {
        format = "JPG";
        return;
    }
    if(filename.endsWith("jpeg",Qt::CaseInsensitive))
    {
        format = "JPEG";
        return;
    }
    if(filename.endsWith("TIFF",Qt::CaseInsensitive))
    {
        format = "TIFF";
        return;
    }
    filename = "";
    ui.label->setText("Photo Preview");
}

void GS_albumer_inserter::clearData()
{
    filename="";
    ui.nameLine->setText("");
    ui.katalogBox->setCurrentIndex(-1);
    ui.autorLine->setText("");
    ui.opisLine->setText("");
    ui.label->setText("Photo Preview");
    ui.dateEdit->setDate(QDate::currentDate ());
}

void GS_albumer_inserter::createCatalog()
{
    bool ok;
    QString temp = QInputDialog::getText(this, tr("Catalog creation"),
                                         tr("New Catalog name:"), QLineEdit::Normal,
                                         "", &ok);
    temp.remove("/");
    temp.remove("\\");
    if(ok&&temp.trimmed()!="")
    {
        if(ui.katalogBox->findText(temp)==-1)
        {
            if(temp.length()>30)
            {
                QMessageBox::information (this,tr("Catalog error"),tr(
                    "The given catalog name is to long. Max lenght 30."
                    ));
                createCatalog();
                return;
            }
            ui.katalogBox->addItem(temp);
            ui.katalogBox->setCurrentIndex(ui.katalogBox->findText(temp));
        }
        else
        {
            QMessageBox::information (this,tr("Catalog exists"),tr(
                    "A catalog with the given name already exists.\nIt will be selected"
                    ));
            ui.katalogBox->setCurrentIndex(ui.katalogBox->findText(temp));
        }
    }
}

void GS_albumer_inserter::insertClicked()
{
    if(filename.trimmed() == "")
    {
        QMessageBox::information (this,tr("No Photo to insert")
                                  ,tr("Please select a Photo to insert."));
        return;
    }
    if(ui.nameLine->text().trimmed()=="")
    {
        QMessageBox::information (this,tr("No Name specified")
                                  ,tr("Please type in a Name."));
        return;
    }
    if(ui.katalogBox->currentIndex()==-1)
    {
        QMessageBox::information (this,tr("No Catalog selected")
                                  ,tr("Please select or create a Catalog."));
        return;
    }
    if(ui.autorLine->text().trimmed()=="")
        ui.autorLine->setText(tr("NONE"));
    if(ui.dateEdit->date()>QDate::currentDate())
    {
        QMessageBox::information (this,tr("Wrong date")
                                  ,tr("Invalid date."));
        return;
    }
    emit readyForInserting();
    hide();
}

void GS_albumer_inserter::clearCatalogs()
{
    if(ui.katalogBox->count()==0) return;
    else
    {
        for(int i=0;i<ui.katalogBox->count();i++)
            ui.katalogBox->removeItem(i);
    }
}

void GS_albumer_inserter::populateCatalogs(QStringList list)
{
    ui.katalogBox->addItems(list);
    ui.katalogBox->setCurrentIndex(-1);
}

void GS_albumer_inserter::closeEvent(QCloseEvent* event)
{
    if(needToClose)
    {
        event->accept();
    }
    else
    {
        event->ignore();
        hide();
    }
}

void GS_albumer_inserter::setNeedToClose()
{
    needToClose = true;
}

QString GS_albumer_inserter::getFile()
{
    return filename.trimmed();
}

QString GS_albumer_inserter::getName()
{
    return ui.nameLine->text().trimmed();
}

QString GS_albumer_inserter::getCatalog()
{
    return ui.katalogBox->currentText().trimmed();
}

QDate GS_albumer_inserter::getDate()
{
    return ui.dateEdit->date();
}

QString GS_albumer_inserter::getAuthor()
{
    return ui.autorLine->text().trimmed();
}

QString GS_albumer_inserter::getDisc()
{
    if(ui.opisLine->toPlainText()=="")
        return QString("NONE");
    else
        return ui.opisLine->toPlainText();
}

QString GS_albumer_inserter::getFormat()
{
    return format;
}

void GS_albumer_inserter::descTextChanged()
{
    if(ui.opisLine->toPlainText().length() > 80)
    {
        QString text = ui.opisLine->toPlainText();
        text.resize(80);
        ui.opisLine->document()->clear();
        ui.opisLine->insertPlainText(text);
        ui.opisLine->textCursor().movePosition(QTextCursor::End);
    }
}

GS_albumer_inserter::GS_albumer_inserter(QWidget* parent)
    :QWidget(parent)
{
    ui.setupUi(this);
    ui.dateEdit->setDate(QDate::currentDate ());
    needToClose = false;
    connect(ui.openPButton,SIGNAL(clicked()),this,SLOT(openFile()));
    connect(ui.goButton,SIGNAL(clicked()),this,SLOT(insertClicked()));
    connect(ui.newKatalogButton,SIGNAL(clicked()),this,SLOT(createCatalog()));
    connect(ui.opisLine,SIGNAL(textChanged()),this,SLOT(descTextChanged()));
}
