#include "folderinserterdragphotolist.h"

void FolderInserterDragPhotoList::addImageItem(QString name)
{
    if(itemMap.contains(name)) return;
    FolderInserterPhotoItem* item = new FolderInserterPhotoItem(defaultNoMini,
                                                                name);
    itemMap[name] = item;
    noMiniatureItems<<item;
    scrollValue = ui.photoView->verticalScrollBar()->value();
    makeScene();
    ui.photoView->verticalScrollBar()->setValue(scrollValue);
    updateMiniatures();
}

void FolderInserterDragPhotoList::addItems(QStringList itemList)
{
    clear();
    qSort(itemList);
    for(int i = 0; i < itemList.count(); ++i)
    {
        FolderInserterPhotoItem* item = new FolderInserterPhotoItem(defaultNoMini,
                                                                    itemList.at(i));
        itemMap[itemList.at(i)] = item;
        noMiniatureItems << item;
    }
    makeScene();
    updateMiniatures();
}

void FolderInserterDragPhotoList::setItemMiniature(QString name,QPixmap mini)
{
    if(itemMap.contains(name))
    {
        itemMap[name]->setMini(mini);
        noMiniatureItems.removeAt(noMiniatureItems.indexOf(itemMap[name]));
    }
    updateMiniatures();
}

void FolderInserterDragPhotoList::removeItem(QString name)
{
    noMiniatureItems.removeAt(noMiniatureItems.indexOf(itemMap[name]));
    ui.photoView->scene()->removeItem(itemMap[name]);
    itemMap.remove(name);
    scrollValue = ui.photoView->verticalScrollBar()->value();
    makeScene();
    ui.photoView->verticalScrollBar()->setValue(scrollValue);
}

void FolderInserterDragPhotoList::updateMiniatures()
{
    singleShotRequestTimer->start();
}

void FolderInserterDragPhotoList::clear()
{
    noMiniatureItems.clear();
    itemMap.clear();
    singleShotRequestTimer->stop();
    ui.photoView->setScene(new QGraphicsScene(0,0,1,1));
    scrollValue = 0;
}

void FolderInserterDragPhotoList::hideEvent(QHideEvent*)
{
    scrollValue = ui.photoView->verticalScrollBar()->value();
}

void FolderInserterDragPhotoList::showEvent(QShowEvent*)
{
     ui.photoView->verticalScrollBar()->setValue(scrollValue);
}

void FolderInserterDragPhotoList::requestMiniature()
{
    if(noMiniatureItems.count() > 0)
        emit requestMiniature(itemMap.key(noMiniatureItems.first()));
}

void FolderInserterDragPhotoList::makeScene()
{
    if(itemMap.count() > 0)
    {
        int sceneWidth = ui.photoView->width() - 20;
        if(sceneWidth < 200) sceneWidth = 200;
        QStringList itemNameList = itemMap.keys();
        qSort(itemNameList);
        int itemWidth = itemMap.values().first()->width() + 5;
        int itemHeight = itemMap.values().first()->height() + 5;
        int itemsInLine = (sceneWidth)/itemWidth;
        int lines = itemMap.count() / itemsInLine;
        sceneWidth = (itemsInLine * itemWidth);
        if(itemMap.count() % itemsInLine != 0) ++lines;

        QGraphicsScene* scene = new QGraphicsScene(0,0,sceneWidth,
                                                   lines * itemHeight);
        int lineIndex = 0;
        int rowIndex = 0;
        for(int i = 0; i < itemNameList.count(); ++i)
        {
            itemMap[itemNameList.at(i)]->setPos(lineIndex * itemWidth,
                                                rowIndex * itemHeight);
            scene->addItem(itemMap[itemNameList.at(i)]);
            ++lineIndex;
            if(lineIndex >= itemsInLine)
            {
                lineIndex = 0;
                ++rowIndex;
            }
        }

        QGraphicsScene* temp = ui.photoView->scene();
        ui.photoView->setScene(scene);
        delete temp;
    }
}

void FolderInserterDragPhotoList::resizeEvent(QResizeEvent*)
{
    scrollValue = ui.photoView->verticalScrollBar()->value();
    makeScene();
    ui.photoView->verticalScrollBar()->setValue(scrollValue);
}

FolderInserterDragPhotoList::FolderInserterDragPhotoList(QWidget *parent)
        :QWidget(parent)
{
    ui.setupUi(this);
    ui.photoView->setScene(new QGraphicsScene(0,0,1,1));
    scrollValue = 0;
    defaultNoMini = new QPixmap(":/eyecandy/noMini.png");
    singleShotRequestTimer = new QTimer(this);
    singleShotRequestTimer->setInterval(25);
    singleShotRequestTimer->setSingleShot(true);
    connect(singleShotRequestTimer,SIGNAL(timeout()),
            this,SLOT(requestMiniature()));
}

FolderInserterDragPhotoList::~FolderInserterDragPhotoList()
{
    delete defaultNoMini;
}
