#include "folderinserterfolderview.h"

void FolderInserterFolderView::addItem(QString name)
{
    if(itemMap.contains(name))
        return;
    FolderInserterFolderItem* item = new FolderInserterFolderItem(name);
    itemMap[name] = item;
    connect(item,SIGNAL(movePhoto(QString,QString)),
            this,SLOT(dropAction(QString,QString)));
    makeScene();
}

void FolderInserterFolderView::addItems(QStringList itemList)
{
    for(int i = 0; i < itemList.count(); ++i)
    {
        if(!itemMap.contains(itemList.at(i)))
        {
            FolderInserterFolderItem* item = new FolderInserterFolderItem(
                    itemList.at(i));
            itemMap[itemList.at(i)] = item;
            connect(item,SIGNAL(movePhoto(QString,QString)),
                    this,SLOT(dropAction(QString,QString)));
        }
    }
    makeScene();
}

void FolderInserterFolderView::clear()
{
    itemMap.clear();
    ui.folderView->setScene(new QGraphicsScene(0,0,1,1));
    scrollValue = 0;
}

void FolderInserterFolderView::resizeEvent(QResizeEvent*)
{
    scrollValue = ui.folderView->verticalScrollBar()->value();
    makeScene();
    ui.folderView->verticalScrollBar()->setValue(scrollValue);
}

void FolderInserterFolderView::hideEvent(QHideEvent*)
{
    scrollValue = ui.folderView->verticalScrollBar()->value();
}

void FolderInserterFolderView::showEvent(QShowEvent*)
{
    ui.folderView->verticalScrollBar()->setValue(scrollValue);
}

void FolderInserterFolderView::dropAction(QString imageName,QString folderName)
{
    emit movePhoto(imageName,folderName);
}

void FolderInserterFolderView::dragMove(int y)
{
    if(y <= mapToGlobal(ui.folderView->rect().topLeft()).y() + 50)
    {
        scrollDownTimer->stop();
        scrollUpTimer->start(50);
        return;
    }
    else if(y >= mapToGlobal(ui.folderView->rect().bottomLeft()).y() - 50)
    {
       scrollUpTimer->stop();
       scrollDownTimer->start(50);
       return;
    }
    scrollUpTimer->stop();
    scrollDownTimer->stop();
}

void FolderInserterFolderView::dragEnd()
{
    scrollDownTimer->stop();
    scrollUpTimer->stop();
}

void FolderInserterFolderView::scrollUp()
{
    int currValye = ui.folderView->verticalScrollBar()->value()-10;
    ui.folderView->verticalScrollBar()->setValue(currValye);
}

void FolderInserterFolderView::scrollDown()
{
    int currValye = ui.folderView->verticalScrollBar()->value()+10;
    ui.folderView->verticalScrollBar()->setValue(currValye);
}

void FolderInserterFolderView::makeScene()
{
    if(itemMap.count() > 0)
    {
        int sceneWidth = ui.folderView->width() - 20;
        if(sceneWidth < 200) sceneWidth = 200;
        QStringList itemNameList = itemMap.keys();
        qSort(itemNameList);
        int itemWidth = itemMap.values().first()->width() + 5;
        int itemHeight = itemMap.values().first()->height() + 5;
        int itemsInLine = (sceneWidth)/itemWidth;
        int lines = itemMap.count() / itemsInLine;
        sceneWidth = (itemsInLine * itemWidth);
        if(itemMap.count() % itemsInLine != 0) ++lines;

        FolderScene* scene = new FolderScene(0,0,sceneWidth,
                                                lines * itemHeight);
        int lineIndex = 0;
        int rowIndex = 0;
        for(int i = 0; i < itemNameList.count(); ++i)
        {
            itemMap[itemNameList.at(i)]->setPos(lineIndex * itemWidth,
                                                rowIndex * itemHeight);
            scene->addItem(itemMap[itemNameList.at(i)]);
            ++lineIndex;
            if(lineIndex >= itemsInLine)
            {
                lineIndex = 0;
                ++rowIndex;
            }
        }

        connect(scene,SIGNAL(dragAtPos(int)),this,SLOT(dragMove(int)));
        connect(scene,SIGNAL(dragFinished()),this,SLOT(dragEnd()));

        QGraphicsScene* temp = ui.folderView->scene();
        ui.folderView->setScene(scene);
        delete temp;
    }
}

FolderInserterFolderView::FolderInserterFolderView(QWidget *parent)
        :QWidget(parent)
{
    ui.setupUi(this);
    ui.folderView->setScene(new QGraphicsScene(0,0,1,1));
    scrollValue = 0;
    this->setMouseTracking(true);
    scrollDownTimer = new QTimer(this);
    scrollUpTimer = new QTimer(this);
    connect(scrollDownTimer,SIGNAL(timeout()),this,SLOT(scrollDown()));
    connect(scrollUpTimer,SIGNAL(timeout()),this,SLOT(scrollUp()));
}

FolderInserterFolderView::~FolderInserterFolderView()
{}
