#include "gs_albumer_folder_inserter.h"

void GS_albumer_folder_inserter::closeEvent(QCloseEvent* event)
{
    if(needToClose)
    {
        event->accept();
    }
    else
    {
        event->ignore();
        photoListWidget->clear();
        folderListWidget->clear();
        hide();
    }
}

void GS_albumer_folder_inserter::hideEvent(QHideEvent*)
{
}

void GS_albumer_folder_inserter::discardClicked()
{
    outList.clear();
    inList.clear();
    fileInfoList.clear();
    close();
}

void GS_albumer_folder_inserter::setNeedToClose()
{
    this->needToClose = true;
}

void GS_albumer_folder_inserter::setInList(QMap<QString,QFileInfo> inList_,
                                           QStringList catList)
{
    //Clear dialogs and prepare data for the users view
    outList.clear();
    inList.clear();
    fileInfoList.clear();
    int counter = 0;
    foreach(const QFileInfo fileInfo, inList_.values())
    {
        if(fileInfo.exists())
        {
            QString tempName = fileInfo.fileName();
            if(tempName.length() > 30)
                tempName.resize(30);
            int i = 1;
            while(inList.contains(tempName))
            {
                if(i == 1)
                {
                    if(tempName.length() == 30)
                        tempName.chop(3);
                    tempName += QString("(%1)").arg(i);
                }
                else
                {
                    if(i<10)
                    {
                        tempName.chop(3);
                        tempName += QString("(%1)").arg(i);
                    }
                    else
                    {
                        if(i<100)
                        {
                            tempName.chop(4);
                            tempName += QString("(%1)").arg(i);
                        }
                        else
                        {
                            tempName.chop(5);
                            tempName += QString("(%1)").arg(i);
                        }
                    }
                }
                i++;
            }
            
            QString namePrefix = fileInfo.absolutePath();
            while(namePrefix.endsWith("."))
                namePrefix.remove(namePrefix.length() - 1, 1);
            namePrefix = QDir(namePrefix).dirName();
            namePrefix.remove("/");
            QString name = namePrefix + "@" + fileInfo.fileName();
            QString currFileFormat;
            if(name.endsWith(".PNG",Qt::CaseInsensitive))
            {
                currFileFormat = "PNG";
                name.chop(4);
            }
            if(name.endsWith(".JPG",Qt::CaseInsensitive))
            {
                currFileFormat = "JPG";
                name.chop(4);
            }
            if(name.endsWith(".JPEG",Qt::CaseInsensitive))
            {
                currFileFormat = "JPEG";
                name.chop(5);
            }
            if(name.endsWith(".TIFF",Qt::CaseInsensitive))
            {
                currFileFormat = "TIFF";
                name.chop(5);
            }
            
            ImageData data;
            data.name = name;
            data.filePath = fileInfo.absoluteFilePath();
            data.date = fileInfo.created().date();
            data.format = currFileFormat;
            QString owner = fileInfo.owner();
            if(owner != "")//Author
            {
                if(owner.length() > 30)
                    owner.resize(30);
            }
            else owner = "Unknown";
            data.author = owner;
            data.desc = "None";
            data.hash = inList_.key(fileInfo);
            inList[tempName] = data;
            fileInfoList[tempName] = fileInfo;
            QCoreApplication::processEvents();
        }
        ++counter;
        emit photoProcesed(counter);
    }
    ui.prvLabel->setText(tr("PhotoPrv"));
    ui.descEdit->setText("");
    ui.nameEdit->setText("");
    ui.authorEdit->setText("");
    ui.dateEdit->setMaximumDate(QDate::currentDate());
    ui.dateEdit->setDate(QDate::currentDate());
    ui.catalogBox->clear();
    ui.catalogBox->addItems(catList);
    folderListWidget->clear();
    folderListWidget->addItems(catList);
    ui.inList->clear();
    ui.inList->addItems(inList.keys());
    ui.outList->clear();
    ui.addButton->setEnabled(false);
    ui.removeButton->setEnabled(false);
    ui.updateButton->setEnabled(false);
    photoListWidget->addItems(inList.keys());
}

void GS_albumer_folder_inserter::newCatalogClicked()
{
    bool ok;
    QString temp = QInputDialog::getText(this, tr("Catalog creation"),
                                         tr("New Catalog name:"), QLineEdit::Normal,
                                         "", &ok);
    temp.remove("/");
    temp.remove("\\");
    if(ok&&temp!="")
    {
        if(ui.catalogBox->findText(temp)==-1)
        {
            if(temp.length()>30)
            {
                QMessageBox::information (this,tr("Catalog error"),tr(
                        "The given catalog name is to long. Max lenght 30."
                        ));
                newCatalogClicked();
                return;
            }
            ui.catalogBox->addItem(temp);
            ui.catalogBox->setCurrentIndex(ui.catalogBox->findText(temp));
            folderListWidget->addItem(temp);
        }
        else
        {
            QMessageBox::information (this,tr("Catalog exists"),tr(
                    "A catalog with the given name already exists.\n"
                    ));
            ui.catalogBox->setCurrentIndex(ui.catalogBox->findText(temp));
        }
    }
}

void GS_albumer_folder_inserter::inListItemChanged()
{
    if(!ui.inList->selectedItems().isEmpty())
    {
        if(!ui.outList->selectedItems().isEmpty())
            ui.outList->setCurrentRow(-1);
        QString key = ui.inList->selectedItems().first()->text();
        ImageData item = inList[key];
        ui.nameEdit->setText(item.name);
        ui.dateEdit->setDate(item.date);
        ui.authorEdit->setText(item.author);
        ui.descEdit->setText(item.desc);
        if(item.hasmini)
            ui.prvLabel->setPixmap(QPixmap::fromImage(item.mini));
        else
        {
            QPixmap temp = QPixmap(item.filePath);
            if(temp.width()>128)
            {
                temp = temp.scaled(128,128,Qt::KeepAspectRatio,
                                   Qt::FastTransformation);
            }
            if(temp.height()>128)
            {
                temp = temp.scaled(128,128,Qt::KeepAspectRatio,
                                   Qt::FastTransformation);
            }
            item.mini = temp.toImage();
            item.hasmini = true;
            ui.prvLabel->setPixmap(temp);
        }
        ui.addButton->setEnabled(true);
        ui.removeButton->setEnabled(false);
        ui.updateButton->setEnabled(false);
    }
    else
    {
        ui.addButton->setEnabled(false);
    }
}

void GS_albumer_folder_inserter::outListItemChanged()
{
    if(!ui.outList->selectedItems().isEmpty())
    {
        if(!ui.inList->selectedItems().isEmpty())
            ui.inList->setCurrentRow(-1);
        QString key = ui.outList->selectedItems().first()->text();
        ImageData item = outList[key];
        int catIndex = ui.catalogBox->findText(item.catalog);
        ui.catalogBox->setCurrentIndex(catIndex);
        ui.nameEdit->setText(item.name);
        ui.authorEdit->setText(item.author);
        ui.descEdit->setText(item.desc);
        currFileFormat = item.format;
        ui.dateEdit->setDate(item.date);
        if(item.hasmini)
            ui.prvLabel->setPixmap(QPixmap::fromImage(item.mini));
        else
        {
            QPixmap temp = QPixmap(item.filePath);
            if(temp.width()>128)
            {
                temp = temp.scaled(128,128,Qt::KeepAspectRatio,
                                   Qt::FastTransformation);
            }
            if(temp.height()>128)
            {
                temp = temp.scaled(128,128,Qt::KeepAspectRatio,
                                   Qt::FastTransformation);
            }
            item.mini = temp.toImage();
            item.hasmini = true;
            ui.prvLabel->setPixmap(temp);
        }
        ui.addButton->setEnabled(false);
        ui.removeButton->setEnabled(true);
        ui.updateButton->setEnabled(true);
        if(item.limitSize)
        {
            ui.imageSizeBox->setEnabled(true);
            ui.widthSpinBox->setValue(item.widthLimit);
            ui.heightSpinBox->setValue(item.heightLimit);
            ui.enableScaleBox->setChecked(true);
        }
        else
        {
          ui.imageSizeBox->setEnabled(false);
          ui.enableScaleBox->setChecked(false);
        }
    }
    else
    {
        ui.removeButton->setEnabled(false);
        ui.updateButton->setEnabled(false);
    }
}

void GS_albumer_folder_inserter::addAllClicked()
{
    if(ui.catalogBox->currentText() != "" || ui.folderNamingBox->isChecked())
    {
        cancel = false;
        myProgress->setMaxCount(inList.keys().count());
        int counter = 0;
        myProgress->setCount(counter);
        myProgress->show();
        foreach(const QString key, inList.keys())
        {
            ImageData item = inList[key];
            if(ui.folderNamingBox->isChecked())
            {
                QString catalogName = fileInfoList[key].absolutePath();
                while(catalogName.endsWith("."))
                    catalogName.remove(catalogName.length() - 1, 1);
                item.catalog = QDir(catalogName).dirName();
            }
            else
                item.catalog = ui.catalogBox->currentText().trimmed();
            if(ui.enableScaleBox->isChecked())
            {
                item.limitSize = true;
                item.keepAspect = ui.keepAspectBox->isChecked();
                item.widthLimit = ui.widthSpinBox->value();
                item.heightLimit = ui.heightSpinBox->value();
            }
            else
                item.limitSize = false;
            outList[key] = item;
            inList.remove(key);
            photoListWidget->removeItem(key);
            ++counter;
            myProgress->setCount(counter);
            if(cancel)
             break;
        }
        ui.inList->clear();
        ui.inList->addItems(inList.keys());
        ui.outList->clear();
        ui.outList->addItems(outList.keys());
        ui.outList->setCurrentRow(0);
        myProgress->hide();
    }
    else
    {
        QMessageBox::information(this,tr("User error"),
                                 tr("Please select a catalog!"));
    }
}

void GS_albumer_folder_inserter::addClicked()
{
    if(ui.catalogBox->currentText().trimmed() != "")
    {
        if(ui.nameEdit->text().trimmed() != "")
        {
            if(!(ui.dateEdit->date()>QDate::currentDate()))
            {
                ui.addButton->setEnabled(false);
                QString itName = ui.inList->selectedItems().first()->text();
                ImageData item = inList[itName];
                item.catalog = ui.catalogBox->currentText().trimmed();
                item.name = ui.nameEdit->text().trimmed();
                item.author = ui.authorEdit->text().trimmed();
                item.desc = ui.descEdit->toPlainText().trimmed();
                item.date = ui.dateEdit->date();
                if(ui.enableScaleBox->isChecked())
                {
                    item.limitSize = true;
                    item.widthLimit = ui.widthSpinBox->value();
                    item.heightLimit = ui.heightSpinBox->value();
                }
                else
                    item.limitSize = false;
                outList[itName] = item;
                int index = ui.inList->currentRow();
                inList.remove(itName);
                photoListWidget->removeItem(itName);
                ui.inList->clear();
                ui.inList->addItems(inList.keys());
                ui.outList->clear();
                ui.outList->addItems(outList.keys());
                if(ui.inList->count() !=0)
                {
                    if(index == ui.inList->count())
                        ui.inList->setCurrentRow(index-1);
                    else ui.inList->setCurrentRow(index);
                }
                else
                {
                    ui.prvLabel->setText(tr("No more Photos."));
                }
            }
            else
            {
                QMessageBox::information(this,tr("User error"),
                                         tr("Invalid date."));
            }
        }
        else
        {
            QMessageBox::information(this,tr("User error"),
                                     tr("Please type in a name for the photo!"));
        }
    }
    else
    {
        QMessageBox::information(this,tr("User error"),
                                 tr("Please select a catalog!"));
    }
}

void GS_albumer_folder_inserter::updateClicked()
{
    if(ui.catalogBox->currentText() != "")
    {
        if(ui.nameEdit->text() != "")
        {
            QString key = ui.outList->selectedItems().first()->text();
            ImageData item = outList[key];
            item.catalog = ui.catalogBox->currentText().trimmed();
            item.name = ui.nameEdit->text().trimmed();
            item.author = ui.authorEdit->text().trimmed();
            item.desc = ui.descEdit->toPlainText().trimmed();
            item.date = ui.dateEdit->date();
            if(ui.enableScaleBox->isChecked())
            {
                item.limitSize = true;
                item.widthLimit = ui.widthSpinBox->value();
                item.heightLimit = ui.heightSpinBox->value();
            }
            else
                item.limitSize = false;
            outList.remove(key);
            outList[key] = item;
            if(ui.inList->count() != 0)
            {
                ui.inList->setCurrentRow(0);
            }
            else
            {
                ui.prvLabel->setText(tr("No more Photos."));
            }
        }
        else
        {
            QMessageBox::information(this,tr("User error"),
                                     tr("Please type in a name for the photo!"));
        }
    }
    else
    {
        QMessageBox::information(this,tr("User error"),
                                 tr("Please select a catalog!"));
    }
}

void GS_albumer_folder_inserter::removeClicked()
{
    QString key = ui.outList->selectedItems().first()->text();
    int index = ui.outList->currentRow();
    inList[key] = outList.value(key);
    outList.remove(key);
    photoListWidget->addImageItem(key);
    ui.inList->clear();
    ui.inList->addItems(inList.keys());
    ui.outList->clear();
    ui.outList->addItems(outList.keys());
    if(ui.outList->count() != 0)
    {
        if(index == ui.outList->count())
            ui.outList->setCurrentRow(index-1);
        else ui.outList->setCurrentRow(index);
    }
    else
    {
        ui.inList->setCurrentRow(0);
    }
}

QList<ImageData> GS_albumer_folder_inserter::getOutList()
{
    inList.clear();
    return outList.values();
}

void GS_albumer_folder_inserter::commitClicked()
{
    close();
    emit insertReady();
}

void GS_albumer_folder_inserter::descTextChanged()
{
    if(ui.descEdit->toPlainText().length() > 80)
    {
        QString text = ui.descEdit->toPlainText();
        text.resize(80);
        ui.descEdit->document()->clear();
        ui.descEdit->insertPlainText(text);
        ui.descEdit->textCursor().movePosition(QTextCursor::End);
    }
}

void GS_albumer_folder_inserter::updateMiniature(QString name)
{
    if(inList.contains(name))
    {
        if(!inList.value(name).hasmini)
        {
            QPixmap temp = QPixmap(inList.value(name).filePath);
            if(temp.width()>128)
            {
                temp = temp.scaled(128,128,Qt::KeepAspectRatio,
                                   Qt::FastTransformation);
            }
            if(temp.height()>128)
            {
                temp = temp.scaled(128,128,Qt::KeepAspectRatio,
                                   Qt::FastTransformation);
            }
            inList[name].mini = temp.toImage();
            inList[name].hasmini = true;
        }
        photoListWidget->setItemMiniature(name,
                                          QPixmap::fromImage(
                                                  inList.value(name).mini));
    }
}

void GS_albumer_folder_inserter::movePhotoTrigered(QString photoName,QString FolderName)
{
    if(inList.contains(photoName) && FolderName.trimmed() != "")
    {
        ImageData item = inList[photoName];
        item.catalog = FolderName.trimmed();
        outList[photoName] = item;
        inList.remove(photoName);
        ui.inList->clear();
        ui.inList->addItems(inList.keys());
        ui.outList->clear();
        ui.outList->addItems(outList.keys());
        photoListWidget->removeItem(photoName);
    }
}

void GS_albumer_folder_inserter::cancelClicked()
{
    cancel = true;
}

void GS_albumer_folder_inserter::sizeLimitClicked(bool state)
{
    ui.imageSizeBox->setEnabled(state);
}

GS_albumer_folder_inserter::GS_albumer_folder_inserter(QWidget *parent)
        :QWidget(parent)
{
    ui.setupUi(this);
    this->needToClose = false;
    cancel = false;
    photoListWidget = new FolderInserterDragPhotoList();
    ui.toolLayout->addWidget(photoListWidget);
    folderListWidget = new FolderInserterFolderView();
    ui.toolLayout->addWidget(folderListWidget);
    myProgress = new GS_albumer_progress();
    connect(ui.inList,SIGNAL(itemSelectionChanged()),
            this,SLOT(inListItemChanged()));
    connect(ui.outList,SIGNAL(itemSelectionChanged()),
            this,SLOT(outListItemChanged()));
    connect(ui.addAllButton,SIGNAL(clicked()),this,SLOT(addAllClicked()));
    connect(ui.addButton,SIGNAL(clicked()),this,SLOT(addClicked()));
    connect(ui.updateButton,SIGNAL(clicked()),this,SLOT(updateClicked()));
    connect(ui.removeButton,SIGNAL(clicked()),this,SLOT(removeClicked()));
    connect(ui.catButton,SIGNAL(clicked()),this,SLOT(newCatalogClicked()));
    connect(ui.catButton2,SIGNAL(clicked()),this,SLOT(newCatalogClicked()));
    connect(ui.disButton,SIGNAL(clicked()),this,SLOT(discardClicked()));
    connect(ui.commitButton,SIGNAL(clicked()),this,SLOT(commitClicked()));
    connect(ui.descEdit,SIGNAL(textChanged()),this,SLOT(descTextChanged()));
    connect(photoListWidget,SIGNAL(requestMiniature(QString)),
            this,SLOT(updateMiniature(QString)));
    connect(folderListWidget,SIGNAL(movePhoto(QString,QString))
            ,this,SLOT(movePhotoTrigered(QString,QString)));
    connect(myProgress,SIGNAL(cancelClicked()),this,SLOT(cancelClicked()));
    connect(ui.enableScaleBox,SIGNAL(clicked(bool)),
            this,SLOT(sizeLimitClicked(bool)));
}
