#include "PluginManager.h"


PluginManager* PluginManager::getInstance()
{
    if(myInstance == NULL)
    {
        myInstance = new PluginManager();
    }
    return myInstance;
}

ImageFilter* PluginManager::getImageFilterByName(QString name)
{
    if(imageFilterRoots.keys().contains(name))
        return imageFilterRoots[name]->getNewInstance();
    return NULL;
}

QStringList PluginManager::getImageFilterNames() const
{
    return imageFilterRoots.keys();
}

void PluginManager::loadPlugins()
{
    QStringList args = QApplication::arguments();
    QString path;
    if(args.count() > 1)
    {
        QFileInfo dir(args.first());
        path = dir.absolutePath()+"/";
    }
    else path = "";
    QDir dir(path + "ImageFilters");
    foreach(QString filename , dir.entryList(QDir::Files))
    {
        QPluginLoader loader(dir.absoluteFilePath(filename));
        QObject* couldBePlugin=loader.instance();
        if(couldBePlugin)
        {
            ImageFilterInterface* filterPlugin =
                    qobject_cast<ImageFilterInterface*>(couldBePlugin);
            if(filterPlugin)
            {
                imageFilterRoots[filterPlugin->filterName()] = filterPlugin;
            }
        }
    }
}

PluginManager::PluginManager()
{
    loadPlugins();
}

PluginManager* PluginManager::myInstance = NULL;
