#include "categoryitem.h"

QRectF CategoryItem::boundingRect() const
{
    qreal penWidth = 1;
    return QRectF(0- penWidth / 2, 0 - penWidth / 2,
                  scene()->width() + penWidth / 2, 60 + penWidth / 2);
}

void CategoryItem::paint(QPainter *painter, const QStyleOptionGraphicsItem*,
                         QWidget*)
{
    painter->setFont(QFont("Courier",8,8));
    if(!isTouched)
    {
        painter->setPen(QColor(Qt::black));
        painter->drawText(QRectF(60.0f,2.0f,scene()->width()-60,58.0f),
                          Qt::AlignLeft|Qt::AlignVCenter|Qt::TextWrapAnywhere,
                          myText,new QRectF(0.0f,0.0f,scene()->width()-64,58));
        painter->drawPixmap(10,11,myPixmap);
    }
    else
    {
        painter->setPen(QColor(Qt::black));
        painter->drawText(QRectF(60.0f,2.0f,scene()->width()-60,58.0f),
                          Qt::AlignLeft|Qt::AlignVCenter|Qt::TextWrapAnywhere,
                          myText,new QRectF(0.0f,0.0f,scene()->width()-64,58));
        painter->setPen(QColor(Qt::white));
        painter->drawText(QRectF(61.0f,1.0f,scene()->width()-60,58.0f),
                          Qt::AlignLeft|Qt::AlignVCenter|Qt::TextWrapAnywhere,
                          myText,new QRectF(0.0f,0.0f,scene()->width()-64,58));
        painter->drawPixmap(10,11,myPixmap_alt);
    }
}

void CategoryItem::hoverEnterEvent(QGraphicsSceneHoverEvent*)
{
    emit touched(myIndex);
    isTouched = true;
    update();
}

void CategoryItem::hoverLeaveEvent(QGraphicsSceneHoverEvent*)
{
    isTouched = false;
    update();
}

void CategoryItem::setIndex(unsigned int index)
{
    myIndex = index;
}

void CategoryItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    if(event->button() == Qt::LeftButton)
    {
        emit clicked(myIndex);
        isTouched = false;
        update();
    }
}

void CategoryItem::dragEnterEvent(QGraphicsSceneDragDropEvent* event)
{
    if(event->mimeData()->hasFormat("GSAlbumer/photo_IdData"))
    {
        event->setAccepted(true);
        emit touched(myIndex);
        isTouched = true;
        update();
    }
    else
        event->setAccepted(false);
}

void CategoryItem::dropEvent(QGraphicsSceneDragDropEvent* event)
{
    isTouched = false;
    int photoId;
    photoId = event->mimeData()->data("GSAlbumer/photo_IdData").toInt();
    emit photoDroped(photoId,myIndex);
    event->acceptProposedAction();
    update();
}

void CategoryItem::dragLeaveEvent(QGraphicsSceneDragDropEvent* event)
{
    Q_UNUSED(event);
    isTouched = false;
    update();
}

CategoryItem::CategoryItem(QPixmap myItemPixmap,QPixmap myItemPixmap_alt,QString text,
                           unsigned int index)
:QGraphicsObject(),myPixmap(myItemPixmap),myPixmap_alt(myItemPixmap_alt),
myText(text),myIndex(index),isTouched(false)
{
    this->setAcceptsHoverEvents(true);
    this->setAcceptDrops(true);
}
