#include "categoryselector.h"

void CategorySelector::itemTouched(unsigned int index)
{
    if(bgHidden)
    {
        bgItem->show();
        bgHidden = false;
    }
    shooter->stop();
    bgItem->setPos(0,wantedY);
    wantedY = static_cast<unsigned int>(catItems.at(index)->pos().y());
    wantedIndex = index;
    shooter->start(30);
    emit itemTouchedSignal(index);
}

void CategorySelector::itemClicked(unsigned int index)
{
    emit itemClickedSignal(index);
}

int CategorySelector::calculateStep(int currentValue, int wantedValue)
{
    int step = (wantedValue - currentValue)/5;
    if(step > 10 || step < -10)
    {
        for(int i = 0; i < catItems.count(); ++i) // ;( the items need refreshing
            catItems.at(i)->update();
    }
    if(step == 0)
    {
        if(currentValue > wantedValue)
            return -1;
        else
            return 1;
    }
    return step;
}

void CategorySelector::moveBg()
{
    unsigned int currY = static_cast<unsigned int>(bgItem->pos().y());
    if(currY > wantedY)
    {
        currY += calculateStep(currY,wantedY);
        if(currY < wantedY)
            currY = wantedY;

        bgItem->setPos(0,currY);
        return;
    }
    else if(currY < wantedY)
    {
        currY += calculateStep(currY,wantedY);
        if(currY > wantedY)
            currY = wantedY;

        bgItem->setPos(0,currY);
        return;
    }
    shooter->stop();
    for(int i = 0; i < catItems.count(); ++i) // ;( the items need refreshing
        catItems.at(i)->update();
}

void CategorySelector::resizeEvent(QResizeEvent*)
{
    gradient = QLinearGradient(0, 0, ui.categoryView->width(), 0);
    gradient.setColorAt(0.0,QColor(120,120,120));
    gradient.setColorAt(1.0,QColor(170,170,170));

    QGraphicsScene* scene = new QGraphicsScene(0,0,ui.categoryView->width(),60*(catItems.count()));
    scene->setBackgroundBrush(gradient);
    scene->addItem(bgItem);
    foreach(CategoryItem* item_, catItems)
        scene->addItem(item_);
    QGraphicsScene* temp = ui.categoryView->scene();
    ui.categoryView->setScene(scene);
    delete temp;
}

void CategorySelector::showEvent(QShowEvent *)
{
    gradient = QLinearGradient(0, 0, ui.categoryView->width(), 0);
    gradient.setColorAt(0.0,QColor(120,120,120));
    gradient.setColorAt(1.0,QColor(170,170,170));

    QGraphicsScene* scene = new QGraphicsScene(0,0,ui.categoryView->width(),60*(catItems.count()));
    scene->setBackgroundBrush(gradient);
    scene->addItem(bgItem);
    foreach(CategoryItem* item_, catItems)
        scene->addItem(item_);
    QGraphicsScene* temp = ui.categoryView->scene();
    ui.categoryView->setScene(scene);
    delete temp;
}

void CategorySelector::addItem(QPixmap pixmap,QPixmap pixmap_pressed,
                               QString text)
{
    int newIndex = catItems.count();
    if(newIndex == 0)
        bgItem->show();
    CategoryItem* item = new CategoryItem(pixmap,pixmap_pressed
                                          ,text,newIndex);
    item->setPos(0,60*newIndex);
    item->setZValue(1);
    connect(item,SIGNAL(clicked(unsigned int)),
            this,SLOT(itemClicked(unsigned int)));
    connect(item,SIGNAL(touched(unsigned int)),
            this,SLOT(itemTouched(unsigned int)));
    connect(item,SIGNAL(photoDroped(int,int)),
            this,SLOT(photoDropDelegationSlot(int,int)));
    catItems<<item;
    QGraphicsScene* scene = new QGraphicsScene(0,0,ui.categoryView->width(),60*(newIndex+1));
    QLinearGradient gradient(0, 0, ui.categoryView->width(), 0);
    gradient.setColorAt(0.0,QColor(120,120,120));
    gradient.setColorAt(1.0,QColor(170,170,170));
    scene->setBackgroundBrush(gradient);
    scene->addItem(bgItem);
    bgItem->hide();
    bgHidden = true;
    foreach(CategoryItem* item_, catItems)
        scene->addItem(item_);
    QGraphicsScene* temp = ui.categoryView->scene();
    ui.categoryView->setScene(scene);
    delete temp;
}

void CategorySelector::removeItem(unsigned int index)
{
    if(index < static_cast<unsigned int>(catItems.count()))
    {
        if(wantedIndex > index)
            --wantedIndex;
        shooter->stop();
        CategoryItem* tempItem = catItems.at(index);
        ui.categoryView->scene()->removeItem(tempItem);
        catItems.remove(index);
        QGraphicsScene* scene = new QGraphicsScene(0,0,ui.categoryView->width(),60*catItems.count());
        scene->setBackgroundBrush(gradient);
        scene->addItem(bgItem);
        for(unsigned int i = 0;i < static_cast<unsigned int>(catItems.count());++i)
        {
            catItems.at(i)->setIndex(i);
            catItems.at(i)->setPos(0,60*i);
            scene->addItem(catItems.at(i));
        }
        QGraphicsScene* temp = ui.categoryView->scene();
        ui.categoryView->setScene(scene);
        if(catItems.count() > 0)
        {
            if(wantedIndex >= static_cast<unsigned int>(catItems.count()))
                wantedIndex = static_cast<unsigned int>(catItems.count()) -1;
            wantedY = static_cast<unsigned int>(
                    catItems.value(wantedIndex)->pos().y());
            shooter->start(5);
        }
        else
        {
            bgItem->hide();
        }
        delete temp;
        delete tempItem;
    }
}

void CategorySelector::scrollUp()
{
    int currValye = ui.categoryView->verticalScrollBar()->value()-10;
    ui.categoryView->verticalScrollBar()->setValue(currValye);
}

void CategorySelector::scrollDown()
{
    int currValye = ui.categoryView->verticalScrollBar()->value()+10;
    ui.categoryView->verticalScrollBar()->setValue(currValye);
}

void CategorySelector::photoDropDelegationSlot(int photoId,int index)
{
    emit photoDroped(photoId,index);
}

void CategorySelector::clearItems()
{
    wantedIndex = 0;
    catItems.clear();
    QGraphicsScene* scene = new QGraphicsScene(0,0,150,60);
    QLinearGradient gradient(0, 0, 150, 0);
    gradient.setColorAt(0.0,QColor(120,120,120));
    gradient.setColorAt(1.0,QColor(170,170,170));
    scene->setBackgroundBrush(gradient);
    scene->addItem(bgItem);
    QGraphicsScene* temp = ui.categoryView->scene();
    ui.categoryView->setScene(scene);
    bgItem->hide();
    delete temp;
}

void CategorySelector::saveVerticalPos()
{
    savedVerticalPositon = ui.categoryView->verticalScrollBar()->value();
}

void CategorySelector::loadVerticalPos()
{
    ui.categoryView->verticalScrollBar()->setValue(savedVerticalPositon);
}

void CategorySelector::setTitle(const QString& title)
{
    ui.titleLabel->setText(title);
}

CategorySelector::CategorySelector(QWidget *parent)
    : QWidget(parent)
{
    ui.setupUi(this);
    shooter = new QTimer(this);
    slider = new QTimer(this);
    wantedY = 0;
    wantedIndex = 0;
    QGraphicsScene* scene = new QGraphicsScene(0,0,150,ui.categoryView->height());
    gradient = QLinearGradient(0, 0, 150, 0);
    gradient.setColorAt(0.0,QColor(120,120,120));
    gradient.setColorAt(1.0,QColor(170,170,170));
    scene->setBackgroundBrush(gradient);
    top = new ScroolOperationWidget(QPixmap(":/eyecandy/upArrow.png"),-2);
    bottom = new ScroolOperationWidget(QPixmap(":/eyecandy/downArrow.png"),2);
    ui.topSlot->addWidget(top);
    ui.bottomSlot->addWidget(bottom);
    bgItem = new CategoryBgItem();
    bgItem->hide();
    bgHidden = true;
    bgItem->setPos(0,0);
    bgItem->setZValue(0);
    scene->addItem(bgItem);

    savedVerticalPositon = 0;
    this->setAcceptDrops(true);

    ui.categoryView->setScene(scene);
    connect(shooter,SIGNAL(timeout()),this,SLOT(moveBg()));
    connect(top,SIGNAL(touched()),this,SLOT(scrollUp()));
    connect(bottom,SIGNAL(touched()),this,SLOT(scrollDown()));
}
