#ifndef CATEGORYSELECTOR_H
#define CATEGORYSELECTOR_H

#include <QtGui/QWidget>
#include <QGraphicsScene>
#include <QList>
#include <QTimer>
#include <QResizeEvent>
#include <QDragEnterEvent>
#include <QScrollBar>
#include "ui_categoryselector.h"
#include "categoryitem.h"
#include "ScroolOperationWidget.h"
#include "categorybgitem.h"

class CategorySelector : public QWidget
{
    Q_OBJECT

public:
    CategorySelector(QWidget *parent = 0);
    ~CategorySelector()
    {
        for(int i=catItems.count()-1;i>=0;--i)
            delete catItems.value(i);
        catItems.clear();
        delete bgItem;
        delete bottom;
        delete top;
    }
    void addItem(QPixmap pixmap,QPixmap pixmap_pressed,QString text);//adds an item with the given pixmap and text
    void removeItem(unsigned int index);
    void clearItems();
    void saveVerticalPos();
    void loadVerticalPos();
    void setTitle(const QString& title);

protected:
    void resizeEvent(QResizeEvent* event);
    void showEvent(QShowEvent *);

private slots:
    void itemTouched(unsigned int index);//slot used 2 animate item hover
    void itemClicked(unsigned int index);//slot used 2 propagate item click
    void moveBg();//slot used 2 moving the background item
    void scrollUp();//slot used 2 scroll Up
    void scrollDown();//slot used 2 scroll Down
    void photoDropDelegationSlot(int photoId,int index);

signals:
    void itemTouchedSignal(int index);
    void itemClickedSignal(int index);//signal emited on item click
    void photoDroped(int photoId, int index);

private:
    int calculateStep(int currentValue, int wantedValue);

    Ui::CategorySelector_ui ui;//UI :P
    QTimer* shooter;//animation timer
    QTimer* slider;
    CategoryBgItem* bgItem;//the background item
    ScroolOperationWidget* top;
    ScroolOperationWidget* bottom;
    QVector<CategoryItem*> catItems;//category item map
    unsigned int wantedY,wantedIndex;//background destination position
    bool bgHidden;
    int savedVerticalPositon;
    QLinearGradient gradient;
};

#endif // CATEGORYSELECTOR_H
