#include "gs_albumer_searcher.h"

void GS_albumer_searcher::setAuthorList(QStringList authorList)
{
    ui.authorComboBox->clear();
    ui.authorComboBox->addItems(authorList);
    ui.nameList->clear();
    ui.nameEdit->setText("");
    nameList.clear();
    names = false;
    author = false;
    date = false;
    ui.namesCheckBox->setChecked(false);
    ui.authorCheckBox->setChecked(false);
    ui.dateCheckBox->setChecked(false);
    ui.toDateEdit->setDate(QDate::currentDate());
}

QDate GS_albumer_searcher::getFromDate()
{
    return ui.fromDateEdit->date();
}

QDate GS_albumer_searcher::getToDate()
{
    return ui.toDateEdit->date();
}

QString GS_albumer_searcher::getAuthor()
{
    return ui.authorComboBox->currentText();
}

QStringList GS_albumer_searcher::getNames()
{
    return nameList;
}

void GS_albumer_searcher::closeEvent(QCloseEvent* event)
{
    if(needToClose)
    {
        event->accept();
    }
    else
    {
        event->ignore();
        hide();
    }
}

void GS_albumer_searcher::setNeedToClose()
{
    needToClose = true;
}

void GS_albumer_searcher::nameSearchChanged(int state)
{
    if(state == Qt::Checked)
    {
        ui.addNameButton->setEnabled(true);
        ui.nameEdit->setEnabled(true);
        names = true;
    }
    else
    {
        ui.addNameButton->setEnabled(false);
        ui.removeNameButton->setEnabled(false);
        ui.nameEdit->setEnabled(false);
        names = false;
    }
}

void GS_albumer_searcher::authorSearchChanged(int state)
{
    if(state == Qt::Checked)
    {
        ui.authorComboBox->setEnabled(true);
        author = true;
    }
    else
    {
        ui.authorComboBox->setEnabled(false);
        author = false;
    }
}

void GS_albumer_searcher::dateSearchChanged(int state)
{
    if(state == Qt::Checked)
    {
        ui.fromDateEdit->setEnabled(true);
        ui.toDateEdit->setEnabled(true);
        date = true;
    }
    else
    {
        ui.fromDateEdit->setEnabled(false);
        ui.toDateEdit->setEnabled(false);
        date = false;
    }
}

void GS_albumer_searcher::nameListSelChanged()
{
    if(!ui.nameList->selectedItems().isEmpty())
        ui.removeNameButton->setEnabled(true);
    else
        ui.removeNameButton->setEnabled(false);
}

void GS_albumer_searcher::addName()
{
    if(ui.nameEdit->text() != "")
    {
        QString name =  ui.nameEdit->text();
        if(ui.nameList->findItems(name,Qt::MatchFixedString).isEmpty())
        {
            ui.nameList->addItem(name);
            nameList<<name;
        }
        else
        {
            QMessageBox::information(this,tr("User error"),
                                     tr("The name %1 was already in the search list").arg(name));
        }
        ui.nameEdit->setText("");
    }
    else
    {
        QMessageBox::information(this,tr("User error"),
                                 tr("Please type in a name!"));
    }
}

void GS_albumer_searcher::removeName()
{
    if(!ui.nameList->selectedItems().isEmpty())
    {
        nameList.removeAt(nameList.indexOf
                          (ui.nameList->takeItem(ui.nameList->currentRow())->text()));
        ui.nameList->setCurrentRow(-1);
    }
}

void GS_albumer_searcher::searchClicked()
{
    if(names||author||date)
    {
        if(ui.namesCheckBox->isChecked())
        {
            if(nameList.isEmpty())
            {
                QMessageBox::information(this,tr("User error"),
                                         tr("The name list is empty!"));
                return;
            }
        }
        if(ui.authorCheckBox->isChecked())
        {
            if(ui.authorComboBox->currentText() == "")
            {
                QMessageBox::information(this,tr("User error"),
                                         tr("No Author, the album is probably still empty."));
                return;
            }
        }
        this->hide();
        emit searchReady(names,author,date);
    }
    else
    {
        QMessageBox::information(this,tr("User error"),
                                 tr("Please choose some search options."));
        return;
    }
}

void GS_albumer_searcher::fromDateChanged(QDate newDate)
{
    if(newDate>=ui.toDateEdit->date())
    {
        ui.fromDateEdit->setDate(ui.toDateEdit->date().addDays(-1));
    }
}

void GS_albumer_searcher::toDateChanged(QDate newDate)
{
    if(ui.fromDateEdit->date()>=newDate)
    {
        ui.toDateEdit->setDate(ui.fromDateEdit->date().addDays(1));
    }
}

GS_albumer_searcher::GS_albumer_searcher(QWidget *parent)
    :QWidget(parent)
{
    ui.setupUi(this);
    needToClose = false;
    names = false;
    author = false;
    date = false;
    ui.fromDateEdit->setEnabled(false);
    ui.toDateEdit->setEnabled(false);
    ui.authorComboBox->setEnabled(false);
    ui.addNameButton->setEnabled(false);
    ui.removeNameButton->setEnabled(false);
    ui.nameEdit->setEnabled(false);
    connect(ui.discardButton,SIGNAL(clicked()),this,SLOT(close()));
    connect(ui.namesCheckBox,SIGNAL(stateChanged(int)),this,
            SLOT(nameSearchChanged(int)));
    connect(ui.authorCheckBox,SIGNAL(stateChanged(int)),this,
            SLOT(authorSearchChanged(int)));
    connect(ui.dateCheckBox,SIGNAL(stateChanged(int)),this,
            SLOT(dateSearchChanged(int)));
    connect(ui.nameList,SIGNAL(itemSelectionChanged()),this,
            SLOT(nameListSelChanged()));
    connect(ui.addNameButton,SIGNAL(clicked()),this,SLOT(addName()));
    connect(ui.nameEdit,SIGNAL(returnPressed()),this,SLOT(addName()));
    connect(ui.removeNameButton,SIGNAL(clicked()),this,SLOT(removeName()));
    connect(ui.searchButton,SIGNAL(clicked()),this,SLOT(searchClicked()));
    connect(ui.fromDateEdit,SIGNAL(dateChanged(QDate)),this,
            SLOT(fromDateChanged(QDate)));
    connect(ui.toDateEdit,SIGNAL(dateChanged(QDate)),this,
            SLOT(toDateChanged(QDate)));
}
