#include "GS_albumer_viewer.h"

void GS_albumer_viewer::closeEvent(QCloseEvent* event)
{
    if(historyIndex > 0)
    {
        int resault = QMessageBox::question(this,tr("Save?"),
                                            tr("There are unsaved changes that will bi lost if You continue."),
                                            QMessageBox::Save,
                                            QMessageBox::Discard,
                                            QMessageBox::Cancel);
        if(resault == QMessageBox::Save)
            saveSlot();
        else if(resault == QMessageBox::Cancel)
        {
            event->ignore();
            unSetClose();
            return;
        }
    }
    GS_albumer_view::closeEvent(event);
}

void GS_albumer_viewer::keyPressEvent(QKeyEvent* event)
{
    if(event->key() == Qt::Key_Z)
    {
        if(event->modifiers() == Qt::CTRL)
            historyBack();
        else if(event->modifiers() == (Qt::CTRL|Qt::SHIFT))
            historyForward();
    }
    QWidget::keyPressEvent(event);
}

void GS_albumer_viewer::setImage(const QImage& image,const QString& format,int id)
{
    myImage = QPixmap::fromImage(image);
    myImageId = id;
    historyList << myImage;
    myImageFormat = format;
    if(myImage.width()>ui.scrollArea->width() -15
       || myImage.height()>ui.scrollArea->height() -15)
    {
        ui.label->setPixmap(myImage.
                            scaled(ui.scrollArea->width()-15,
                                   ui.scrollArea->height()-15,Qt::KeepAspectRatio));
    }
    else
        ui.label->setPixmap(myImage);
}

bool GS_albumer_viewer::isMyImage(int id)
{
    bool resault = false;
    if(myImageId == id)
        resault = true;
    return resault;
}

void GS_albumer_viewer::resizeEvent(QResizeEvent* event)
{
    QWidget::resizeEvent(event);
    updateImage();
}

void GS_albumer_viewer::mouseReleaseEvent(QMouseEvent* event )
{
    if(event->button() == Qt::RightButton)
        myMenu->exec(event->globalPos());
    if(!fit && event->button() == Qt::LeftButton)
        ui.label->setCursor(Qt::OpenHandCursor);
    drag = false;
}

void GS_albumer_viewer::mousePressEvent(QMouseEvent* event)
{
    if(childAt(event->pos()) != ui.bar)
    {
        if(!fit && event->button() == Qt::LeftButton)
        {
            ui.label->setCursor(Qt::ClosedHandCursor);
            x = event->pos().x();
            y = event->pos().y();
            drag = true;
        }
    }
}

void GS_albumer_viewer::mouseMoveEvent(QMouseEvent* event)
{
    if(!fit && drag)
    {
        int dx = ui.scrollArea->horizontalScrollBar()->value() -
                 ((event->pos().x() - x));
        int dy = ui.scrollArea->verticalScrollBar()->value() -
                 ((event->pos().y() - y));
        x = event->pos().x();
        y = event->pos().y();
        ui.scrollArea->horizontalScrollBar()->setValue(dx);
        ui.scrollArea->verticalScrollBar()->setValue(dy);
    }
}

void GS_albumer_viewer::exportImage()
{
    QString fileFilter = myImageFormat + " (*." + myImageFormat.toLower() + ")";
    QString fname="";
    fname = QFileDialog::getSaveFileName(this, tr("Export File"),"./",fileFilter);
    if(fname.trimmed() != "")
    {
        myProgress->setCancel(false);
        myProgress->setMaxCount(2);
        myProgress->setCount(0);
        myProgress->show();
        if(!fname.endsWith("." + myImageFormat.toLower()))
            fname += "." + myImageFormat.toLower();
        myProgress->setCount(1);
        QImageWriter writer(fname,myImageFormat.toAscii());
        writer.write(myImage.toImage());
        myProgress->setCount(2);
        myProgress->hide();
    }
}

void GS_albumer_viewer::setFit()
{
    fit = true;
    ui.label->setCursor(Qt::ArrowCursor);
    setFitAction->setEnabled(false);
    ui.fitButton->setEnabled(false);
    disetFitAction->setEnabled(true);
    ui.orginalButton->setEnabled(true);
    updateImage();
    ui.scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    ui.scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
}

void GS_albumer_viewer::disetFit()
{
    fit = false;
    ui.label->setCursor(Qt::OpenHandCursor);
    setFitAction->setEnabled(true);
    ui.fitButton->setEnabled(true);
    disetFitAction->setEnabled(false);
    ui.orginalButton->setEnabled(false);
    updateImage();
    ui.scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    ui.scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
}

void GS_albumer_viewer::rotateClockWise()
{
    QTransform transformation;
    transformation.rotate(90.0f);
    myImage = myImage.transformed(transformation,Qt::SmoothTransformation);
    updateHistory();
    updateImage();
    updateSaveButtons();
}

void GS_albumer_viewer::rotateCClockWise()
{
    QTransform transformation;
    transformation.rotate(-90.0f);
    myImage = myImage.transformed(transformation,Qt::SmoothTransformation);
    updateHistory();
    updateImage();
    updateSaveButtons();
}

void GS_albumer_viewer::vMirror()
{
    QTransform transformation;
    transformation.rotate(180,Qt::YAxis);
    myImage = myImage.transformed(transformation,Qt::SmoothTransformation);
    updateHistory();
    updateImage();
    updateSaveButtons();
}

void GS_albumer_viewer::hMirror()
{
    QTransform transformation;
    transformation.rotate(180,Qt::XAxis);
    myImage = myImage.transformed(transformation,Qt::SmoothTransformation);
    updateHistory();
    updateImage();
    updateSaveButtons();
}

void GS_albumer_viewer::updateHistory()
{
    if(historyIndex < historyList.count() - 1)
    {
        while(historyIndex < historyList.count() - 1)
            historyList.removeLast();
    }
    historyList.append(myImage);
    ++historyIndex;
    ui.reDoButton->setEnabled(false);
    ui.unDoButton->setEnabled(true);
}

void GS_albumer_viewer::historyBack()
{
    if(historyIndex > 0)
    {
        --historyIndex;
        myImage = historyList.at(historyIndex);
        if(historyIndex == 0)
            ui.unDoButton->setEnabled(false);
        if(historyIndex < historyList.count() - 1)
            ui.reDoButton->setEnabled(true);
        updateImage();
        updateSaveButtons();
    }
}

void GS_albumer_viewer::historyForward()
{
    if(historyIndex < historyList.count()-1)
    {
        ++historyIndex;
        myImage = historyList.at(historyIndex);
        if(historyIndex == historyList.count()-1)
            ui.reDoButton->setEnabled(false);
        if(historyIndex != 0)
            ui.unDoButton->setEnabled(true);
        updateImage();
        updateSaveButtons();
    }
}

void GS_albumer_viewer::saveSlot()
{
    myProgress->setCancel(false);
    myProgress->setMaxCount(2);
    myProgress->setCount(0);
    myProgress->show();
    QPixmap miniature = myImage.scaled(128,128,Qt::KeepAspectRatio,
                                       Qt::SmoothTransformation);
    QBuffer buffer;
    myImage.toImage().save(&buffer,myImageFormat.toAscii());
    QString hash(QCryptographicHash::hash(buffer.data(),QCryptographicHash::Md4));
    myProgress->setCount(1);
    myManipulator->changeImage(myImageId,myImage.toImage(),
                               miniature.toImage(),hash,myImageFormat);
    historyIndex = 0;
    historyList.clear();
    historyList.append(myImage);
    ui.reDoButton->setEnabled(false);
    ui.unDoButton->setEnabled(false);
    emit imageChanged(myImageId);
    myProgress->setCount(2);
    updateSaveButtons();
    myProgress->hide();
}

void GS_albumer_viewer::saveAsSlot()
{
    Viewer_SaveAs_Widget saveDialog(myManipulator->getFolders());
    int resault = saveDialog.exec();
    if(resault == 1)
    {
        myProgress->setCancel(false);
        myProgress->setMaxCount(2);
        myProgress->setCount(0);
        myProgress->show();
        ImageData data;
        data.name = saveDialog.getName();
        data.date = QDate::currentDate();
        QString catalog = saveDialog.getCatalog();
        data.catalog = catalog;
        data.author = (saveDialog.getAutor().trimmed() != "")
                      ? saveDialog.getAutor() : tr("Unknown");
        data.desc = (saveDialog.getDesc().trimmed() != "")
                    ? saveDialog.getDesc() : tr("None");
        data.format = saveDialog.getFormat();
        QBuffer buffer;
        myImage.toImage().save(&buffer,myImageFormat.toAscii());
        QString hash(QCryptographicHash::hash(buffer.data(),QCryptographicHash::Md4));
        data.hash = hash;
        data.mini = myImage.scaled(128,128,Qt::KeepAspectRatio,
                                   Qt::SmoothTransformation).toImage();
        myProgress->setCount(1);
        int index = myManipulator->addImage(myImage.toImage(),data,GS_config_holder::getQuality());
        emit newImageAdded(catalog,index);
        myImageId = index;
        historyIndex = 0;
        historyList.clear();
        historyList.append(myImage);
        ui.reDoButton->setEnabled(false);
        ui.unDoButton->setEnabled(false);
        myProgress->setCount(2);
        updateSaveButtons();
        myProgress->hide();
    }
}

void GS_albumer_viewer::applyFilter(int)
{
    if(ui.filterList->currentIndex() != -1)
    {
        ui.filterList->setEditable(false);
        QString filterName = ui.filterList->currentText();
        currentFilter = PluginManager::getInstance()->getImageFilterByName(filterName);
        connect(currentFilter,SIGNAL(finished()),this,SLOT(filterFinished()));
        connect(currentFilter,SIGNAL(blockInput()),this,SLOT(filterBlock()));
        connect(currentFilter,SIGNAL(canceled()),this,SLOT(filterCancel()));
        disconnect(ui.filterList,SIGNAL(currentIndexChanged(int)),this,SLOT(applyFilter(int)));
        currentFilter->setPixmap(myImage);
        ui.filterList->setCurrentIndex(-1);
        this->setFocus();
    }
}

void GS_albumer_viewer::filterFinished()
{
    if(currentFilter != NULL)
    {
        myProgress->setCount(1);
        myImage = currentFilter->getResault();
        updateHistory();
        updateImage();
        updateSaveButtons();
        disconnect(currentFilter,SIGNAL(finished()),this,SLOT(filterFinished()));
        disconnect(currentFilter,SIGNAL(blockInput()),this,SLOT(filterBlock()));
        disconnect(currentFilter,SIGNAL(canceled()),this,SLOT(filterCancel()));
        connect(ui.filterList,SIGNAL(currentIndexChanged(int)),this,SLOT(applyFilter(int)));
        delete currentFilter;
        currentFilter = NULL;
        myProgress->hide();
        ui.filterList->setCurrentIndex(-1);
    }
}

void GS_albumer_viewer::filterBlock()
{
    myProgress->setMaxCount(1);
    myProgress->setCount(0);
    myProgress->setCancel(false);
    myProgress->show();
}

void GS_albumer_viewer::filterCancel()
{
    if(currentFilter != NULL)
    {
        disconnect(currentFilter,SIGNAL(finished()),this,SLOT(filterFinished()));
        disconnect(currentFilter,SIGNAL(blockInput()),this,SLOT(filterBlock()));
        disconnect(currentFilter,SIGNAL(canceled()),this,SLOT(filterCancel()));
        connect(ui.filterList,SIGNAL(currentIndexChanged(int)),this,SLOT(applyFilter(int)));
        delete currentFilter;
        currentFilter = NULL;
        myProgress->hide();
        ui.filterList->setCurrentIndex(-1);
        this->setFocus();
    }
}

void GS_albumer_viewer::updateImage()
{
    if(fit)
    {
        if(myImage.width()>ui.scrollArea->width() -15
           || myImage.height()>ui.scrollArea->height() -15)
        {
            ui.label->setPixmap(myImage); // temporary KDE QT 4.6 resize bug?
            ui.label->setPixmap(myImage.
                                scaled(ui.scrollArea->width()-15,
                                       ui.scrollArea->height()-15,
                                       Qt::KeepAspectRatio));
        }
        else
            ui.label->setPixmap(myImage);
    }
    else
        ui.label->setPixmap(myImage);
}

void GS_albumer_viewer::updateSaveButtons()
{
    if(historyIndex > 0)
    {
        ui.saveAsButton->setEnabled(true);
        ui.saveButton->setEnabled(true);
    }
    else
    {
        ui.saveAsButton->setEnabled(false);
        ui.saveButton->setEnabled(false);
    }

}

void GS_albumer_viewer::createActions()
{
    exportAction = new QAction(tr("&Export"),this);
    connect(exportAction,SIGNAL(triggered()),this,SLOT(exportImage()));

    setFitAction = new QAction(tr("&Fit to view"),this);
    connect(setFitAction,SIGNAL(triggered()),this,SLOT(setFit()));

    disetFitAction = new QAction(tr("Full &size"),this);
    connect(disetFitAction,SIGNAL(triggered()),this,SLOT(disetFit()));
}

void GS_albumer_viewer::createMenus()
{
    myMenu = new QMenu(this);
    myMenu->addAction(setFitAction);
    myMenu->addAction(disetFitAction);
    myMenu->addSeparator();
    myMenu->addAction(exportAction);
}

GS_albumer_viewer::GS_albumer_viewer(ColectionManipulator* man, QWidget* parent)
    :GS_albumer_view(parent)
{
    drag = false;
    fit = GS_config_holder::getViewFit();
    historyIndex = 0;
    createActions();
    createMenus();
    setFitAction->setEnabled(!fit);
    disetFitAction->setEnabled(fit);
    currentFilter = NULL;
    ui.setupUi(this);
    static int instanceCount = 0;
    instanceCount++;
    setIndex(instanceCount);
    if(fit)
    {
        ui.scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        ui.scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    }
    else
    {
        ui.scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        ui.scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    }
    ui.fitButton->setEnabled(!fit);
    ui.orginalButton->setEnabled(fit);
    ui.editOptions->setFolded(GS_config_holder::getViewCollEdit());
    ui.fileOptions->setFolded(GS_config_holder::getViewCollFile());
    ui.filterList->addItems(PluginManager::getInstance()->getImageFilterNames());
    ui.filterList->setCurrentIndex(-1);
    myManipulator = man;
    setAttribute(Qt::WA_DeleteOnClose);
    myProgress = new GS_albumer_progress(this);
    connect(ui.fitButton,SIGNAL(clicked()),this,SLOT(setFit()));
    connect(ui.orginalButton,SIGNAL(clicked()),this,SLOT(disetFit()));
    connect(ui.cClockWiseButton,SIGNAL(clicked()),this,SLOT(rotateCClockWise()));
    connect(ui.clockWiseButton,SIGNAL(clicked()),this,SLOT(rotateClockWise()));
    connect(ui.hMirrorButton,SIGNAL(clicked()),this,SLOT(hMirror()));
    connect(ui.vMirrorButton,SIGNAL(clicked()),this,SLOT(vMirror()));
    connect(ui.exportButton,SIGNAL(clicked()),this,SLOT(exportImage()));
    connect(ui.reDoButton,SIGNAL(clicked()),this,SLOT(historyForward()));
    connect(ui.unDoButton,SIGNAL(clicked()),this,SLOT(historyBack()));
    connect(ui.saveAsButton,SIGNAL(clicked()),this,SLOT(saveAsSlot()));
    connect(ui.saveButton,SIGNAL(clicked()),this,SLOT(saveSlot()));
    connect(ui.filterList,SIGNAL(currentIndexChanged(int)),this,SLOT(applyFilter(int)));
}
