#include "MirrorViewButtonItem.h"

QRectF MirrorViewButtonItem::boundingRect() const
{
    return QRectF(0.0f,0.0f,static_cast<float>(myWidth),
                  static_cast<float>(myHeight));
}

void MirrorViewButtonItem::paint(QPainter* painter,const QStyleOptionGraphicsItem* option,
                                 QWidget* widget)
{
    Q_UNUSED(option);
    Q_UNUSED(widget);
    painter->fillRect(0,0,myWidth,myHeight,Qt::transparent);
    painter->save();

    QColor penColor = touched ? QColor(120,220,240,90) : QColor(255,240,240,90);
    QColor brushColor = touched ? QColor(42,65,85,50) : QColor(100,85,85,50);

    painter->setPen(QPen(QBrush(penColor),2,
                         Qt::SolidLine,Qt::FlatCap,Qt::MiterJoin));
    painter->setBrush(QBrush(brushColor));

    painter->drawRect(0,0,myWidth,myHeight);
    painter->setFont(myFont);
    painter->drawText(0.0f,0.0f,myWidth,myHeight,Qt::AlignHCenter|Qt::AlignVCenter,
                      myText);
    painter->restore();
}

void MirrorViewButtonItem::hoverEnterEvent(QGraphicsSceneHoverEvent*)
{
    touched = true;
    update();
}

void MirrorViewButtonItem::hoverLeaveEvent(QGraphicsSceneHoverEvent*)
{
    touched = false;
    update();
}

void MirrorViewButtonItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    if(event->button() == Qt::LeftButton)
        emit clicked();
}

int MirrorViewButtonItem::getHeight() const
{
    return myHeight;
}

int MirrorViewButtonItem::getWidth() const
{
    return myWidth;
}

MirrorViewButtonItem::MirrorViewButtonItem(QFont font,QString buttonText)
        :QGraphicsObject()
{
    myFont = font;
    myFont.setPointSize(12);
    myFont.setBold(true);
    QFontMetrics metrics(myFont);
    QRect fontRect = metrics.boundingRect(buttonText);
    setZValue(9999);
    touched = false;
    myWidth = fontRect.width() + 20;
    myHeight = fontRect.height() + 20;
    myText = buttonText;
    this->setCursor(QCursor(Qt::PointingHandCursor));
    this->setAcceptHoverEvents(true);
}
