#include "MyBarWidget.h"
#include <QPainter>
#include <QLinearGradient>
#include <QFontMetrics>
#include <QString>
#include <QStyle>
#include <QStyleOptionButton>

void MyBarWidget::paintEvent(QPaintEvent*)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);
    int alfa = hover ? 0 : 24;
    QColor color(128,128,128,alfa);
    painter.fillRect(this->rect(),color);
    QLinearGradient grad(0.0f,0.0f,0.0f,rect().height());
    grad.setColorAt(0.0f,QColor(128,128,128,128));
    grad.setColorAt(1.0f,QColor(255,255,255,255));

    if(leftB)
        painter.fillRect(QRectF(rect().topLeft(),QPointF(1.0f,rect().bottom())),
                         grad);
    if(rightB)
        painter.fillRect(QRectF(QPointF(static_cast<float>(rect().width())
                                        -0.5f,rect().top()),
                                rect().bottomRight()),grad);
    //text drowing
    QStyle* myStyle = myFoldable ? style() : 0;
    int decoW = myStyle ?
                myStyle->pixelMetric(QStyle::PM_IndicatorWidth, 0, 0) : 0;
    int decoH = myStyle ?
                myStyle->pixelMetric(QStyle::PM_IndicatorHeight, 0, 0) : 0;
    int dx = myFoldable ? 5 : 0;

    QFontMetrics metrics(font);
    QRect fontRect = metrics.boundingRect(myTitle);
    painter.setFont(QFont());
    painter.save(); 
    if(fold)
    {
        float x1 = rect().left();
        float x2 = rect().right() - fontRect.height();
        painter.translate(x2 - x1,0);
        painter.rotate(90);
    }
    else
        painter.translate((width() - fontRect.width() - decoW) / 2 - dx,
                          -fontRect.top() + 4);

    int dText = 0;
    if(myFoldable)
    {
        QStyleOptionButton opt;
        opt.state = QStyle::State_Enabled;
        int styleOffset = (fontRect.height() - decoH) / 2; //align text and indycator
        opt.rect = QRect(0, -decoH + 2 - styleOffset, decoW, decoH);
        opt.state |= fold ? QStyle::State_Off : QStyle::State_On;
        dText = decoW + 4;
        if (underMouse())
            opt.state |= QStyle::State_MouseOver;
        style()->drawPrimitive(QStyle::PE_IndicatorRadioButton, &opt, &painter, this);
    }
    painter.drawText(dText,0,myTitle);
    painter.restore();
}

void MyBarWidget::mousePressEvent(QMouseEvent*)
{
    if(myFoldable && !animating)
    {
        fold = !fold;
        updateFold();
    }
}

void MyBarWidget::enterEvent(QEvent*)
{
    hover = true;
    update(rect());
}

void MyBarWidget::leaveEvent(QEvent*)
{
    hover = false;
    update(rect());
}

QString MyBarWidget::title() const
{
    return myTitle;
}

void MyBarWidget::setTitle(const QString& title)
{
    myTitle = title;
    updateMinimumHeight();
}

bool MyBarWidget::isFoldable() const
{
    return myFoldable;
}

void MyBarWidget::setFoldable(bool foldable)
{
    myFoldable = foldable;
    updateMinimumHeight();
    if(!foldable && isHidden())
        show();
}

bool MyBarWidget::isFolded() const
{
    return fold;
}

void MyBarWidget::setFolded(bool folded)
{
    if(myFoldable)
    {
        fold = folded;
        updateFold();
    }
}

int MyBarWidget::minimumWidth()
{
    return qMax(QFontMetrics(font).width(myTitle) + 12,QWidget::sizeHint().width());
}

bool MyBarWidget::leftBorder() const
{
    return leftB;
}

void MyBarWidget::setLeftBorder(bool left)
{
    leftB = left;
    update();
}

bool MyBarWidget::rightBorder() const
{
    return rightB;
}

void MyBarWidget::setRightBorder(bool right)
{
    rightB = right;
    update();
}

void MyBarWidget::updateFold()
{
    if(fold)
    {
        QObjectList childrenList = this->children();
        for(int i = 0; i < childrenList.count(); ++i)
        {
            QWidget* child = qobject_cast<QWidget*>(childrenList.at(i));
            if(child)
                child->hide();
        }
        QPropertyAnimation *animation = new QPropertyAnimation(this, "widgetSize");
        animation->setDuration(250);
        animation->setEndValue(font.pixelSize() + 8);
        animation->setEasingCurve(QEasingCurve::OutBack);
        connect(animation,SIGNAL(finished()),this,SLOT(finishAnimation()));
        animation->start(QAbstractAnimation::DeleteWhenStopped);
    }
    else
    {
        QObjectList childrenList = this->children();
        for(int i = 0; i < childrenList.count(); ++i)
        {
            QWidget* child = qobject_cast<QWidget*>(childrenList.at(i));
            if(child)
                child->show();
        }
        setContentsMargins(0,font.pixelSize() + 8, 0, 0);
        QPropertyAnimation *animation = new QPropertyAnimation(this, "widgetSize");
        animation->setDuration(500);
        animation->setEndValue(minimumWidth());
        animation->setEasingCurve(QEasingCurve::OutBack);
        connect(animation,SIGNAL(finished()),this,SLOT(finishAnimation()));
        animation->start(QAbstractAnimation::DeleteWhenStopped);
    }
    animating = true;
}

int MyBarWidget::getWidgetSize() const
{
    return animationWidth;
}

void MyBarWidget::setWidgetSize(int size)
{
    animationWidth = size;
    if(size < 0)
        size = 0;
    setMinimumWidth(size);
    setMaximumWidth(size);
}

 void MyBarWidget::finishAnimation()
 {
     if(fold)
     {
         setMaximumHeight(maxHeight);
         setContentsMargins(font.pixelSize() + 8, 0, 0, 0);
         setCursor(Qt::PointingHandCursor);
     }
     else
     {
         setMaximumHeight(maxHeight);
         setCursor(Qt::ArrowCursor);
     }
     animating = false;
     if(this->isHidden()) show();
 }

void MyBarWidget::updateMinimumHeight()
{
    if(myFoldable)
    {
        int dW = style()->pixelMetric(QStyle::PM_IndicatorWidth, 0, 0) + 4;
        font.setPixelSize(10);
        QFontMetrics metrics(font);
        QRect fontRect = metrics.boundingRect(myTitle);
        setMinimumHeight(fontRect.width() + dW + 12);
    }
    else
        setMinimumHeight(10);
}

MyBarWidget::MyBarWidget(QWidget * parent)
        :QWidget(parent),myTitle(""),hover(false),fold(false),myFoldable(true),
        leftB(false),rightB(false),animating(false),font(QFont())
{
    font.setPixelSize(10);
    setContentsMargins(0,font.pixelSize() + 8, 0, 0);
    setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
    maxHeight = maximumHeight();
    animationWidth = width();
    hide();//hide because it might be dirti at first
}
