#ifndef GS_ALBUMER_FOLDER_VIEW2_H
#define GS_ALBUMER_FOLDER_VIEW2_H

#include <QGraphicsScene>
#include <QMap>
#include <QResizeEvent>
#include <QMenu>
#include <QScrollBar>
#include <QHideEvent>
#include <QShowEvent>
#include <QList>
#include <QTimer>
#include "Colection/colectionManipulator.h"
#include "GS_albumer_view.h"
#include "ui_gs_albumer_folder_view2.h"
#include "gs_albumer_view_item2.h"
#include "gs_albumer_deco_line2.h"
#include "gs_view_info_item.h"

class GS_albumer_folder_view2 : public GS_albumer_view
{
    Q_OBJECT
public:
    explicit GS_albumer_folder_view2(ColectionManipulator* manipulator
                                     ,QWidget *parent = 0);
    ~GS_albumer_folder_view2()
    {
        itemMap.clear();
        scene->removeItem(infoItem);
        delete scene;
        delete infoItem;
    }

    void setFolderImages(QList<int> indexList,QStringList titleList,
                         QStringList dateList,QString title,QString catalog = "");
    void resetFolderImages(QList<int> indexList);
    void addItem(int idex);
    bool wasItMe();
    bool isItMe(QString title_) const;

signals:
    void imageClicked(int);
    void imageMovePicked(int,int);
    void imageRemovePicked(int,int);
    void imageExportPicked(int);
    void imageEditDscPicked(int);
    void removeTagPicked(int);
    void groupMovePicked(QList<int>,int);
    void groupRemovePicked(QList<int>,int);
    void groupExport(QList<int>);
    void imageInfoShowPicked(int);
    void startSlideShow(QList<int>);
    void startMirror(QList<int>);
    void addTags(QList<int>);
    void renameImages(QList<int>);

public slots:
    void updateTheme() const;
    void removeItem(int,bool);
    void unMarkItems();
    void updateMiniature(int index);
    void loadNames(QList<int> itemList);
    void closeView();

protected slots:
    void itemClicked(int itemIndex);
    void itemRightClicked(int itemIndex,QPoint pos);
    void updateMiniature();
    void markAll();
    void infoStateChange();
    void itemMarked(int index);
    void itemUnMarked(int index);
    void itemTouched(int index);
    void itemUntouched(int index);
    void moveImage();
    void removeImage();
    void exportImage();
    void editDsc();
    void removeTag();
    void removeImages();
    void moveImages();
    void exportImages();
    void editTags();
    void nameImages();
    void showImageInfo();
    void slideShow();
    void mirrorView();

protected:
    void resizeEvent(QResizeEvent* event);
    void hideEvent(QHideEvent* event);
    void showEvent(QShowEvent* event);
    void keyPressEvent(QKeyEvent* event);
    void closeEvent(QCloseEvent *event);

private:
    void updateScene();
    void createActions();
    void createMenus();
    int getWItemCount();
    int getPhotoLineCount();
    int rightClickedItemIndex;
    Ui::GS_albumer_folder_view2_ui ui;
    QGraphicsScene * scene;
    QMenu* myItemMenu;
    QAction* movePhoto;
    QAction* removePhoto;
    QAction* exportPhoto;
    QAction* editPhotoDsc;
    QAction* removePhotoTag;
    QAction* showPhotoInfo;
    QTimer* miniTimer;
    QMap <int,GS_albumer_view_item2*> itemMap;
    QList<GS_albumer_view_item2*> noPhotoItems;
    GS_view_info_item* infoItem;
    QString title;
    QList<int> markedItems;
    bool first,sentRequest,showInfo;
    GS_albumer_deco_line2* decoLine;
    int sliderValue;
    ColectionManipulator* myManipulator;
    QTimer* closeTimer;
};

#endif // GS_ALBUMER_FOLDER_VIEW2_H
