#include "gs_albumer_slideshow.h"

void GS_albumer_slideShow::keyPressEvent(QKeyEvent* event)
{
    if(event->key() == Qt::Key_Escape)
        hideClicked();
    if(event->key() == Qt::Key_Right && !show)
        nextPhoto();
    if(event->key() == Qt::Key_Left && !show)
        prevPhoto();
}

void GS_albumer_slideShow::hideClicked()
{
    stopShow();
    hideTools(false);
    setCursor(QCursor(Qt::ArrowCursor));
    right = true;
    hide();
}

void GS_albumer_slideShow::hideEvent(QHideEvent*)
{
    this->releaseKeyboard();
}

void GS_albumer_slideShow::showEvent(QShowEvent*)
{
        this->grabKeyboard();
}

void GS_albumer_slideShow::resizeEvent(QResizeEvent*)
{
    QGraphicsScene* temp = scene;
    scene = new QGraphicsScene(0,0,ui.slideView->width(),ui.slideView->height());
    if(image.width() > ui.slideView->width())
        image = image.scaled(ui.slideView->size(),Qt::KeepAspectRatio,
                             Qt::SmoothTransformation);
    if(image.height() > ui.slideView->height())
        image = image.scaled(ui.slideView->size(),Qt::KeepAspectRatio,
                             Qt::SmoothTransformation);
    item->setPixmap(image);
    if(image.width() < ui.slideView->width() ||
       image.height() < ui.slideView->height())
    {
        qreal x = 0;
        qreal y = 0;
        x = ((ui.slideView->width()-image.width())/2);
        y = ((ui.slideView->height()-image.height())/2);
        item->setPos(x,y);
    }
    else
        item->setPos(0,0);
    scene->addItem(item);
    ui.slideView->setScene(scene);
    delete temp;
}

void GS_albumer_slideShow::startShow()
{
    ui.stopButton->setEnabled(true);
    ui.playButton->setEnabled(false);
    ui.leftButton->setEnabled(false);
    ui.rightButton->setEnabled(false);
    show = true;
    right = true;
    scaleImage();
    hideTools(true);
    setCursor(QCursor(QPixmap(":/eyecandy/cur.png"),3,3));
}

void GS_albumer_slideShow::stopShow()
{
    ui.stopButton->setEnabled(false);
    ui.playButton->setEnabled(true);
    ui.leftButton->setEnabled(true);
    ui.rightButton->setEnabled(true);
    show = false;
    showTimer->stop();
    hideTools(false);
    setCursor(QCursor(Qt::ArrowCursor));
}

void GS_albumer_slideShow::nextPhoto()
{
    ++currentIndex;
    right = true;
    if(currentIndex >= photoList.count())
        currentIndex = 0;
    emit giveMeThisPhoto(photoList.value(currentIndex));
}

void GS_albumer_slideShow::prevPhoto()
{
    --currentIndex;
    right = false;
    if(currentIndex == -1)
        currentIndex = photoList.count()-1;
    emit giveMeThisPhoto(photoList.value(currentIndex));
}

void GS_albumer_slideShow::setPhoto(QPixmap photo)
{
    image = photo;
    if(photo.width() > ui.slideView->width())
        photo = photo.scaled(ui.slideView->size(),Qt::KeepAspectRatio,
                             Qt::SmoothTransformation);
    if(photo.height() > ui.slideView->height())
        photo = photo.scaled(ui.slideView->size(),Qt::KeepAspectRatio,
                             Qt::SmoothTransformation);
    if(photo.width() < ui.slideView->width() ||
       photo.height() < ui.slideView->height())
    {
        wPosX = ((ui.slideView->width()-photo.width())/2);
        wPosY = ((ui.slideView->height()-photo.height())/2);
    }
    else
    {
        wPosX = 0.0f;
        wPosY = 0.0f;
    }
    stepX = photo.width()/6;
    if(stepX < 10.0f)
        stepX = 20.0f;
    else if(stepX > 50.0f)
        stepX = 45.0f;
    if(right)
        item->setPos(-photo.width(),wPosY);
    else
        item->setPos(ui.slideView->width(),wPosY);
    showTimer->stop();
    animationTimer->start(30);
    item->setPixmap(photo);
}

void GS_albumer_slideShow::setPhotoList(QList<int> photoList_)
{
    currentIndex = 0;
    photoList = photoList_;
}

void GS_albumer_slideShow::scaleImage()
{
    if(right)
        item->setPos(item->pos().x()+stepX,wPosY);
    else
        item->setPos(item->pos().x()-stepX,wPosY);
    if((item->pos().x() < wPosX && right) || (item->pos().x() > wPosX && !right))
    {
        animationTimer->start(30);
    }
    else
    {
        item->setPos(wPosX,wPosY);
        if(show)
            showTimer->start(5000);
    }

}

void GS_albumer_slideShow::mouseMoveEvent (QMouseEvent*)
{
    hideTools(false);
    setCursor(QCursor(Qt::ArrowCursor));
}

void GS_albumer_slideShow::hideTools(bool hide)
{
    if(hide)
    {
        ui.playButton->hide();
        ui.stopButton->hide();
        ui.leftButton->hide();
        ui.rightButton->hide();
        ui.closeButton->hide();
    }
    else
    {
        ui.playButton->show();
        ui.stopButton->show();
        ui.leftButton->show();
        ui.rightButton->show();
        ui.closeButton->show();
    }
}

GS_albumer_slideShow::GS_albumer_slideShow(QWidget *parent)
        :QWidget(parent)
{
    right = true;
    ui.setupUi(this);
    ui.stopButton->setEnabled(false);
    showTimer = new QTimer(this);
    currentIndex = 0;
    animationTimer = new QTimer(this);
    animationTimer->setSingleShot(true);
    scene = new QGraphicsScene(0,0,ui.slideView->width(),ui.slideView->height());
    ui.slideView->setScene(scene);
    ui.slideView->setFocusPolicy(Qt::NoFocus);
    item = new QGraphicsPixmapItem(QPixmap());
    scene->addItem(item);
    wPosX = 0.0f;
    wPosY = 0.0f;
    show = false;
    connect(ui.playButton,SIGNAL(clicked()),this,SLOT(startShow()));
    connect(ui.stopButton,SIGNAL(clicked()),this,SLOT(stopShow()));
    connect(animationTimer,SIGNAL(timeout()),this,SLOT(scaleImage()));
    connect(showTimer,SIGNAL(timeout()),this,SLOT(nextPhoto()));
    connect(ui.rightButton,SIGNAL(clicked()),this,SLOT(nextPhoto()));
    connect(ui.leftButton,SIGNAL(clicked()),this,SLOT(prevPhoto()));
    connect(ui.closeButton,SIGNAL(clicked()),this,SLOT(hideClicked()));
    setMouseTracking(true);
}
